/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.communication;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketTimeoutException;
import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetAgentInterface;
import de.fhg.iais.outdoor.simba.value.NormalizedDoubleValue;

public class TCPSchedulerConnection { 
	
	public static final int FAILURE = -1;
	public static final int COMMAND_COMPLETE = 1;
	public static final int COMMAND_PREPARE_GENERATION = 2;
	public static final int COMMAND_PREPARE_INDIVIDUAL = 3;
	public static final int COMMAND_PREPARE_TRY = 4;
	public static final int COMMAND_EXECUTE_STEP = 5;
	public static final int COMMAND_TERMINATE = 6;
	public static final int COMMAND_RESET_SYSTEM = 7;
	public static final int COMMAND_GET_SYSTEM_INFO = 8;
	
	private NeuralNetAgentInterface mNeuralNetController;
	private ServerSocket mServerSocket;
	private Socket mSocket;
	private boolean mShutDown;
	private DataInputStream  mInput;
	private DataOutputStream  mOutput;
	
	public TCPSchedulerConnection(NeuralNetAgentInterface controller) {
		mNeuralNetController = controller;
		mShutDown = false;
	}
	
	public void startServer(int port) {
		
		while(!mShutDown) {
			while(mSocket == null && !mShutDown) {
				try {
					if(mServerSocket == null) {
						mServerSocket = new ServerSocket(port);
					}
					System.out.println("Server Creatd at port: " + mServerSocket.getLocalPort() + " / " + mServerSocket.getLocalSocketAddress());
					//mServerSocket.setSoTimeout(1000);
					mSocket = mServerSocket.accept();
					System.out.println("Connection established.");
				}
				catch(SocketTimeoutException e) {
				}
				catch(IOException e) {
					try {
						mServerSocket.close();
					}
					catch(IOException ioe) {
					}
					return;
				}
			}
			if(mSocket == null) {
				return;
			}
			
			try {
				mInput = new DataInputStream(mSocket.getInputStream());
				mOutput = new DataOutputStream(mSocket.getOutputStream());
			}
			catch(IOException e) {
				
			}
			
			boolean connected = true;
			while(!mShutDown && connected) {		
				try {
					int command = mInput.readInt();
					switch(command) {
						case COMMAND_EXECUTE_STEP:
							int stepNumber = mInput.readInt();
							//TODO read network output
						
							Vector outputNeurons = mNeuralNetController.getOutputNeurons();
							for(int i = 0; i < outputNeurons.size(); ++i) {
								((NormalizedDoubleValue) outputNeurons.elementAt(i)).setNormalized(mInput.readDouble());
							}
							Simulator.getInstance().getScheduler().executeSimulationStep(stepNumber);
							Vector inputNeurons = mNeuralNetController.getInputNeurons();
							for(int i = 0; i < inputNeurons.size(); ++i) {
								mOutput.writeDouble(((NormalizedDoubleValue) inputNeurons.elementAt(i)).getNormalized());
							}

							mOutput.writeInt(COMMAND_COMPLETE);
							mOutput.flush();
							break;
						case COMMAND_PREPARE_GENERATION:
							int generation = mInput.readInt();
							int numberOfTries = mInput.readInt();
							Simulator.getInstance().getScheduler().prepareNextGeneration(generation, numberOfTries, 100); //TODO 100 exchange with variable read from the stream
							mOutput.writeInt(COMMAND_COMPLETE);
							mOutput.flush();
							break;
						case COMMAND_PREPARE_INDIVIDUAL:
							int individualNumber = mInput.readInt();
							Simulator.getInstance().getScheduler().prepareNextIndividual(individualNumber);
							mOutput.writeInt(COMMAND_COMPLETE);
							mOutput.flush();
							break;
						case COMMAND_PREPARE_TRY:
							int tryNumber = mInput.readInt();
							Simulator.getInstance().getScheduler().prepareNextTry(tryNumber);
							mOutput.writeInt(COMMAND_COMPLETE);
							mOutput.flush();
							break;
						case COMMAND_RESET_SYSTEM:
							Simulator.getInstance().clearInstance();
							mOutput.writeInt(COMMAND_COMPLETE);
							mOutput.flush();
							break;
						case COMMAND_TERMINATE:
							Simulator.getInstance().getScheduler().terminateSimulationRun();
							mOutput.writeInt(COMMAND_COMPLETE);
							mOutput.flush();
							break;
						case COMMAND_GET_SYSTEM_INFO:
							int numberOfInputNeurons = 0;
							int numberOfOutputNeurons = 0;
							if(mNeuralNetController != null) {
								numberOfInputNeurons = mNeuralNetController.getInputNeurons().size();
								numberOfOutputNeurons = mNeuralNetController.getOutputNeurons().size();
							}
							mOutput.writeInt(numberOfInputNeurons);
							mOutput.writeInt(numberOfOutputNeurons);
							mOutput.writeInt(COMMAND_COMPLETE);
							mOutput.flush();
							break;
						default:
							System.out.println("Failure: Unexpected command in TCPSchedulerConnection.");
					}
				}
				catch(IOException e) {
					System.out.println("Client disconnected.");
					Simulator.getInstance().getScheduler().terminateSimulationRun();
					connected = false;
					mSocket = null;
				}
		
			}
			
		}
	
	}
	
	public void stopServer() {
		mShutDown = true;
	}
	
	
	

}
