/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.core;

import java.awt.geom.Line2D;
import java.awt.geom.Point2D;

public class GeometricMath {

	
	/**
	 * Calculate the intersection point for a line and the scanner.
	 * If there is no intersection point, then false is returned. If an intersection
	 * point was found, then this method returns true and the result is written to 
	 * parameter intersection.
	 * 
	 * @param line the line to intersect with.
	 * @param intersection the buffer that will contain the resulting intersection point.
	 * @return
	 */
	public static boolean getIntersectionPoint(Line2D.Double line, Line2D.Double line2, 
												Point2D.Double intersection) 
	{
		
		if (!line2.intersectsLine(line)) {
			return false;
		}
		
		
		double det1 = det(line2.x1, line2.y1, line2.x2, line2.y2);
		double det2 = det(line.x1, line.y1, line.x2, line.y2);
		double det3 = det(line2.x1 - line2.x2, line2.y1 - line2.y2, line.x1 - line.x2, line.y1 - line.y2);
		
		if(det3 != 0.0) {
			intersection.x = det(det1, line2.x1 - line2.x2,	det2, line.x1 - line.x2) / det3;
				
			intersection.y = det(det1, line2.y1 - line2.y2, det2, line.y1 - line.y2) / det3;
		}
		else {
			intersection.setLocation(0.0, 0.0);
		}
		

		return true;
	}
	
	/**
	 * Calculates the determinant of a 2-D matrix given by four values.
	 * @param a
	 * @param b
	 * @param c
	 * @param d
	 * @return the determinant.
	 */
	public static double det(double a, double b, double c, double d) {
		return a * d - b * c;
	}
	
}
