/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.core;

import java.awt.Polygon;
import java.util.Random;

public class Sensor extends SimObject {
	
	protected Random mRandom;
	protected double mVariance;

	public Sensor() {
		super();
		mRandom = new Random();
		mVariance = 0.0;
	}
	
	public Sensor(String name, double x, double y, double angle) {
		super(name, x, y, angle, new Polygon());
		mRandom = new Random();
		mVariance = 0.0;
	}
	
	public Sensor(String name, double x, double y, double angle, Polygon shape) {
		super(name, x, y, angle, shape);
		mRandom = new Random();
		mVariance = 0.0;
	}
	
	public double getSensorReading() {
		return 0.0;
	}
	
	public void setNoiseVariance(double variance) {
		mVariance = variance;
	}
	
	public double getNoiseVariance() {
		return mVariance;
	}
	
	
	/**
	 * Adds noise to a value.
	 * 
	 * @param value the value to modify.
	 * @return the value modified by a noise.
	 */
	public double addNoise(double value) {
		
		return value + ((mRandom.nextGaussian() * mVariance));
	}
	
}
