/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.core;

import java.util.Iterator;
import java.util.Vector;

public class SimObjectSet {
	
	private Vector mObjects = new Vector();
	
	public SimObjectSet() {
	}
	
	public SimObjectSet(Vector elements) {
		mObjects.addAll(elements);
	}
	
	public boolean add(SimObject object) {
		if(object == null || mObjects.contains(object)) {
			return false;
		}
		mObjects.add(object);
		return true;
	}
	
	public boolean remove(SimObject object) {
		if(object == null || !mObjects.contains(object)) {
			return false;
		}
		mObjects.remove(object);
		return true;
	}
	
	public Iterator getIterator() {
		return mObjects.iterator();
	}
	
	public int getSize() {
		return mObjects.size();
	}
	
	/**
	 * This method returns a COPY of the internal Vector, so this method is relatively 
	 * expensive to use. If you want to access its members it is better to use the
	 * getIterator method and iterate with that Iterator over the elements.
	 * @return
	 */
	public Vector getElements() {
		return new Vector(mObjects);
	}

}
