/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.devices;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.geom.NoninvertibleTransformException;
import java.awt.geom.Point2D;
import java.util.Iterator;
import java.util.logging.Logger;

import de.fhg.iais.outdoor.simba.core.SimObject;
import de.fhg.iais.outdoor.simba.core.SimObjectSet;

public class DistanceToPointScanner extends DistanceSensor {
	
	protected Point2D.Double mNearestObject;

	public DistanceToPointScanner(String name, double x, double y, double angle, 
			double minScanDistance, double maxScanDistance) 
	{
		super(name, x, y, angle, minScanDistance, maxScanDistance);
	}

	/**
	* Calculates the shortest distance to the center point of the closest obstacle.
	*/
	public synchronized boolean update() {
		boolean ok = super.update();
		if(!isActive()) {
			if(getSensorReading() != mMaxScanDistance.get()) {
				mDistance.set(mMaxScanDistance.get());
			}
			return ok;
		}
		
		double distance = mMaxScanDistance.get();
		mNearestObject = null;
		
		Point2D.Double globalPos = getGlobalPosition();
		
		SimObjectSet obstacles = mObstacles;
		int setIndex = 0;

		do {
			for(Iterator i = obstacles.getIterator(); i.hasNext();) {
				SimObject obs = (SimObject) i.next();
				if(!obs.isActive() || mIgnorableObjects.contains(obs)) {
					continue;
				}
				
				
				Point2D.Double objPosition = obs.getGlobalPosition();
				double dist = objPosition.distance(globalPos);
				
				if(dist <= mMaxScanDistance.get()) 
				{
					if(dist < distance) {
						if(dist < mMinScanDistance.get()) {
							distance = mMinScanDistance.get();
							break;
						}
						else {
							distance = dist;
						}
						mNearestObject = objPosition;
					}
				}
			}
			if(setIndex < mObstacleLists.size()) {
				obstacles = (SimObjectSet) mObstacleLists.elementAt(setIndex);
				setIndex++;
			}
			else {
				obstacles =  null;
			}
		}
		while(obstacles != null);
		
		double dist = Math.max(Math.min(addNoise(distance), mMaxScanDistance.get()), mMinScanDistance.get());

		mDistance.set(Math.max(0.0, dist)); //do not allow distances less than zero.

		return ok;
	}
	
	public synchronized void paintSelf(Graphics2D g) {
		if(mNearestObject == null) {
			return;
		}
		Color c = g.getColor();
		if(mDistance.get() >= 0.0 && mDistance.get() <= mMaxScanDistance.get()) {
			if(mObstacleDetectedColor == null) {
				g.setColor(mShapeColor);
			}
			else {
				g.setColor(mObstacleDetectedColor);
			}
		}
		else {
			g.setColor(mShapeColor);
		}
		AffineTransform transformation = getAffineTransform();
		try {
			transformation = transformation.createInverse();
		}
		catch(NoninvertibleTransformException ex) {
			Logger.getLogger("Simulator")
				.warning("DistanceToPointScanner.paintSelf: Could not invert AffineTransform.");
			g.setColor(c);
			return;
		}
		Point2D.Double object = new Point2D.Double();
		transformation.transform(mNearestObject, object);
		g.drawLine(0, 0, (int) Math.round(object.x), (int) Math.round(object.y));
		g.setColor(c);
	}
	
}
