/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.devices;

import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.Sensor;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.DoubleValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueManager;
import de.fhg.iais.outdoor.simba.value.ValueReplacedListener;

public class LogicOrDevice extends Sensor implements ValueReplacedListener {
	
	
	Vector mBooleanValues = new Vector();
	BooleanValue mSensorValue;
	DoubleValue mSensorValueDouble;
	
	public LogicOrDevice(String name, double x, double y, double angle) {
		super(name, x, y, angle);
		mSensorValue = new BooleanValue(false);
		mSensorValueDouble = new DoubleValue(0.0);
		
		ValueManager vm = Simulator.getInstance().getValueManager();
		vm.addValue("/SimObjects/" + getName() + "/LogicOr", mSensorValue);
		vm.addValue("/SimObjects/" + getName() + "/LogicOrDouble", mSensorValueDouble);
		
		mSensorValue.addValueReplacedListener(this);
		mSensorValueDouble.addValueReplacedListener(this);
	}
	
	/**
	 * This sensor has a sensor value of true (boolean) or 1.0 (double)
	 * if at least one of the registered BooleanValues is true.
	 */
	public boolean update() {
		boolean ok = super.update();
		
		boolean valueTrue = false;
		for(int i = 0; i < mBooleanValues.size(); ++i) {
			if(((BooleanValue) mBooleanValues.elementAt(i)).get()) {
				valueTrue = true;
				break;
			}
		}
		mSensorValue.set(valueTrue);
		if(valueTrue) {
			mSensorValueDouble.set(1.0);
		}
		else {
			mSensorValueDouble.set(0.0);
		}
		return ok;
	}
	
	public boolean addBooleanValue(BooleanValue value) {
		if(value == null || mBooleanValues.contains(value)) {
			return false;
		}
		mBooleanValues.add(value);
		value.addValueReplacedListener(this);
		return true;
	}
	
	public boolean removeBooleanValue(BooleanValue value) {
		if(value == null || !mBooleanValues.contains(value)) {
			return false;
		}
		mBooleanValues.remove(value);
		value.removeValueReplacedListener(this);
		return true;
	}
	
	
	/**
	 * Called whenever an observed Value object is replaced by another one in the 
	 * ValueManager. The object can then change its observatoin to the new Value object
	 * and cancel the observation of the old Value. When overriding this method
	 * in subclassed do not forget to call this method from the subclass.
	 */
	public void valueReplaced(Value oldValue, Value newValue) {
		super.valueReplaced(oldValue, newValue);
		
		mSensorValue = (BooleanValue) synchronizeValue(mSensorValue, oldValue, newValue);
		mSensorValueDouble = (DoubleValue) synchronizeValue(mSensorValueDouble, oldValue, newValue);
		
		if(mBooleanValues.contains(oldValue)) {
			mBooleanValues.remove(oldValue);
			oldValue.removeValueReplacedListener(this);
			mBooleanValues.add(newValue);
			newValue.addValueReplacedListener(this);
		}
	}
	

}
