/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.devices;

import java.awt.geom.AffineTransform;
import java.awt.geom.NoninvertibleTransformException;
import java.awt.geom.Point2D;
import java.util.Iterator;

import de.fhg.iais.outdoor.simba.core.SimObject;
import de.fhg.iais.outdoor.simba.core.SimObjectSet;

public class OrthogonalDistanceScanner extends DistanceSensor {
	
	
	public OrthogonalDistanceScanner(String name, double x, double y, double angle, 
					double minScanDistance, double maxScanDistance) 
	{
		super(name, x, y, angle, minScanDistance, maxScanDistance);
	}
	
	/**
	 * Calculates the orthogonal distance to the center point of the closest obstacle.
	 * 
	 */
	public boolean update() {
		boolean ok = super.update();
	
		double maxScanDistance = mMaxScanDistance.get();
		double minScanDistance = mMinScanDistance.get();
		
		double distance = maxScanDistance;
		
		AffineTransform transformation = new AffineTransform();
		Point2D.Double globalPos = getGlobalPosition();
		transformation.translate(globalPos.x, globalPos.y);
		transformation.rotate(Math.toRadians(getGlobalAngle()));
		try {
			transformation = transformation.createInverse();
		}
		catch(NoninvertibleTransformException ex) {
		}
		
		
		SimObjectSet obstacles = mObstacles;
		int setIndex = 0;
		do {
			for(Iterator i = obstacles.getIterator(); i.hasNext();) {
				SimObject obs = (SimObject) i.next();
				if(!obs.isActive() || mIgnorableObjects.contains(obs)) {
					continue;
				}
				
				//Get the object center point in scanner coordinates.
				Point2D.Double objPosition = obs.getGlobalPosition();
				Point2D.Double objPositionTransformed = new Point2D.Double();
				transformation.transform(objPosition, objPositionTransformed);
				if(objPositionTransformed.x >= 0.0 && objPositionTransformed.x <= maxScanDistance) 
				{
					//this scanner only measures the distance in x direction along the scanner line.
					if(objPositionTransformed.x < distance) {
						if(objPositionTransformed.x < minScanDistance) {
							distance = minScanDistance;
							break;
						}
						else {
							distance = objPositionTransformed.x;
						}
					}
				}
			}
			if(setIndex < mObstacleLists.size()) {
				obstacles = (SimObjectSet) mObstacleLists.elementAt(setIndex);
				setIndex++;
			}
			else {
				obstacles =  null;
			}
		}
		while(obstacles != null);
		
		//mDistance -= mMinScanDistance;
		double dist = Math.max(Math.min(addNoise(distance), maxScanDistance), minScanDistance);
		
		mDistance.set(Math.max(0.0, dist));

		return ok;
	}
	
}
