/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.devices;

import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.EnvironmentModel;
import de.fhg.iais.outdoor.simba.core.Sensor;
import de.fhg.iais.outdoor.simba.core.SimObject;

public class ScannerGroup extends Sensor {

	public static final int MODE_MIN = 1;
	public static final int MODE_MAX = 2;
	public static final int MODE_MEAN = 3;
	public static final int MODE_GREATEST_DIFFERENCE = 4;
	
	protected int mMode;
	protected Vector mScanners = new Vector();
	protected InternalSensor mInternalSensor;
	
	public ScannerGroup(String name, double x, double y, double angle, int mode, boolean alignSensors) {
		super(name, x, y, angle);
		mMode = mode;
		mInternalSensor = new InternalSensor(name + "/GroupSensor");
	}
	
	/**
	 * This method is called whenever this SimObject was added to an EnvironmentModel.
	 * Subclasses of SimObject can used this method to register sub components at the
	 * EnvironmentModel.
	 * Here the internal sensor is added to the sensor list.
	 * 
	 * @param model the EnvironmentModel where the child SimObjects should be registered.
	 */
	public void addChildren(EnvironmentModel model) {
		model.addSimObject(mInternalSensor);
	}
	
	/**
	 * Calculates the distance to the closest obstacle.
	 * 
	 */
	public boolean update() {
		boolean ok = super.update();
		
		return ok;
	}
	
	/**
	 * Returns the last measured value of this sensor.
	 * @return
	 */
	public double getSensorReading() {
		return mInternalSensor.getSensorReading();
	}
	
	/**
	 * Adds a scanner object to the group.
	 * 
	 * @param scanner the scanner to add.
	 * @return true if successful.
	 */
	public boolean addScanner(DistanceSensor scanner) {
		if(scanner == null || mScanners.contains(scanner)) {
			return false;
		}
		mScanners.add(scanner);
		mInternalSensor.addDependency(scanner);
		return true;
	}
	
	/**
	 * Adds a scanner object to the group and sets itself as reference object
	 * of that scanner.
	 * @param scanner the scanner to add.
	 * @return true if successful.
	 */
	public boolean addScannerAsChild(DistanceSensor scanner) {
		if(scanner == null) {
			return false;
		}
		addScanner(scanner);
		if(scanner instanceof SimObject) {
			SimObject scanObj = (SimObject) scanner; 
			scanObj.setReferenceObject(this);
			
		}
		return true;
	}
	
	/**
	 * Returns a Vector with all registered Scanner objects.
	 * @return 
	 */
	public Vector getScanners() {
		return new Vector(mScanners);
	}
	
	public class InternalSensor extends Sensor {
		
//		private Vector mSensorValues = new Vector();
		
		public InternalSensor(String name) {
			super(name, 0.0, 0.0, 0.0);
		}
		
		/**
		 * Calculates the distance to the closest obstacle.
		 * TODO - Postponed.
		 */
		public boolean update() {
			boolean ok = super.update();
//			mSensorValues.clear();
//			double[] scanValues = new double[mScanners.size()];
//			
//			//Read all sensor values.
//			for(int i = 0; i < mScanners.size(); ++i) {
//				DistanceSensor sensor = (DistanceSensor) mScanners.elementAt(i);
//				scanValues[i] = sensor.getCurrentValue();
//			}
//			
//			//determin the sensor's value
//			
//			//TODO
//			//aligh the values of the sensors.
//			for(Iterator i = mScanners.iterator(); i.hasNext();) {
//				//DistanceSensor sensor = (DistanceSensor) i.next();
//				//Vector readings = sensor.getSensorReading();
//			}
//			
			return ok;
		}
		
		/**
		 * Returns the last measured value of this sensor.
		 * @return
		 */
		public double getSensorReading() {
			return 0.0;
		}
		
	}
	
}
