/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.evolution;

import java.util.Iterator;
import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.EnvironmentModel;
import de.fhg.iais.outdoor.simba.core.GenerationListener;
import de.fhg.iais.outdoor.simba.core.Scheduler;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.fitness.FitnessFunction;

public class EvolutionConfigLogHandler implements GenerationListener {
	
	public EvolutionConfigLogHandler() {
		Simulator.getInstance().getScheduler().addGenerationListener(this);
		
	}
	
	

	public void generationCompleted(int generationId) {
	}



	public void generationStarted(int generationId) {
		EvolutionManager em = Simulator.getInstance().getEvolutionManager();
		Scheduler scheduler = Simulator.getInstance().getScheduler();
		EvolutionLogger logger = scheduler.getEvolutionLogger();
		
		logger.logXML("<configuration>", "</configuration>");
		
		StringBuffer entry = new StringBuffer();
		
		if(em.getEvolutionRunningValue().get()) {
			//Evolution parameters
			
			entry.append("<evolutionManager meanPopulationSize=\"").append(em.getMeanPopulationSizeValue().get())
					.append("\" preserveParents=\"" + em.getPreserveParentsValue().get() + "\">\n");
			//entry.append("\t<selectionMethod name=\"").append(em.getSelectionMethod().getName())
			//	.append("\"/>\n");
			entry.append("\t<evaluationMethod name=\"").append(em.getEvaluationMethod().getName())
				.append("\"/>\n");
			entry.append("</evolutionManager>\n");
			
			logger.logXML(entry.toString());
			
			
			//EvolutionOperators
			logger.logXML("<evolutionOperators>", "</evolutionOperators>");
			Vector operators = Simulator.getInstance().getEvolutionManager().getEvolutionOperators();
			for(Iterator i = operators.iterator(); i.hasNext();) {
				String opEntry = ((EvolutionOperator) i.next()).createParameterLog();
				logger.logXML(opEntry);
			}
			logger.closeTag();
		}
		
		//Scheduler parameters
		entry = new StringBuffer();
		entry.append("<scheduler numberOfTries=\"").append(scheduler.getNumberOfTries())
			.append("\" numberOfSteps=\"").append(scheduler.getNumberOfSteps()).append("\"/>");
		logger.logXML(entry.toString());
		
		//Fitness functions
		Vector fitnessFunctions = Simulator.getInstance().getFitnessManager().getFitnessFunctions();
		logger.logXML("<fitnessFunctions>", "</fitnessFunctions>");
		for(Iterator i = fitnessFunctions.iterator(); i.hasNext();) {
			String ffEntry = ((FitnessFunction) i.next()).createParameterLog();
			logger.logXML(ffEntry);
		}
		logger.closeTag();
		
		//Environments
		Vector environments = scheduler.getEnvironmentModels();
		logger.logXML("<environments>", "</environments>");
		for(int i = 0; i < environments.size(); ++i) {
			EnvironmentModel model = (EnvironmentModel) environments.elementAt(i);
			logger.logXML("<environment index=\"" + i + "\" name=\"" + model.getName() + "\">", "</environment>");
			logger.logXML(model.createParameterLog());
			logger.logXML(model.createRandomizationParameterLog());
			logger.closeTag();
		}
		logger.closeTag();
		
		
		logger.closeTag();
	}

}
