/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.evolution;

import de.fhg.iais.outdoor.simba.value.DoubleValue;
import de.fhg.iais.outdoor.simba.value.NormalizedDoubleValue;

public class EvolutionOperatorESMutateStepSize extends EvolutionOperator {

	private NormalizedDoubleValue mProbabilityOfChange;
	private DoubleValue mMaxChange;
	private DoubleValue mMaxStepSize;
	private DoubleValue mMinStepSize;
	
	public EvolutionOperatorESMutateStepSize() {
		super("ESMutateStepSize", 1);
		
		mProbabilityOfChange = new NormalizedDoubleValue(0.0, 0.0, 1.0, 0.0, 1.0);
		mMaxChange = new DoubleValue(0.5);
		mMaxStepSize = new DoubleValue(1.0);
		mMinStepSize = new DoubleValue(0.05);
		
		
		addParameter(mProbabilityOfChange, "ChangeProbability");
		addParameter(mMaxChange, "MaxChange");
		addParameter(mMaxStepSize, "MaxStepSize");
		addParameter(mMinStepSize, "MinStepSize");
	}
	
	
	public boolean executeOperation(EvolutionIndividual individual) {

		Double stepSize = (Double) individual.getProperty("ESStepSize");
		if(stepSize == null) {
			stepSize = new Double(mMaxStepSize.get());
			individual.setProperty("ESStepSize", stepSize);
			return true;
		}
		
		if(mRandom.nextDouble() < mProbabilityOfChange.get()) {
			double step = stepSize.doubleValue() + (((mRandom.nextDouble() * 2) - 1) * mMaxChange.get());
			step = Math.max(mMinStepSize.get(), Math.min(mMaxStepSize.get(), step));
			individual.setProperty("ESStepSize", new Double(step));
		}
		
		return true;
	}

}
