/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.evolution;

import java.util.Iterator;
import java.util.Vector;
import java.util.logging.Logger;

import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.neuralnet.Neuron;
import de.fhg.iais.outdoor.simba.value.IntValue;
import de.fhg.iais.outdoor.simba.value.NormalizedDoubleValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueManager;

public class EvolutionOperatorNNAddNeuron extends EvolutionOperator {

	private NormalizedDoubleValue mInsertProbability;
	private IntValue mMaxNumberOfNewNeurons;
	private IntValue mMaxNumberOfHiddenNeurons;
	
	public EvolutionOperatorNNAddNeuron() {
		super("NNAddNeuron", 500);
	
		mInsertProbability = new NormalizedDoubleValue(0.0, 0.0, 1.0, 0.0, 1.0);
		addParameter(mInsertProbability, "InsertionProbability");
		
		mMaxNumberOfNewNeurons = new IntValue(1);
		addParameter(mMaxNumberOfNewNeurons, "MaxNumberOfNewNeurons");
		
		mMaxNumberOfHiddenNeurons = new IntValue(100);
		addParameter(mMaxNumberOfHiddenNeurons, "MaxNumberOfHiddenNeurons");
	}

	public boolean executeOperation(EvolutionIndividual individual) {

		NeuralNet net = null;
		try {
			net = (NeuralNet) individual.getGenom();
		}
		catch(ClassCastException e) {
		}
		if(net == null) {
			Logger.getLogger("Simulator")
			.warning("The EvolutionIndividual did not have a NeuralNet as genom.");
			return true;
		}
		
		Vector neurons = net.getNeurons();
		//Determine the number of hidden neurons in the network.
		int numberOfHiddenNeurons = 0;
		for(Iterator i = neurons.iterator(); i.hasNext();) {
			Neuron n = (Neuron) i.next();
			if(n.getType().equals(Neuron.HIDDEN)) {
				numberOfHiddenNeurons++;
			}
		}
		
		int maxNumberOfNewNeurons = mMaxNumberOfNewNeurons.get();
		int maxNumberOfHiddenNeurons = mMaxNumberOfHiddenNeurons.get();
		for(int i = 0; i < maxNumberOfNewNeurons; ++i) {
			//do not insert more neurons that allowed.
			if(numberOfHiddenNeurons >= maxNumberOfHiddenNeurons) {
				break;
			}
			if(mRandom.nextDouble() < mInsertProbability.get()) {
				//insert neuron
				Neuron neuron = new Neuron("HiddenNeuron");
				neuron.setType(Neuron.HIDDEN);
				net.addNeuron(neuron);
				numberOfHiddenNeurons++;
			}
		}

		return true;
	}

	public void valueReplaced(Value oldValue, Value newValue) {
		super.valueReplaced(oldValue, newValue);
		mInsertProbability = (NormalizedDoubleValue) ValueManager.synchronizeValue(mInsertProbability, oldValue, newValue, this);
		mMaxNumberOfNewNeurons = (IntValue) ValueManager.synchronizeValue(mMaxNumberOfNewNeurons, oldValue, newValue, this);
		mMaxNumberOfHiddenNeurons = (IntValue) ValueManager.synchronizeValue(mMaxNumberOfHiddenNeurons, oldValue, newValue, this);
	}
	
	
}
