/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.evolution;

import java.util.Iterator;
import java.util.Vector;
import java.util.logging.Logger;

import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.neuralnet.Neuron;
import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.IntValue;

public class EvolutionOperatorNNRestrictSynapses extends EvolutionOperator {

	private IntValue mMaxInSynapsesHiddenNeurons;
	private IntValue mMaxInSynapsesOutputNeurons;
	private IntValue mMaxOutSynapsesHiddenNeurons;
	private IntValue mMaxOutSynapsesInputNeurons;
	private IntValue mMaxOutSynapsesOutputNeurons;
	private BooleanValue mPreserveRestrictions;
	
	
	public EvolutionOperatorNNRestrictSynapses() {
		super("NNRestrictSynapses", 500);
		
		mMaxInSynapsesHiddenNeurons = new IntValue(-1);
		mMaxInSynapsesOutputNeurons = new IntValue(-1);
		mMaxOutSynapsesHiddenNeurons = new IntValue(-1);
		mMaxOutSynapsesInputNeurons = new IntValue(-1);
		mMaxOutSynapsesOutputNeurons = new IntValue(-1);
		mPreserveRestrictions = new BooleanValue(false);
		
		addParameter(mMaxInSynapsesHiddenNeurons, "HiddenMaxIn");
		addParameter(mMaxOutSynapsesHiddenNeurons, "HiddenMaxOut");
		addParameter(mMaxInSynapsesOutputNeurons, "OutputMaxIn");
		addParameter(mMaxOutSynapsesOutputNeurons, "OutputMaxOut");
		addParameter(mMaxOutSynapsesInputNeurons, "InputMaxOut");
		addParameter(mPreserveRestrictions, "PreservePreviousRestrictions");
		
	}
	public boolean executeOperation(EvolutionIndividual individual) {
		NeuralNet net = null;
		try {
			net = (NeuralNet) individual.getGenom();
		}
		catch(ClassCastException e) {
		}
		if(net == null) {
			Logger.getLogger("Simulator")
			.warning("The EvolutionIndividual did not have a NeuralNet as genom.");
			return true;
		}
		
		Vector neurons = net.getNeurons();
		
		int hiddenMaxIn = mMaxInSynapsesHiddenNeurons.get();
		int hiddenMaxOut = mMaxOutSynapsesHiddenNeurons.get();
		int inputMaxOut = mMaxOutSynapsesInputNeurons.get();
		int outputMaxIn = mMaxInSynapsesOutputNeurons.get();
		int outputMaxOut = mMaxOutSynapsesOutputNeurons.get();
		boolean preserve = mPreserveRestrictions.get();
		
		for(Iterator i = neurons.iterator(); i.hasNext();) {
			Neuron n = (Neuron) i.next();
			if(n.getType().equals(Neuron.HIDDEN)) {
				
				if(!(preserve && n.getMaxIncomingSynapsesValue().get() != -1)
					&&	n.getMaxIncomingSynapsesValue().get() != hiddenMaxIn) 
				{
					n.getMaxIncomingSynapsesValue().set(hiddenMaxIn);
				}
				if(!(preserve && n.getMaxOutgoingSynapsesValue().get() != -1)
					&&	n.getMaxOutgoingSynapsesValue().get() != hiddenMaxOut) 
				{
					n.getMaxOutgoingSynapsesValue().set(hiddenMaxOut);
				}
			}
			else if(n.getType().equals(Neuron.INPUT)) {
				if(!(preserve && n.getMaxOutgoingSynapsesValue().get() != -1)
					&&	n.getMaxOutgoingSynapsesValue().get() != inputMaxOut) 
				{
					n.getMaxOutgoingSynapsesValue().set(inputMaxOut);
				}
			}
			else if(n.getType().equals(Neuron.OUTPUT)) {
				if(!(preserve && n.getMaxIncomingSynapsesValue().get() != -1)
					&&	n.getMaxIncomingSynapsesValue().get() != outputMaxIn) 
				{
					n.getMaxIncomingSynapsesValue().set(outputMaxIn);
				}
				if(!(preserve && n.getMaxOutgoingSynapsesValue().get() != -1)
					&&	n.getMaxOutgoingSynapsesValue().get() != outputMaxOut) 
				{
					n.getMaxOutgoingSynapsesValue().set(outputMaxOut);
				}
			}
		}
		
		return true;
	}

}
