/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.evolution;

import java.util.logging.Logger;

import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;

public class EvolutionOperatorNNSimpleParentCopy extends EvolutionOperator {

	
	public EvolutionOperatorNNSimpleParentCopy() {
		super("NNSimpleParentCopy", 1);
	}

	/**
	 * Can work with EvolutionIndividuals that provide at least 1 parent.
	 */
	public boolean executeOperation(EvolutionIndividual individual) {
		
		if(individual == null || individual.getParents().isEmpty()) {
			Logger.getLogger("Simulator")
				.warning("Could not find a parent to copy the neural network from!");
			return true;
		}
		//TODO choose the parent with the best fitness.
		EvolutionIndividual parent = (EvolutionIndividual) individual.getParents().firstElement();
		
		NeuralNet net = null;
		try {
			net = (NeuralNet) parent.getGenom();
		}
		catch(ClassCastException e) {
			net = null;
		}
		
		if(net == null) {
			Logger.getLogger("Simulator")
			.warning("The genom of the parent was not a NeuralNet object!");
			return true;
		}
		
		//Copy the network
		NeuralNet copy = null;
		try {
			copy = (NeuralNet) net.clone();
		} 
		catch (CloneNotSupportedException e) {
			copy = null;
		}
		if(copy == null) {
			Logger.getLogger("Simulator")
			.warning("Copy not clone the NeuralNet of the parent.");
			return true;
		}
		
		if(!individual.setGenom(copy)) {
			Logger.getLogger("Simulator")
			.warning("Could not set the NeuralNet as genom for the child.");
			return true;
		}
		
		return true;
	}

	public void reset() {
		super.reset();
	}
	
	
	
}
