/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.evolution;

import java.util.Iterator;
import java.util.Vector;
import java.util.logging.Logger;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.neuralnet.ConnectivityRating;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetAgentInterface;
import de.fhg.iais.outdoor.simba.neuralnet.Neuron;
import de.fhg.iais.outdoor.simba.neuralnet.NeuronWrapper;

public class EvolutionOperatorNNStructureValidator extends
		EvolutionOperator 
{

	
	public EvolutionOperatorNNStructureValidator() {
		super("NNStructureValidator", 500);
	}

	public boolean executeOperation(EvolutionIndividual individual) {
		
		
		NeuralNet net = null;
		try {
			net = (NeuralNet) individual.getGenom();
		}
		catch(ClassCastException e) {
		}
		if(net == null) {
			Logger.getLogger("Simulator")
			.warning("The EvolutionIndividual did not have a NeuralNet as genom.");
			return true;
		}
		
		Vector inputNeurons = new Vector();
		Vector outputNeurons = new Vector();
		
		NeuralNetAgentInterface agentInterface = Simulator.getInstance().getNeuralNetManager().getNeuralNetInterface();
		
		synchronized(net) {
			
			Vector inputNeuronWrappers = agentInterface.getInputNeurons();
			Vector outputNeuronWrappers = agentInterface.getOutputNeurons();
			
			Vector neurons = net.getNeurons();
			
			for(Iterator i = neurons.iterator(); i.hasNext();) {
				Neuron neuron = (Neuron) i.next();
				for(Iterator j = inputNeuronWrappers.iterator(); j.hasNext();) {
					NeuronWrapper nw = (NeuronWrapper) j.next();
					if(neuron.getActivationValue() == nw) {
						inputNeurons.add(neuron);
					}
				}
				for(Iterator j = outputNeuronWrappers.iterator(); j.hasNext();) {
					NeuronWrapper nw = (NeuronWrapper) j.next();
					if(neuron.getActivationValue() == nw) {
						outputNeurons.add(neuron);
					}
				}
			}
		
			//make sure that all inout neurons are connected to at least on output neuron
			for(Iterator i = inputNeurons.iterator(); i.hasNext();) {
				if(!ConnectivityRating.isConnected((Neuron) i.next(), outputNeurons)) {
					return false;
				}
			}
			
			for(Iterator i = outputNeurons.iterator(); i.hasNext();) {
				if(!ConnectivityRating.isInverseConnected((Neuron) i.next(), inputNeurons)) {
					return false;
				}
			}
		}
		return true;
	}
	
	
}
