/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.evolution;

import java.util.Iterator;
import java.util.Vector;
import java.util.logging.Logger;

import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.neuralnet.Synapse;
import de.fhg.iais.outdoor.simba.value.DoubleValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueChangedListener;
import de.fhg.iais.outdoor.simba.value.ValueManager;

public class EvolutionOperatorNNSynapseDiscretizer extends EvolutionOperator
			implements ValueChangedListener {

	private DoubleValue mFirstIntervalAccuracy;
	private DoubleValue mFirstIBorder;
	private DoubleValue mSecondIntervalAccuracy;
	private DoubleValue mSecondBorder;
	private DoubleValue mThirdIntervalAccuracy;
	
	public EvolutionOperatorNNSynapseDiscretizer() {
		super("NNSynapseDiscretizer", 500);
		
		mFirstIntervalAccuracy = new DoubleValue(0.1);
		mFirstIBorder = new DoubleValue(-1.0);
		mSecondIntervalAccuracy = new DoubleValue(0.05);
		mSecondBorder = new DoubleValue(1.0);
		mThirdIntervalAccuracy = new DoubleValue(0.1);
		
		mFirstIBorder.addValueChangedListener(this);
		mSecondBorder.addValueChangedListener(this);
		
		addParameter(mFirstIntervalAccuracy, "FirstIntervalAccuracy");
		addParameter(mSecondIntervalAccuracy, "SecondIntervalAccuracy");
		addParameter(mThirdIntervalAccuracy, "ThirdIntervalAccuracy");
		addParameter(mFirstIBorder, "FirstIntervalBorder");
		addParameter(mSecondBorder, "SecondIntervalBorder");
		
	}
	
	public boolean executeOperation(EvolutionIndividual individual) {
		
		NeuralNet net = null;
		try {
			net = (NeuralNet) individual.getGenom();
		}
		catch(ClassCastException e) {
		}
		if(net == null) {
			Logger.getLogger("Simulator")
			.warning("The EvolutionIndividual did not have a NeuralNet as genom.");
			return true;
		}
		
		Vector synapses = net.getSynapses();
		for(Iterator i = synapses.iterator(); i.hasNext();) {
			Synapse s = (Synapse) i.next();
			double strength = s.getStrength();
			if(strength < mFirstIBorder.get()) {
				strength = ((double) ((int) Math.round(strength / mFirstIntervalAccuracy.get()))) * mFirstIntervalAccuracy.get();
			}
			else if(strength < mSecondBorder.get()) {
				strength = ((double) ((int) Math.round(strength / mSecondIntervalAccuracy.get()))) * mSecondIntervalAccuracy.get();
			}
			else {
				strength = ((double) ((int) Math.round(strength / mThirdIntervalAccuracy.get()))) * mThirdIntervalAccuracy.get();
			}
			s.setStrength(strength);
		}
		
		
		return true;
		
	}

	public void valueChanged(Value value) {
		if(value == null) {
			return;
		}
		if(value == mFirstIBorder) {
			if(mFirstIBorder.get() > mSecondBorder.get()) {
				mFirstIBorder.set(mSecondBorder.get());
			}
		}
		else if(value == mSecondBorder) {
			if(mSecondBorder.get() < mFirstIBorder.get()) {
				mSecondBorder.set(mFirstIBorder.get());
			}
		}
	}

	public void valueReplaced(Value oldValue, Value newValue) {
		super.valueReplaced(oldValue, newValue);
		mFirstIntervalAccuracy = (DoubleValue) ValueManager.synchronizeValue(mFirstIntervalAccuracy, oldValue, newValue, this);
		mFirstIBorder = (DoubleValue) ValueManager.synchronizeValue(mFirstIBorder, oldValue, newValue, this);
		mSecondIntervalAccuracy = (DoubleValue) ValueManager.synchronizeValue(mSecondIntervalAccuracy, oldValue, newValue, this);
		mSecondBorder = (DoubleValue) ValueManager.synchronizeValue(mSecondBorder, oldValue, newValue, this);
		mThirdIntervalAccuracy = (DoubleValue) ValueManager.synchronizeValue(mThirdIntervalAccuracy, oldValue, newValue, this);
	}
	
	
	
	
	
	
	
	
}
