/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.evolution;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Iterator;
import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetAgentInterface;
import de.fhg.iais.outdoor.simba.xml.NeuralNetParser;

public class GenerationFileHandler {

	public GenerationFileHandler() {
		
	}
	
	public boolean saveGeneration(String filename) {
		
		EvolutionLogger logger = new EvolutionLogger();
		int mCurrentGeneration = Simulator.getInstance().getScheduler().getCurrentGeneration();
		if(!logger.startEvolutionLogger(filename)) {
			logger.closeEvolutionLogger();
			return false;
		}
		logger.logXML("<generation id=\"" + mCurrentGeneration + "\">", "</generation>");
		
		Vector individuals = Simulator.getInstance().getEvolutionManager().getGeneration();
		for(Iterator i = individuals.iterator(); i.hasNext();) {
			EvolutionIndividual ind = (EvolutionIndividual) i.next();
			if(ind != null && ind.getGenom() instanceof NeuralNet) {
				NeuralNet net = (NeuralNet) ind.getGenom();
				logger.logXML(NeuralNetParser.createPureSimbaXMLFromNet(net));
			}
		}
		
		logger.closeEvolutionLogger();
		
		return true;
	}
	
	public boolean loadGeneration(String filename) {

		Vector generation = new Vector();
		
		NeuralNetAgentInterface agent = Simulator.getInstance().getNeuralNetManager().getNeuralNetInterface();
		try {
			BufferedReader reader = new BufferedReader(new FileReader(filename));
			
			String line = reader.readLine();
			
			while(line != null) {
				while(line != null && line.indexOf("<net") != -1) {
					StringBuffer xml = new StringBuffer("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
					do {
						xml.append(line + "\n");
						line = reader.readLine();
					} while(line.indexOf("</net") == -1);
					xml.append(line + "\n");
					
					
					NeuralNet net = NeuralNetParser.createNetFromSimbaXML(xml.toString(), agent);
					if(net != null) {
						EvolutionIndividual ind = new EvolutionIndividual(net);
						generation.add(ind);
					}
					
					line = reader.readLine();
					
				}
				line = reader.readLine();
			}
			
			if(!generation.isEmpty()) {
				Simulator.getInstance().getEvolutionManager().replaceGeneration(generation);
			}
			
		} catch (FileNotFoundException e) {
			return false;
		} catch (IOException e) {
			return false;
		}
		
		
		return true;
	}
}
