/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.evolution;

import java.util.Iterator;
import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.xml.NeuralNetParser;

public abstract class NeuralNetSelectionMethod extends SelectionMethod {

	public NeuralNetSelectionMethod(String name) {
		super(name);
	}
	
	/**
	 * Logs the Individuals in the Vector to the evolution logger.
	 * All individuals have to have a NeuralNetwork as genom. If not, then these
	 * Individuals are not logged.
	 * 
	 * @param networks a Vector with the Individuals to log.
	 */
	public void logBestNetworks(Vector networks) {
		
		EvolutionLogger logger = Simulator.getInstance().getScheduler().getEvolutionLogger();
		
		int counter = 0;
		for(Iterator i = networks.iterator(); i.hasNext() && counter < getNumberOfGenomsToLog(); counter++) {
			EvolutionIndividual individual = (EvolutionIndividual) i.next();
			
			NeuralNet net = (NeuralNet) individual.getGenom();
			
			if(net != null) {
				logger.logXML("<individual id=\"" + individual.getId() + "\" age=\"" + individual.getAge() 
								+ "\" numberOfChildren=\"" + individual.getNumberOfChildren() + "\">", "</individual>");
				Vector fitnessValues = individual.getFitnessValues();
				for(int j = 0; j < fitnessValues.size(); ++j) {
					logger.logXML("<fitness index=\"" + j + "\" value=\"" + ((Double) fitnessValues.elementAt(j)).doubleValue() + "\"/>");
				}
				logger.logXML(NeuralNetParser.createPureSimbaXMLFromNet(net));
				logger.closeTag();
			}
		}
	}
	
	public void logGenoms(Vector genoms) {
		logBestNetworks(genoms);
	}

}
