/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.evolution;

import java.util.Iterator;
import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.ParameterElement;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.value.IntValue;
import de.fhg.iais.outdoor.simba.value.NormalizedDoubleValue;
import de.fhg.iais.outdoor.simba.value.ObjectValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueManager;
import de.fhg.iais.outdoor.simba.value.ValueReplacedListener;

public abstract class SelectionMethod extends ParameterElement implements ValueReplacedListener {
	
	private IntValue mNumberOfGenomsToLog;
	private IntValue mFitnessFunctionIndex;
	private NormalizedDoubleValue mOffspringPercentage;
	
	public SelectionMethod(String name) {
		super(name, "/Evolution/Selection/");
		mName = name;

		ValueManager vm = Simulator.getInstance().getValueManager();
		
		int counter = 1;
		ObjectValue ov = new ObjectValue(this);
		while(!vm.addValue("/Evolution/Selection/" + mName, ov)) {
			mName = name + counter;
			counter++;
		}
		
		name = mName;
		
		mNumberOfGenomsToLog = new IntValue(3);
		mFitnessFunctionIndex = new IntValue(0);
		mOffspringPercentage = new NormalizedDoubleValue(100.0, 0.0, 100.0, 0.0, 100.0);
		
		addParameter(mNumberOfGenomsToLog, "NumberOfBestGenomsToLog");
		addParameter(mFitnessFunctionIndex, "FitnessFunctionIndex");
		addParameter(mOffspringPercentage, "OffspringPercentage");
	}
	
	public abstract Vector selectParents(Vector individuals);
	public abstract Vector createChildrenStubs(Vector parents, int numberOfChildren);

	public Vector sortIndividualsByFitness(Vector individuals) {
		
		int fitnessFunctionIndex = mFitnessFunctionIndex.get();
		
		Vector fitnessSortedIdividuals = new Vector();
		for(Iterator i = individuals.iterator(); i.hasNext();) {
			EvolutionIndividual ind = (EvolutionIndividual) i.next();
			if(ind.getFitnessValues().size() > fitnessFunctionIndex) {
				double fitness = Math.max(0.0, ((Double) ind.getFitnessValues().elementAt(fitnessFunctionIndex)).doubleValue());
				boolean added = false;
				for(int j = 0; j < fitnessSortedIdividuals.size(); ++j) {
					EvolutionIndividual sInd = (EvolutionIndividual) fitnessSortedIdividuals.elementAt(j);
					double sFitness = Math.max(0.0, ((Double) sInd.getFitnessValues().elementAt(fitnessFunctionIndex)).doubleValue());
					
					if(fitness > sFitness) {
						fitnessSortedIdividuals.add(j, ind);
						added = true;
						break;
					}
				}
				if(!added) {
					fitnessSortedIdividuals.add(ind);
				}
			}
			else {
				fitnessSortedIdividuals.add(ind);
			}
		}
		return fitnessSortedIdividuals;
	}
	
	public void valueReplaced(Value oldValue, Value newValue) {
		super.valueReplaced(oldValue, newValue);
	}
	
	public int getNumberOfGenomsToLog() {
		return mNumberOfGenomsToLog.get();
	}
	
	public void logGenoms(Vector genoms) {
		
	}
	
	public void setFitnessFunctionIndex(int index) {
		mFitnessFunctionIndex.set(index);
	}
	
	public IntValue getFitnessFunctionIndexValue() {
		return mFitnessFunctionIndex;
	}
	
	public NormalizedDoubleValue getOffspringPercentageValue() {
		return mOffspringPercentage;
	}
	
	
	
}
