/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.evolution;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.statistic.EvolutionStatistics;

public class SimbaDefaultEvolution {

	public SimbaDefaultEvolution() {
		

		EvolutionManager evoManager = Simulator.getInstance().getEvolutionManager();
		
		
		//add selection methods
		
		
		//set current selection method
		
		
		//add operators.
		evoManager.addEvolutionOperator(new EvolutionOperatorNNSimpleParentCopy(), 0);
		evoManager.addEvolutionOperator(new EvolutionOperatorNNRemoveNeuron(), 8);
		evoManager.addEvolutionOperator(new EvolutionOperatorNNRemoveSynapse(), 10);
		evoManager.addEvolutionOperator(new EvolutionOperatorNNRestrictSynapses(), 15);
		evoManager.addEvolutionOperator(new EvolutionOperatorNNAddNeuron(), 20);
		evoManager.addEvolutionOperator(new EvolutionOperatorNNChangeTransferFunction(), 25);
		evoManager.addEvolutionOperator(new EvolutionOperatorNNAddSynapse(), 30);
		evoManager.addEvolutionOperator(new EvolutionOperatorNNInitializeZeroSynapses(), 35);
		evoManager.addEvolutionOperator(new EvolutionOperatorESMutateStepSize(), 38);
		evoManager.addEvolutionOperator(new EvolutionOperatorNNChangeStrength(), 40);
		evoManager.addEvolutionOperator(new EvolutionOperatorNNChangeBias(), 50);
		
		evoManager.addEvolutionOperator(new EvolutionOperatorNNSynapseDiscretizer(), 300);
		evoManager.addEvolutionOperator(new EvolutionOperatorNNStructureValidator(), 1000);
		
		//deactivate some of the operators
		
		
		//Set the default evaluation method
		evoManager.setEvaluationMethod(new NeuralNetEvaluationMethod());
		
		//Add selection methods
		evoManager.addSelectionMethod(new SimpleRouletteWheelSelection());
		
	
		//If there is no initial individual, then one is created automatically
//		if(evoManager.getGeneration().isEmpty()) {
//			NeuralNet net = NeuralNet.createMinimalNet(Simulator.getInstance().getNeuralNetManager().getNeuralNetInterface());
//			if(net != null) {
//				EvolutionIndividual ind = new EvolutionIndividual(net);
//				Vector generation = new Vector();
//				generation.add(ind);
//				evoManager.replaceGeneration(generation);
//			}
//		}
		
		//Set default steps, tries, population size
		Simulator.getInstance().getScheduler().getNumberOfStepsValue().set(2000);
		Simulator.getInstance().getScheduler().getNumberOfTriesValue().set(6);
		Simulator.getInstance().getEvolutionManager().getMeanPopulationSizeValue().set(200);
		
		
		//Set evolution loggers and statistics
		new EvolutionStatistics();
		GenerationAutoLogger genLogger = new GenerationAutoLogger("Default");
		genLogger.setActive(false);
		
	}
}
