/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.fitness;

import java.util.Iterator;
import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.SystemObject;

public class FitnessManager implements SystemObject {
	
	
	private Vector mFitnessFunctions = new Vector();
	
	public FitnessManager() {
		
	}
	
	public boolean cleanUp() {
		return true;
	}

	public boolean init() {
		return true;
	}


	public void prepareNextTry(int tryId) {
		//reset the try accumulators of the fitness functions
		synchronized (mFitnessFunctions) {
			for(Iterator i = mFitnessFunctions.iterator(); i.hasNext(); ){
				FitnessFunction f = (FitnessFunction) i.next();
				if(f.isActive()) {
					f.prepareNextTry(tryId);
				}
			}
		}
	}

	public void executeFitnessFunctions() {
		synchronized (mFitnessFunctions) {
			for(Iterator i = mFitnessFunctions.iterator(); i.hasNext(); ){
				FitnessFunction f = (FitnessFunction) i.next();
				if(f.isActive()) {
					f.update();
				}
			}
		}
	}
	
	
	public boolean addFitnessFunction(FitnessFunction fitnessFunction) {
		synchronized (mFitnessFunctions) {
			if(fitnessFunction == null || mFitnessFunctions.contains(fitnessFunction)) {
				return false;
			}
			mFitnessFunctions.add(fitnessFunction);
		}
		return true;
	}
	
	public boolean removeFitnessFunction(FitnessFunction fitnessFunction) {
		synchronized (mFitnessFunctions) {
			if(fitnessFunction == null || !mFitnessFunctions.contains(fitnessFunction)) {
				return false;
			}
			mFitnessFunctions.remove(fitnessFunction);
		}
		return true;
	}
	
	public Vector getFitnessFunctions() {
		synchronized (mFitnessFunctions) {
			return new Vector(mFitnessFunctions);
		}
	}
	
	public void removeAllFitnessFunctions() {
		synchronized (mFitnessFunctions) {
			mFitnessFunctions.clear();
		}
	}

}
