/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.fitnessfunctions;

import java.util.logging.Logger;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.devices.Box;
import de.fhg.iais.outdoor.simba.fitness.FitnessFunction;
import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.DoubleValue;
import de.fhg.iais.outdoor.simba.value.SimObjectValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueManager;

public class DelayedResponse2BeaconFitness1 extends FitnessFunction {

	private BooleanValue mHitTargetLeft;
	private BooleanValue mHitTargetRight;
	private DoubleValue mDistTargetLeft;
	private DoubleValue mDistTargetRight;
	private DoubleValue mTargetZoneLeft;
	private DoubleValue mSecondTargetZoneLeft;
	private SimObjectValue mRobot;
	private SimObjectValue mLeftConnector;
	private double mInitDistance;
	private double mAccu;
	private boolean mFirstOccuranceOfLeft;
	private boolean mFirstOccuranceOfRight;
	private double mMinimalGain;
	private double mFirstOccurenceMinimalGain;
	private boolean mFitnessIsMinimalGain;
	private int mSectorCounter;
	
	
	public DelayedResponse2BeaconFitness1() {
		super("DelayedResponse2BeaconFitness1");
		mAccu = 0.0;
		mInitDistance = Double.MAX_VALUE;
	}

	public boolean init() {
		boolean ok = super.init();
		ValueManager vm = Simulator.getInstance().getValueManager();
		
		mHitTargetLeft = vm.getBooleanValue("/SimObjects/HitTargetLeft/BumpDetected");
		mHitTargetRight = vm.getBooleanValue("/SimObjects/HitTargetRight/BumpDetected");
		mDistTargetLeft = vm.getDoubleValue("/SimObjects/DistanceToLeftArea/Distance");
		mDistTargetRight = vm.getDoubleValue("/SimObjects/DistanceToRightArea/Distance");
		mTargetZoneLeft = vm.getDoubleValue("/FitnessInfo/TargetZoneLeft");
		mSecondTargetZoneLeft = vm.getDoubleValue("/FitnessInfo/SecondTargetZoneLeft");
		mRobot = vm.getSimObjectValue("/SimObjects/Robot");
		mLeftConnector = vm.getSimObjectValue("/SimObjects/LeftConnector");
		
		if(mHitTargetLeft == null || mHitTargetRight == null
			|| mDistTargetLeft == null || mDistTargetRight == null
			|| mTargetZoneLeft == null || mRobot == null 
			|| mLeftConnector == null || mSecondTargetZoneLeft == null)
		{
			ok = false;
			Logger.getLogger("Simulator")
				.warning(getName() + ".init: Did not find all required Value objects.\n"
						+ mHitTargetLeft + "\n" + mHitTargetRight + "\n" + mDistTargetLeft + "\n" + mDistTargetRight
						+ "\n" + mTargetZoneLeft);
		}
		else {
			mHitTargetLeft.addValueReplacedListener(this);
			mHitTargetRight.addValueReplacedListener(this);
			mDistTargetLeft.addValueReplacedListener(this);
			mDistTargetRight.addValueReplacedListener(this);
			mTargetZoneLeft.addValueReplacedListener(this);
			mSecondTargetZoneLeft.addValueReplacedListener(this);
			mRobot.addValueReplacedListener(this);
			mLeftConnector.addValueReplacedListener(this);
		}
		return ok;
	}

	public void reset() {
		mInitDistance = Double.MAX_VALUE;
		mAccu = 0.0;
		mFirstOccuranceOfLeft = true;
		mFirstOccuranceOfRight = true;
		mMinimalGain = 0.0;
		mFitnessIsMinimalGain = false;
		mFirstOccurenceMinimalGain = 0.0;
		mSectorCounter = 0;
	}

	public double calculateFitness() {
		
		if(mHitTargetLeft == null || mHitTargetRight == null
				|| mDistTargetLeft == null || mDistTargetRight == null
				|| mTargetZoneLeft == null || mSecondTargetZoneLeft == null)
		{
			return 0.0;
		}
		

		
		//determine target zone
		double dist = Double.MAX_VALUE;
		
		if(mTargetZoneLeft.get() == 0.0) {
			dist = mDistTargetRight.get();
		}
		else {
			dist = mDistTargetLeft.get();
		}
		if(mInitDistance >= Double.MAX_VALUE) {
			mInitDistance = dist;
		}
		
		if((mSectorCounter % 2 == 0)) {
			if(mTargetZoneLeft.get() > 0.0) {
				if(mHitTargetLeft.get()) {
					mAccu += Math.pow((mInitDistance - dist), 2);
					mMinimalGain = 0.0;
					mInitDistance = Double.MAX_VALUE;
					dist = Double.MAX_VALUE;
					mFirstOccuranceOfLeft = false;
					mSectorCounter++;
				}
				else if(mHitTargetRight.get() && mFirstOccuranceOfLeft) {
					mFitnessIsMinimalGain = true;
				}
			}
			else {
				if(mHitTargetRight.get()) {
					mAccu += Math.pow((mInitDistance - dist), 2);
					mMinimalGain = 0.0;
					mInitDistance = Double.MAX_VALUE;
					dist = Double.MAX_VALUE;
					mFirstOccuranceOfRight = false;
					mSectorCounter++;
				}
				else if(mHitTargetLeft.get() && mFirstOccuranceOfRight) {
					mFitnessIsMinimalGain = true;
				}
			}
		}
		else {
			if(mSecondTargetZoneLeft.get() > 0.0) {
				if(mHitTargetLeft.get()) {
					mAccu += Math.pow((mInitDistance - dist), 2);
					mMinimalGain = 0.0;
					mInitDistance = Double.MAX_VALUE;
					dist = Double.MAX_VALUE;
					mFirstOccuranceOfLeft = false;
					mSectorCounter++;
				}
				else if(mHitTargetRight.get() && mFirstOccuranceOfLeft) {
					mFitnessIsMinimalGain = true;
				}
			}
			else {
				if(mHitTargetRight.get()) {
					mAccu += Math.pow((mInitDistance - dist), 2);
					mMinimalGain = 0.0;
					mInitDistance = Double.MAX_VALUE;
					dist = Double.MAX_VALUE;
					mFirstOccuranceOfRight = false;
					mSectorCounter++;
				}
				else if(mHitTargetLeft.get() && mFirstOccuranceOfRight) {
					mFitnessIsMinimalGain = true;
				}
			}
		}
		
	
		if(mMinimalGain == 0.0) { 
			Box leftConnector = (Box) mLeftConnector.get();
			if(mRobot.get().getGlobalPosition().y >= 
				leftConnector.getGlobalPosition().y + (leftConnector.getHeight() / 2.0))
			{
				mMinimalGain = (mInitDistance - dist);
			}
			if(mFirstOccurenceMinimalGain == 0.0) {
				mFirstOccurenceMinimalGain = mMinimalGain;
			}
		}
	
		if(mFitnessIsMinimalGain) {
			return (Math.pow(mFirstOccurenceMinimalGain, 2) / 10000.0);
		}
		else {
			return ((mAccu + (Math.pow((Math.max((mInitDistance - dist), mMinimalGain)), 2))) / 10000.0);
		}
		
	}

	public void valueReplaced(Value oldValue, Value newValue) {
		super.valueReplaced(oldValue, newValue);
		
		mHitTargetLeft = (BooleanValue) ValueManager.synchronizeValue(mHitTargetLeft, oldValue, newValue, this);
		mHitTargetRight = (BooleanValue) ValueManager.synchronizeValue(mHitTargetRight, oldValue, newValue, this);
		mDistTargetLeft = (DoubleValue) ValueManager.synchronizeValue(mDistTargetLeft, oldValue, newValue, this);
		mDistTargetRight = (DoubleValue) ValueManager.synchronizeValue(mDistTargetRight, oldValue, newValue, this);
		mTargetZoneLeft = (DoubleValue) ValueManager.synchronizeValue(mTargetZoneLeft, oldValue, newValue, this);
		mSecondTargetZoneLeft = (DoubleValue) ValueManager.synchronizeValue(mSecondTargetZoneLeft, oldValue, newValue, this);
		mRobot = (SimObjectValue) ValueManager.synchronizeValue(mRobot, oldValue, newValue, this);
		mLeftConnector = (SimObjectValue) ValueManager.synchronizeValue(mLeftConnector, oldValue, newValue, this);
		
	}

}
