/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.neuralnet;

import java.util.Iterator;
import java.util.Vector;

public class ConnectivityRating {
	
	
	private static Vector mVisitedNeurons = new Vector();
	
	public synchronized static boolean isConnected(Neuron neuron, Vector targetNeurons) {
		
		mVisitedNeurons.clear();
		Vector neuronsToCheck = new Vector();
		neuronsToCheck = new Vector(targetNeurons);
		
		return checkRecursivelyIfConnected(neuronsToCheck, neuron);
	}
	
	public static boolean checkRecursivelyIfConnected(Vector neuronsToCheck, Neuron targetNeuron) {
		if(neuronsToCheck.isEmpty()) {
			return false;
		}
		Vector newNeuronList = new Vector();
		for(Iterator i = neuronsToCheck.iterator(); i.hasNext();) {
			Neuron neuron = (Neuron) i.next();
			if(neuron == targetNeuron) {
				return true;
			}
			if(!mVisitedNeurons.contains(neuron)) {
				mVisitedNeurons.add(neuron);
				
				Vector synapses = neuron.getSynapses();
				for(Iterator j = synapses.iterator(); j.hasNext();) {
					Synapse synapse = (Synapse) j.next();
					Neuron target = (Neuron) synapse.getSourceNeuron();
					if(target == null) {
						continue;
					}
					if(!mVisitedNeurons.contains(target)) {
						newNeuronList.add(target);
					}
				}
			}
		}
		return checkRecursivelyIfConnected(newNeuronList, targetNeuron);
	}
	
	public synchronized static boolean isInverseConnected(Neuron neuron, Vector targetNeurons) {
		
		mVisitedNeurons.clear();
		Vector neuronsToCheck = new Vector();
		neuronsToCheck.clear();
		neuronsToCheck.add(neuron);
		Vector targets = new Vector(targetNeurons);
		
		return checkRecursivelyIfInverseConnected(neuronsToCheck, targets);
	}
	
	public static boolean checkRecursivelyIfInverseConnected(Vector neuronsToCheck, Vector targetNeurons) {
		
		if(neuronsToCheck.isEmpty()) {
			return false;
		}
		Vector newNeuronList = new Vector();
		for(Iterator i = neuronsToCheck.iterator(); i.hasNext();) {
			Neuron neuron = (Neuron) i.next();
			if(targetNeurons.contains(neuron)) {
				return true;
			}
			if(!mVisitedNeurons.contains(neuron)) {
				mVisitedNeurons.add(neuron);
				
				Vector synapses = neuron.getSynapses();
				for(Iterator j = synapses.iterator(); j.hasNext();) {
					Synapse synapse = (Synapse) j.next();
					Neuron target = (Neuron) synapse.getSourceNeuron();
					if(target == null) {
						continue;
					}
					if(!mVisitedNeurons.contains(target)) {
						newNeuronList.add(target);
					}
				}
			}
		}
		return checkRecursivelyIfInverseConnected(newNeuronList, targetNeurons);
	}
	
	

}
