/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.neuralnet;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.Simulator;


public class NetworkLibrary {
	
	
	public static String getNetworkFingerPrint(NeuralNet net) {
	
		StringBuffer buffer = new StringBuffer();
		
		if(net == null) {
			return "";
		}
		synchronized(net) {
			Vector neurons = net.getNeurons();
			NeuralNetAgentInterface agentInterface = Simulator.getInstance().getNeuralNetManager().getNeuralNetInterface();
			neurons = sortNeurons(neurons, agentInterface);
			
			for(int i = 0; i < neurons.size(); ++i) {
				Neuron n = (Neuron) neurons.elementAt(i);
				
				buffer.append("(" + i);
				Vector synapses = n.getSynapses();
				synapses = sortSynapses(synapses);
				
				for(Iterator j = synapses.iterator(); j.hasNext();) {
					Synapse s = (Synapse) j.next();
					
					int k = 0;
					for(; k < neurons.size(); ++k) {
						if(neurons.elementAt(k) == s.getSourceNeuron()) {
							break;
						}
					}
					buffer.append("|" + k + "," + s.getStrength());
				}
				buffer.append(")");
			}
		}
		
		return buffer.toString();
	}
	
	public static Vector sortNeurons(Vector neurons, NeuralNetAgentInterface agentInterface) {
		Vector sortedNeurons = new Vector();
		LinkedList list = new LinkedList(neurons);
		
		Vector inputWrappers = agentInterface.getInputNeurons();
		for(Iterator i = inputWrappers.iterator(); i.hasNext();) {
			NeuronWrapper nw = (NeuronWrapper) i.next();
			for(Iterator j = list.iterator(); j.hasNext();) {
				Neuron n = (Neuron) j.next();
				if(n.getActivationValue() == nw) {
					sortedNeurons.add(n);
					list.remove(n);
					break;
				}
			}
		}
		
		Vector hiddenNeurons = new Vector();
		for(Iterator i = list.iterator(); i.hasNext();) {
			Neuron n = (Neuron) i.next();
			if(!n.getType().equals(Neuron.HIDDEN)) {
				continue;
			}
			Vector nSynapses = n.getSynapses();
			boolean added = false;
			for(int j = 0; j < hiddenNeurons.size(); ++j) {
				Neuron hidden = (Neuron) hiddenNeurons.elementAt(j);
				Vector hSynapses = hidden.getSynapses();
				if(nSynapses.size() >= hSynapses.size()) {
					if(nSynapses.size() > hSynapses.size()) {
						//prefer the neuron with more synapses.
						hiddenNeurons.add(j, n);
						added = true;
						break;
					}
					else {
						//Check which of the neurons has synapses with higher strength.
						double hStrength = 0.0;
						double nStrength = 0.0;
						LinkedList hList = new LinkedList(hSynapses);
						LinkedList nList = new LinkedList(nSynapses);
						while(hStrength == nStrength) {
							if(hList.isEmpty() && nList.isEmpty()) {
								hStrength = 1.0;
								nStrength = 0.0;
							}
							hStrength = Double.MIN_VALUE;
							nStrength = Double.MIN_VALUE;
							Synapse best = null;
							for(int k = 0; k < nList.size(); ++k) {
								Synapse s = (Synapse) nList.get(k);
								if(nStrength < s.getStrength()) {
									nStrength = s.getStrength();
									best = s;
								}
							}
							nList.remove(best);
							for(int k = 0; k < hList.size(); ++k) {
								Synapse s = (Synapse) hList.get(k);
								if(hStrength < s.getStrength()) {
									hStrength = s.getStrength();
									best = s;
								}
							}
							hList.remove(best);
						}
						if(nStrength > hStrength) {
							hiddenNeurons.add(j, n);
							added = true;
							break;
						}
					}
				}
			}
			if(!added) {
				hiddenNeurons.add(n);
			}
		}
		sortedNeurons.addAll(hiddenNeurons);
		
		Vector outputWrappers = agentInterface.getOutputNeurons();
		for(Iterator i = outputWrappers.iterator(); i.hasNext();) {
			NeuronWrapper nw = (NeuronWrapper) i.next();
			for(Iterator j = list.iterator(); j.hasNext();) {
				Neuron n = (Neuron) j.next();
				if(n.getActivationValue() == nw) {
					sortedNeurons.add(n);
					list.remove(n);
					break;
				}
			}
		}
		
		return sortedNeurons;
	}
	
	public static Vector sortSynapses(Vector synapses) {
		Vector sorted = new Vector();
		for(int i = 0; i < synapses.size(); ++i) {
			boolean added = false;
			Synapse s = (Synapse) synapses.elementAt(i);
			for(int j = 0; j < sorted.size(); ++j) {
				Synapse ss = (Synapse) sorted.elementAt(j);
				if(ss.getStrength() < s.getStrength()) {
					sorted.add(j, s);
					added = true;
					break;
				}
			}
			if(!added) {
				sorted.add(s);
			}
		}
		return sorted; 
	}

}
