/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.neuralnet;

import java.util.Vector;

import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.DoubleValue;

public class Synapse implements Cloneable {

	public static final String DYNAMIC = "dynamic";
	public static final String PROTECTED = "protected";
	public static final String FIXED = "fixed";
	public static Vector mProtectionModes = new Vector();
	
	private Neuron mSource;
	private DoubleValue mStrength;
	private String mProtection;
	private Neuron mTarget;
	private BooleanValue mEnabled;
	
	public Synapse(Neuron sourceNeuron) {
		this(sourceNeuron, 0.0);
	}
	
	public Synapse(Neuron sourceNeuron, double strength) {
		this(sourceNeuron, strength, Synapse.DYNAMIC);
	}
	
	public Synapse(Neuron sourceNeuron, double strength, String protection) {
		//check if the static protection mode vector is already set up.
		if(mProtectionModes.isEmpty()) {
			mProtectionModes.add(Synapse.DYNAMIC);
			mProtectionModes.add(Synapse.PROTECTED);
			mProtectionModes.add(Synapse.FIXED);
		}
		
		mSource = sourceNeuron;
		mStrength = new DoubleValue(strength);
		mEnabled = new BooleanValue(true);
		mProtection = protection;
		mTarget = null;
	}
	
	
	public double getActivation() {
		if(!mEnabled.get()) {
			return 0.0;
		}
		return (mSource.getLastActivity() * mStrength.get());
	}
	
	public Neuron getSourceNeuron() {
		return mSource;
	}
	
	public Neuron getTargetNeuron() {
		return mTarget;
	}
	
	public void setStrength(double strength) {
		mStrength.set(strength);
	}
	
	public double getStrength() {
		return mStrength.get();
	}
	
	public DoubleValue getStrengthValue() {
		return mStrength;
	}
	
	public void setSourceNeuron(Neuron source) {
		mSource = source;
	}
	
	/**
	 * This method is called by the target neuron itself only!
	 * @param target
	 */
	public void setTargetNeuron(Neuron target) {
		mTarget = target;
	}
	
	public BooleanValue getEnabledValue() {
		return mEnabled;
	}
	
	/**
	 * One of DYNAMIC, PROTECTED, FIXED.
	 * @param type
	 */
	public void setProtectionMode(String type) {
		mProtection = type;
	}
	
	public String getProtectionMode() {
		return mProtection;
	}
	
	public Object clone() throws CloneNotSupportedException {
		Synapse obj = (Synapse) super.clone();
		obj.mStrength = new DoubleValue(mStrength.get());
		obj.mEnabled = new BooleanValue(mEnabled.get());
		obj.mSource = mSource; //the clone works with the same source, this has to be change manually if required.
		obj.mTarget = mTarget; //the clone works with the same target, this has to be change manually if required.
		return obj;
	}
	
}
