/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.simulators;


import java.awt.Color;
import java.awt.Polygon;
import java.util.Iterator;
import java.util.Random;
import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.EnvironmentModel;
import de.fhg.iais.outdoor.simba.core.SimObject;
import de.fhg.iais.outdoor.simba.core.SimObjectSet;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.devices.Beacon;
import de.fhg.iais.outdoor.simba.devices.Box;
import de.fhg.iais.outdoor.simba.devices.DifferentialDriveUnit;
import de.fhg.iais.outdoor.simba.devices.DistanceScanner;
import de.fhg.iais.outdoor.simba.devices.DistanceToPointScanner;
import de.fhg.iais.outdoor.simba.devices.LightSensor;
import de.fhg.iais.outdoor.simba.devices.PathDrawer;
import de.fhg.iais.outdoor.simba.devices.SimpleBumpDetector;
import de.fhg.iais.outdoor.simba.fitnessfunctions.DelayedResponse2BeaconFitness1;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetAgentInterface;
import de.fhg.iais.outdoor.simba.neuralnet.NeuronWrapper;
import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.IntValue;
import de.fhg.iais.outdoor.simba.value.ValueManager;
import de.fhg.iais.outdoor.simba.visualization.SimbaDefaultFrame;

public class DelayResponseMazeMultiBeacon1 extends SimbaDefaultFrame {

	private NeuralNetAgentInterface mMainController;
	
	public DelayResponseMazeMultiBeacon1(String args[]) {
		super("DelayResponseMazeMultiBeacon1", 700, 800);
		
		setDefaultOffset(-660.0, -680.0);
		setDefaultZoom(0.5);
		setVisualizationInterval(1);
		setSimulationDelay(15);
		
		mMainController = Simulator.getInstance().getNeuralNetManager().getNeuralNetInterface();
		
		setFitnessFunction(new DelayedResponse2BeaconFitness1(), 10000.0);
		setMasterEnvironment(new MyEnvironment(4));

		Simulator.getInstance().getNeuralNetManager().setCurrentNet(NeuralNet.createMinimalNet(mMainController));
		
		startSystem(mMainController, args);
	}
	
	

	
	public class MyEnvironment extends EnvironmentModel {
		
		//Arrays have to be cloned deeply in a custom clone() method.
		private int[] mOuterWidth;
		private int[] mInnerWidth;
		private int[] mOuterHeight;
		private int[] mInnerHeight;
		private double[] mRobotPositionX;
		private double[] mRobotPositionY;
		private double[] mInitAngle;
		private double[] mBeaconSize;
		private boolean[] mLightIsLeft;
		private boolean[] mSecondLightIsLeft;
		private double[] mBeaconOffset;
		private double[] mBeacon2Offset;
		private double[] mAngleBoardAngle;
		
		//SimObjects used in the environments (clones should use the identical objects (flat copy)
		private DifferentialDriveUnit mRobot;
		private SimObject mLowerT;
		private Box mUpperConnector;
		private Box mLeftConnector;
		private Box mRightConnector;
		private Box mTargetAreaLeft;
		private Box mTargetAreaRight;
		private Box mAngleBoard;
		private Beacon mBeaconLeft;
		private Beacon mBeaconRight;
		private Beacon mBeacon2Left;
		private Beacon mBeacon2Right;
		private DistanceToPointScanner mDistanceToLeftArea;
		private DistanceToPointScanner mDistanceToRightArea;
		private SimpleBumpDetector mTargetAreaLeftBumper;
		private SimpleBumpDetector mTargetAreaRightBumper;
		private boolean mLightOnTheLeft;
		private boolean mSecondLightOnTheLeft;
		private NeuronWrapper mCorrectTargetLeft;
		private NeuronWrapper mCorrectSecondTargetLeft;
		private PathDrawer mPathDrawer;
		
		private Random mRandom = new Random();
		
		private IntValue mMaxNumberOfSectors;
		private int mNumberOfSectors;
		private int mCurrentSector;
		private int mDesiredSector;
		private IntValue mCurrentSectorValue;
		private double mSectorStartAngle;
		
		public MyEnvironment(int numberOfSectors) {
			super("DelayResponseMazeMultiBeacon1");
			mMaxNumberOfSectors = new IntValue(numberOfSectors);
			Simulator.getInstance().getValueManager().addValue("/Environment/NumberOfSectors", mMaxNumberOfSectors);
			mNumberOfSectors = numberOfSectors;
			
			mCurrentSectorValue = new IntValue();
			Simulator.getInstance().getValueManager().addValue("/Environment/CurrentSector", mCurrentSectorValue);
			
			
			mOuterWidth = new int[numberOfSectors];
			mInnerWidth = new int[numberOfSectors];
			mOuterHeight = new int[numberOfSectors];
			mInnerHeight = new int[numberOfSectors];
			mRobotPositionX = new double[numberOfSectors];
			mRobotPositionY = new double[numberOfSectors];
			mInitAngle = new double[numberOfSectors];
			mBeaconSize = new double[numberOfSectors];
			mLightIsLeft = new boolean[numberOfSectors];
			mSecondLightIsLeft = new boolean[numberOfSectors];
			mLightOnTheLeft = true;
			mBeaconOffset = new double[numberOfSectors];
			mBeacon2Offset = new double[numberOfSectors];
			mAngleBoardAngle = new double[numberOfSectors];
		}
		
		
		public void toggleFirstLightSource() {
			mLightOnTheLeft = !mLightOnTheLeft;
		}
		
		public void toggleSecondLightSource() {
			mSecondLightOnTheLeft = !mSecondLightOnTheLeft;
		}
		

		
		/**
		 * Overridden for this special case: allow sector changes at the begining
		 * of each step.
		 * 
		 * Updates the environment, then the actuators and finally the sensors.
		 */
		public void executeSimulationStep() {

			if(mDesiredSector != mCurrentSector) {
				mCurrentSector = mDesiredSector;
				mCurrentSectorValue.set(mCurrentSector); //only for observation purposes, is never used for control.
				changeEnvironmentToSector(mCurrentSector);
			}
			
			super.executeSimulationStep();
		}
		
		/**
		 * Overridden for this special case: each created variation environment 
		 * is available twice, each with the opposite light distribution.
		 * 
		 * Creates the given number of environment variation clones 
		 * based on this Environment object and returns all these
		 * EnvironmentModels in a Vector.
		 * 
		 * This version creates two scenarios for each randomization pattern,
		 * one for a light on the left and one for a light on the right.
		 * 
		 * @param numberOfEnvironments the number of EnvironmentModels to create.
		 * @return a Vector with the newly created EnvironmentModels.
		 */
		public Vector createEnvironments(int numberOfEnvironments) {
			
			//create always for variants of each environment.
			if(numberOfEnvironments % 2 != 0) {
				numberOfEnvironments += (numberOfEnvironments % 2);
			}
			Vector clones = super.createEnvironments(numberOfEnvironments / 2);
		
			Vector environments = new Vector();
			for(Iterator i = clones.iterator(); i.hasNext();) {
				MyEnvironment env = (MyEnvironment) i.next();
				if(env != null) {
					environments.add(env);
				
					MyEnvironment envClone = null;
					try {
						envClone = (MyEnvironment) env.clone();
						envClone.toggleFirstLightSource();
						envClone.toggleSecondLightSource();
					} catch (CloneNotSupportedException e) {
						e.printStackTrace();
					}
					if(envClone != null) {
						environments.add(envClone);
					}
				}
			}
			return environments;
		}
		
		
		

		protected Object clone() throws CloneNotSupportedException {
			//Clone the arrays manually.
			MyEnvironment obj = (MyEnvironment) super.clone();

			obj.mOuterWidth = new int[mNumberOfSectors];
			obj.mInnerWidth = new int[mNumberOfSectors];
			obj.mOuterHeight = new int[mNumberOfSectors];
			obj.mInnerHeight = new int[mNumberOfSectors];
			obj.mRobotPositionX = new double[mNumberOfSectors];
			obj.mRobotPositionY = new double[mNumberOfSectors];
			obj.mInitAngle = new double[mNumberOfSectors];
			obj.mBeaconSize = new double[mNumberOfSectors];
			obj.mLightIsLeft = new boolean[mNumberOfSectors];
			obj.mSecondLightIsLeft = new boolean[mNumberOfSectors];
			obj.mBeaconOffset = new double[mNumberOfSectors];
			obj.mBeacon2Offset = new double[mNumberOfSectors];
			obj.mAngleBoardAngle = new double[mNumberOfSectors];
			
			for(int i = 0; i < mNumberOfSectors; ++i) {
				obj.mOuterWidth[i] = mOuterWidth[i];
				obj.mInnerWidth[i] = mInnerWidth[i];
				obj.mOuterHeight[i] = mOuterHeight[i];
				obj.mInnerHeight[i] = mInnerHeight[i];
				obj.mRobotPositionX[i] = mRobotPositionX[i];
				obj.mRobotPositionY[i] = mRobotPositionY[i];
				obj.mInitAngle[i] = mInitAngle[i];
				obj.mBeaconSize[i] = mBeaconSize[i];
				obj.mLightIsLeft[i] = mLightIsLeft[i];
				obj.mSecondLightIsLeft[i] = mSecondLightIsLeft[i];
				obj.mBeaconOffset[i] = mBeaconOffset[i];
				obj.mBeacon2Offset[i] = mBeacon2Offset[i];
				obj.mAngleBoardAngle[i] = mAngleBoardAngle[i];
			}
			return obj;
		}


		public void updateRandomizationParameters() {
			
			for(int i = 0; i < mNumberOfSectors; ++i) {
				mOuterWidth[i] = 270 + (int) (250.0 * mRandom.nextDouble());
				mInnerWidth[i] = mOuterWidth[i] - 40;
				mOuterHeight[i] = 80 + (int) (40.0 * mRandom.nextDouble());;
				mInnerHeight[i] = mOuterHeight[i] - 40;
				mRobotPositionX[i] = 0.0; //((-1.0 * (mInnerHeight[i] - 40.0)) + (2.0 * mRandom.nextDouble() * (mInnerHeight[i] - 40.0)));
				mRobotPositionY[i] = -150.0 + (10.0 - (mRandom.nextDouble() * 20.0));
				mInitAngle[i] = -25.0 + (mRandom.nextDouble() * 50.0);
				mBeaconSize[i] = 90.0 - (mRandom.nextDouble() * 55.0);
				if(i == 0) {
					mLightIsLeft[i] = mRandom.nextBoolean();
					mSecondLightIsLeft[i] = !mLightIsLeft[i]; //the first time, the beacons are on different sides always.
					
				}
				else if((i % 2 == 0)) {
					mLightIsLeft[i] = mRandom.nextBoolean();
					mSecondLightIsLeft[i] = (mRandom.nextDouble() > 0.7) ? !mLightIsLeft[i] : mRandom.nextBoolean();
				}
				else {
					mLightIsLeft[i] = mLightIsLeft[i - 1];
					mSecondLightIsLeft[i] = mSecondLightIsLeft[i - 1];
				}
				mBeaconOffset[i] = (mRandom.nextDouble() - 0.5) * ((mOuterWidth[i] - mInnerHeight[i]) / 2.0);
				mBeacon2Offset[i] = mBeaconOffset[i] + 150 + (mRandom.nextDouble() * 100.0);
				mAngleBoardAngle[i] = ((mRandom.nextDouble() - 0.5) * 30.0);
			}
			
		}
		

		public void resetEnvironmentObjects() {
			
			mDesiredSector = 0;
			mCurrentSector = 0;
			mCurrentSectorValue.set(0);
			changeEnvironmentToSector(0);
			
		}
		
		public void changeEnvironmentToSector(int sector) {
			
			if(sector == 0) {
				mSectorStartAngle = mInitAngle[0];
			}
			
			mPathDrawer.reset();
			
			mRobot.setPosition(mRobotPositionX[sector], mRobotPositionY[sector]);
			mRobot.setAngle(mSectorStartAngle);

			
			Polygon shape = new Polygon();
			shape.addPoint(-mInnerHeight[sector], mOuterHeight[sector]);
			shape.addPoint(-mInnerHeight[sector] - 30, mOuterHeight[sector]);
			shape.addPoint(-mInnerHeight[sector] - 30, -mOuterHeight[sector] - 150);
			shape.addPoint(mInnerHeight[sector] + 30, -mOuterHeight[sector] - 150);
			shape.addPoint(mInnerHeight[sector] + 30, mOuterHeight[sector]);
			
			shape.addPoint(mInnerHeight[sector], mOuterHeight[sector]);
			shape.addPoint(mInnerHeight[sector], mInnerHeight[sector]);
			shape.addPoint(mInnerHeight[sector], mInnerHeight[sector]);
			shape.addPoint(mInnerHeight[sector], -mInnerHeight[sector] - 150);
			shape.addPoint(-mInnerHeight[sector], -mInnerHeight[sector] - 150);
			shape.addPoint(-mInnerHeight[sector], mInnerHeight[sector]);
			
			shape.addPoint(-mInnerHeight[sector], mInnerHeight[sector]);
			
			mLowerT.setShape(new Polygon(shape.xpoints, shape.ypoints, shape.npoints));
			mUpperConnector.setPosition(0.0, mOuterHeight[sector] + mOuterWidth[sector] + mInnerHeight[sector] + ((mOuterHeight[sector] - mInnerHeight[sector]) / 2));
			mUpperConnector.setSize(mOuterWidth[sector] * 2, mOuterHeight[sector] - mInnerHeight[sector]);
			mLeftConnector.setPosition((mInnerHeight[sector] + ((mOuterWidth[sector] - mInnerHeight[sector]) / 2)) * -1, (mOuterHeight[sector] + ((mOuterWidth[sector] - mInnerHeight[sector]) / 2)));
			mLeftConnector.setSize(mOuterWidth[sector] - mInnerHeight[sector], mOuterWidth[sector] - mInnerHeight[sector]);
			mRightConnector.setPosition(mInnerHeight[sector] + ((mOuterWidth[sector] - mInnerHeight[sector]) / 2), (mOuterHeight[sector] + ((mOuterWidth[sector] - mInnerHeight[sector]) / 2)));
			mRightConnector.setSize(mOuterWidth[sector] - mInnerHeight[sector], mOuterWidth[sector] - mInnerHeight[sector]);
			mTargetAreaLeft.setSize((mOuterWidth[sector] - mInnerHeight[sector]), 2 * mInnerHeight[sector]);
			mTargetAreaLeft.setPosition(((mOuterWidth[sector] - mInnerHeight[sector]) + mInnerHeight[sector]) * -1, (mOuterWidth[sector] - mInnerHeight[sector]) +  mOuterHeight[sector] + mInnerHeight[sector]);
			mTargetAreaRight.setSize((mOuterWidth[sector] - mInnerHeight[sector]), 2 * mInnerHeight[sector]);
			mTargetAreaRight.setPosition(((mOuterWidth[sector] - mInnerHeight[sector]) + mInnerHeight[sector]), (mOuterWidth[sector] - mInnerHeight[sector]) +  mOuterHeight[sector] + mInnerHeight[sector]);
			mAngleBoard.setSize(mInnerHeight[sector] * 2, 40);
			mAngleBoard.setPosition(0.0, mOuterHeight[sector] + mOuterWidth[sector] + mInnerHeight[sector] + ((mOuterHeight[sector] - mInnerHeight[sector]) / 2) - 15);
			mAngleBoard.setAngle(mAngleBoardAngle[sector]);
			
			boolean leftLight = mLightOnTheLeft ? mLightIsLeft[sector] : !mLightIsLeft[sector];
			boolean leftSecondLight = mSecondLightOnTheLeft ? mSecondLightIsLeft[sector] : !mSecondLightIsLeft[sector];
			
			if(mCurrentSector % 2 == 0) {
				mBeaconLeft.setActive(true);
				mBeaconLeft.setPosition((mOuterWidth[sector] - mInnerHeight[sector]) / 2.0, -200.0 + mBeaconOffset[sector]);
				mBeaconLeft.getLightArea().setRadius(mInnerHeight[sector] * 2.0);
				mBeaconLeft.setInitialBrightness(leftLight ? 1.0 : 0.0);
				mBeaconLeft.getLightArea().setMaximalAngle(mBeaconSize[sector]);
				mBeaconLeft.getLightArea().setAngle(((90.0 - mBeaconSize[sector]) / 2.0));
				mBeaconRight.setActive(true);
				mBeaconRight.setPosition(-1 * ((mOuterWidth[sector] - mInnerHeight[sector]) / 2.0), -200.0 + mBeaconOffset[sector]);
				mBeaconRight.getLightArea().setRadius(mInnerHeight[sector] * 2.0);
				mBeaconRight.setInitialBrightness(leftLight ? 0.0 : 1.0);
				mBeaconRight.getLightArea().setMaximalAngle(mBeaconSize[sector]);
				mBeaconRight.getLightArea().setAngle(((90.0 - mBeaconSize[sector]) / 2.0));
				mBeacon2Left.setActive(true);
				mBeacon2Left.setPosition((mOuterWidth[sector] - mInnerHeight[sector]) / 2.0,  -200.0 + mBeacon2Offset[sector]);
				mBeacon2Left.getLightArea().setRadius(mInnerHeight[sector] * 2.0);
				mBeacon2Left.setInitialBrightness(leftSecondLight ? 1.0 : 0.0);
				mBeacon2Left.getLightArea().setMaximalAngle(mBeaconSize[sector]);
				mBeacon2Left.getLightArea().setAngle(((90.0 - mBeaconSize[sector]) / 2.0));
				mBeacon2Right.setActive(true);
				mBeacon2Right.setPosition(-1 * ((mOuterWidth[sector] - mInnerHeight[sector]) / 2.0), -200.0 + mBeacon2Offset[sector]);
				mBeacon2Right.getLightArea().setRadius(mInnerHeight[sector] * 2.0);
				mBeacon2Right.setInitialBrightness(leftSecondLight ? 0.0 : 1.0);
				mBeacon2Right.getLightArea().setMaximalAngle(mBeaconSize[sector]);
				mBeacon2Right.getLightArea().setAngle(((90.0 - mBeaconSize[sector]) / 2.0));
			}
			else {
				mBeaconLeft.setActive(false);
				mBeaconRight.setActive(false);
				mBeacon2Left.setActive(false);
				mBeacon2Right.setActive(false);
				mBeaconLeft.setInitialBrightness(0.0);
				mBeaconRight.setInitialBrightness(0.0);
				mBeacon2Left.setInitialBrightness(0.0);
				mBeacon2Right.setInitialBrightness(0.0);
			}
			
			mCorrectTargetLeft.set(leftLight ? 1.0 : 0.0);
			mCorrectSecondTargetLeft.set(leftSecondLight ? 1.0 : 0.0);
			
			if(mCurrentSector % 2 == 0) {
				mDistanceToLeftArea.setVisible(leftLight);
				mDistanceToRightArea.setVisible(!leftLight);
			}
			else {
				mDistanceToLeftArea.setVisible(leftSecondLight);
				mDistanceToRightArea.setVisible(!leftSecondLight);
			}
		}

		public void createEnvironment() {

			//Set up scenario.
			
			mRobot = new DifferentialDriveUnit("Robot", 0.0, 10.0, 0.0, 20.0, 5.0, -1.8, 1.8);
			Box robotBody = new Box("Robot/Body", 0.0, 0.0, 90.0, 14.0, 20.0);
			robotBody.setReferenceObject(mRobot);
			robotBody.setShapeColor(Color.GREEN);
			addSimObject(mRobot);
			addSimObject(robotBody);
			
			mPathDrawer = new PathDrawer("RobotTrajectory", 0.0, 0.0, 2, 100, 1000);
			mPathDrawer.setReferenceObject(mRobot);
			mPathDrawer.setLayer(1);
			mPathDrawer.setShapeColor(Color.blue);
			mPathDrawer.setSecondColor(Color.cyan);
			addSimObject(mPathDrawer);
			
//			TextLabel label = new TextLabel("Name", 0.0, -10.0, 0.0, "HalloWelt", 10, true);
//			label.setReferenceObject(mRobot);
//			label.setLayer(100);
//			label.setShapeColor(Color.blue);
//			addSimObject(label);
			
			
			mLowerT = new SimObject("LowerT", 0.0, 0.0, 0.0, new Polygon());
			addSimObject(mLowerT);
	
			mUpperConnector = new Box("UpperConnector", 0.0, 0.0, 0.0, 0.0, 0.0);
			addSimObject(mUpperConnector);
			
			mLeftConnector = new Box("LeftConnector", 0.0, 0.0, 0.0, 0.0, 0.0);
			addSimObject(mLeftConnector);
			
			mRightConnector = new Box("RightConnector", 0.0, 0.0, 0.0, 0.0, 0.0);
			addSimObject(mRightConnector);
			
			mTargetAreaLeft = new Box("TargetAreaLeft", 0.0, 0.0, 0.0, 0.0, 0.0);
			mTargetAreaLeft.setShapeColor(Color.lightGray);
			mTargetAreaLeft.setLayer(-5);
			addSimObject(mTargetAreaLeft);
			
			mTargetAreaRight = new Box("TargetAreaRight", 0.0, 0.0, 0.0, 0.0, 0.0);
			mTargetAreaRight.setShapeColor(Color.lightGray);
			mTargetAreaRight.setLayer(-5);
			addSimObject(mTargetAreaRight);
			
			mAngleBoard = new Box("AngleBoard", 0.0, 0.0, 0.0, 0.0, 0.0);
			addSimObject(mAngleBoard);
			
			SimObjectSet obstacleList = new SimObjectSet();
			obstacleList.add(mLowerT);
			obstacleList.add(mLeftConnector);
			obstacleList.add(mRightConnector);
			obstacleList.add(mUpperConnector);
			obstacleList.add(mAngleBoard);
			
			DistanceScanner scanLeft = new DistanceScanner("DistanceScannerLeft", 0.0, 10.0, 60.0, 0.0, 50.0);
			scanLeft.setReferenceObject(robotBody);
			scanLeft.addDependencies(obstacleList);
			scanLeft.addObstacleSet(obstacleList);
			scanLeft.setNoiseVariance(1.0);
			addSimObject(scanLeft);
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForDistanceScanner(scanLeft));
			
			DistanceScanner scanRight = new DistanceScanner("DistanceScannerRight", 0.0, -10.0, -60.0, 0.0, 50.0);
			scanRight.setReferenceObject(robotBody);
			scanRight.addDependencies(obstacleList);
			scanRight.addObstacleSet(obstacleList);
			scanRight.setNoiseVariance(1.0);
			addSimObject(scanRight);
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForDistanceScanner(scanRight));
			
			DistanceScanner scanMid = new DistanceScanner("DistanceScannerFront", 7.0, 0.0, 0.0, 0.0, 40.0);
			scanMid.setReferenceObject(robotBody);
			scanMid.addObstacleSet(obstacleList);
			scanMid.addObstacleSet(obstacleList);
			scanMid.setNoiseVariance(1.0);
			addSimObject(scanMid);
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForDistanceScanner(scanMid));
			
			
			//Beacons
			mBeaconLeft = new Beacon("BeaconLeft", 0.0, 0.0, 315.0, 0.0, 20, 90.0, 1.0, 1.0);
			mBeaconLeft.setReferenceObject(mLeftConnector);
			mBeaconLeft.setLayer(1);
			mBeaconLeft.getLightArea().setLayer(-1);
			mBeaconLeft.getLightArea().setShapeColor(Color.ORANGE);
			addSimObject(mBeaconLeft);

			mBeaconRight = new Beacon("BeaconRight", 0.0, 0.0, 135.0, 0.0, 20, 90.0, 1.0, 1.0);
			mBeaconRight.setReferenceObject(mRightConnector);
			mBeaconRight.setLayer(1);
			mBeaconRight.getLightArea().setLayer(-1);
			mBeaconRight.getLightArea().setShapeColor(Color.GREEN);
			addSimObject(mBeaconRight);
			
			mBeacon2Left = new Beacon("Beacon2Left", 0.0, 0.0, 315.0, 0.0, 20, 90.0, 1.0, 1.0);
			mBeacon2Left.setReferenceObject(mLeftConnector);
			mBeacon2Left.setLayer(1);
			mBeacon2Left.getLightArea().setLayer(-1);
			mBeacon2Left.getLightArea().setShapeColor(Color.RED);
			addSimObject(mBeacon2Left);

			mBeacon2Right = new Beacon("Beacon2Right", 0.0, 0.0, 135.0, 0.0, 20, 90.0, 1.0, 1.0);
			mBeacon2Right.setReferenceObject(mRightConnector);
			mBeacon2Right.setLayer(1);
			mBeacon2Right.getLightArea().setLayer(-1);
			mBeacon2Right.getLightArea().setShapeColor(Color.BLUE);
			addSimObject(mBeacon2Right);
			
			
			//Light sensors
			LightSensor leftLightSensor = new LightSensor("LeftLightSensor", 0.0, 0.0, 0.0, 0.0, 1.0, true);
			leftLightSensor.addLightSource(mBeaconLeft);
			leftLightSensor.setReferenceObject(robotBody);
			leftLightSensor.setShapeColor(Color.BLUE);
			leftLightSensor.setNoiseVariance(0.1);
			addSimObject(leftLightSensor);
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForLightSensor(leftLightSensor));
			
			LightSensor rightLightSensor = new LightSensor("ReftLightSensor", 0.0, 0.0, 0.0, 0.0, 1.0, true);
			rightLightSensor.addLightSource(mBeaconRight);
			rightLightSensor.setReferenceObject(robotBody);
			rightLightSensor.setShapeColor(Color.CYAN);
			rightLightSensor.setNoiseVariance(0.1);
			addSimObject(rightLightSensor);
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForLightSensor(rightLightSensor));
			
			LightSensor left2LightSensor = new LightSensor("Left2LightSensor", 0.0, 0.0, 0.0, 0.0, 1.0, true);
			left2LightSensor.addLightSource(mBeacon2Left);
			left2LightSensor.setReferenceObject(robotBody);
			left2LightSensor.setShapeColor(Color.BLUE);
			left2LightSensor.setNoiseVariance(0.1);
			addSimObject(left2LightSensor);
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForLightSensor(left2LightSensor));
			
			LightSensor right2LightSensor = new LightSensor("Reft2LightSensor", 0.0, 0.0, 0.0, 0.0, 1.0, true);
			right2LightSensor.addLightSource(mBeacon2Right);
			right2LightSensor.setReferenceObject(robotBody);
			right2LightSensor.setShapeColor(Color.CYAN);
			right2LightSensor.setNoiseVariance(0.1);
			addSimObject(right2LightSensor);
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForLightSensor(right2LightSensor));
			
			
			//Add motor neurons
			ValueManager vm = Simulator.getInstance().getValueManager();
			String name = "/SimObjects/" + mRobot.getName() + "/VelocityLeft";
			NeuronWrapper motorLeft = new NeuronWrapper(name, 0.0, mRobot.getMinVelocity(), mRobot.getMaxVelocity());
			vm.replaceValue(name, motorLeft);
			mMainController.addOutputNeuron(motorLeft);
			
			name = "/SimObjects/" + mRobot.getName() + "/VelocityRight";
			NeuronWrapper motorRight = new NeuronWrapper(name, 0.0, mRobot.getMinVelocity(), mRobot.getMaxVelocity());
			vm.replaceValue(name, motorRight);
			mMainController.addOutputNeuron(motorRight);
			
			
			SimpleBumpDetector bump = new SimpleBumpDetector("BumpObstacles", 0.0, 0.0, 0.0);
			Polygon p = robotBody.getShape();
			bump.setShape(new Polygon(p.xpoints, p.ypoints, p.npoints));
			bump.setReferenceObject(robotBody);
			bump.addDependencies(obstacleList);
			bump.addObstacleSet(obstacleList);
			addSimObject(bump);
			Simulator.getInstance().getScheduler().addTerminateTryValue(bump.getSensorValueObject());
			
			////Used in fitness function
			
			mTargetAreaLeftBumper = new SimpleBumpDetector("HitTargetLeft", 0.0, 0.0, 0.0);
			mTargetAreaLeftBumper.setReferenceObject(robotBody);
			mTargetAreaLeftBumper.addObstacle(mTargetAreaLeft);
			addSimObject(mTargetAreaLeftBumper);
//			mMainController.addBumpValue(mTargetAreaLeftBumper.getSensorValueObject());
			
			mTargetAreaRightBumper = new SimpleBumpDetector("HitTargetRight", 0.0, 0.0, 0.0);
			mTargetAreaRightBumper.setReferenceObject(robotBody);
			mTargetAreaRightBumper.addObstacle(mTargetAreaRight);
			addSimObject(mTargetAreaRightBumper);
//			mMainController.addBumpValue(mTargetAreaRightBumper.getSensorValueObject());
			
			mDistanceToLeftArea = new DistanceToPointScanner("DistanceToLeftArea",0.0, 0.0, 0.0, 0.0, 5000.0);
			mDistanceToLeftArea.addObstacle(mTargetAreaLeft);
			mDistanceToLeftArea.setReferenceObject(robotBody);
			addSimObject(mDistanceToLeftArea);
			
			mDistanceToRightArea = new DistanceToPointScanner("DistanceToRightArea",0.0, 0.0, 0.0, 0.0, 5000.0);
			mDistanceToRightArea.addObstacle(mTargetAreaRight);
			mDistanceToRightArea.setReferenceObject(robotBody);
			addSimObject(mDistanceToRightArea);
			
			mLightOnTheLeft = false;
			mCorrectTargetLeft = new NeuronWrapper("/FitnessInfo/TargetZoneLeft", 0.0, 0.0, 1.0);
			vm.addValue(mCorrectTargetLeft.getName(), mCorrectTargetLeft);
			
			mSecondLightOnTheLeft = false;
			mCorrectSecondTargetLeft = new NeuronWrapper("/FitnessInfo/SecondTargetZoneLeft", 0.0, 0.0, 1.0);
			vm.addValue(mCorrectSecondTargetLeft.getName(), mCorrectSecondTargetLeft);
			
			
			
			//Create and add Warp controller
			WarpController warpController = new WarpController();
			warpController.addDependency(mRobot);
			warpController.addDependency(bump);
			warpController.addDependency(mDistanceToLeftArea);
			warpController.addDependency(mDistanceToRightArea);
			warpController.addDependency(mTargetAreaLeftBumper);
			warpController.addDependency(mTargetAreaRightBumper);
			warpController.addDependency(leftLightSensor);
			warpController.addDependency(rightLightSensor);
			warpController.addDependency(left2LightSensor);
			warpController.addDependency(right2LightSensor);
			addSimObject(warpController);
		}
		
		/**
		 * The warp controller warps the robot back to the start postion when it reached the 
		 * correct target area. This simulates multiple sectors of concatenated T junctions
		 * without the need to build and simulate a huge maze.
		 * 
		 * @author Chris
		 *
		 */
		public class WarpController extends SimObject {

			private BooleanValue mBumped;
			private int mCurrentSector;
			
			public WarpController() {
				mBumped = new BooleanValue(false);
				Simulator.getInstance().getScheduler().addTerminateTryValue(mBumped);
			}
			
			public boolean init() {
				boolean ok = super.init();		
				if(mBumped.get()) {
					mBumped.set(false);
				}
				mCurrentSector = 0;
				return ok;
			}

			public void reset() {
				super.reset();
				if(mBumped.get()) {
					mBumped.set(false);
				}
				mCurrentSector = 0;
			}

			public boolean update() {
				boolean ok =  super.update();

				if(mCurrentSector % 2 == 0) {
					if(mCorrectTargetLeft.get() > 0.0) {
						if(mTargetAreaRightBumper.getSensorReading() > 0.0) {
							//bumped on wrong target area.
							mBumped.set(true);
						}
						else if(mTargetAreaLeftBumper.getSensorReading() > 0.0) {
							//bumped on the correct target area.
							mCurrentSector++;
							if(mCurrentSector >= Math.min(mNumberOfSectors, mMaxNumberOfSectors.get())) {
								mBumped.set(true); //End after max number of sectors
							}
							else {
								//Modify the current EvironmentModel object
								MyEnvironment env = (MyEnvironment) Simulator.getInstance().getScheduler().getCurrentEnvironment();
								
								env.mSectorStartAngle = mRobot.getAngle() - 90.0;
								env.mDesiredSector = mCurrentSector;
							}
						}
					}
					else {
						if(mTargetAreaLeftBumper.getSensorReading() > 0.0) {
							//bumped on wrong target area.
							mBumped.set(true);
						}
						else if(mTargetAreaRightBumper.getSensorReading() > 0.0) {
							//bumped on the correct target area.
							mCurrentSector++;
							if(mCurrentSector >= Math.min(mNumberOfSectors, mMaxNumberOfSectors.get())) {
								mBumped.set(true); //End after max number of sectors
							}
							else {
								//Modify the current EvironmentModel object
								MyEnvironment env = (MyEnvironment) Simulator.getInstance().getScheduler().getCurrentEnvironment();
								
								env.mSectorStartAngle = mRobot.getAngle() + 90.0;
								env.mDesiredSector = mCurrentSector;
							}
						}
					}
				}
				else {
					if(mCorrectSecondTargetLeft.get() > 0.0) {
						if(mTargetAreaRightBumper.getSensorReading() > 0.0) {
							//bumped on wrong target area.
							mBumped.set(true);
						}
						else if(mTargetAreaLeftBumper.getSensorReading() > 0.0) {
							//bumped on the correct target area.
							mCurrentSector++;
							if(mCurrentSector >= Math.min(mNumberOfSectors, mMaxNumberOfSectors.get())) {
								mBumped.set(true); //End after max number of sectors
							}
							else {
								//Modify the current EvironmentModel object
								MyEnvironment env = (MyEnvironment) Simulator.getInstance().getScheduler().getCurrentEnvironment();
								
								env.mSectorStartAngle = mRobot.getAngle() - 90.0;
								env.mDesiredSector = mCurrentSector;
							}
						}
					}
					else {
						if(mTargetAreaLeftBumper.getSensorReading() > 0.0) {
							//bumped on wrong target area.
							mBumped.set(true);
						}
						else if(mTargetAreaRightBumper.getSensorReading() > 0.0) {
							//bumped on the correct target area.
							mCurrentSector++;
							if(mCurrentSector >= Math.min(mNumberOfSectors, mMaxNumberOfSectors.get())) {
								mBumped.set(true); //End after max number of sectors
							}
							else {
								//Modify the current EvironmentModel object
								MyEnvironment env = (MyEnvironment) Simulator.getInstance().getScheduler().getCurrentEnvironment();
								
								env.mSectorStartAngle = mRobot.getAngle() + 90.0;
								env.mDesiredSector = mCurrentSector;
							}
						}
					}
				}
				return ok;
			}
			
		}
		
	}
	
	
	
	public static void main(String[] args) {
		new DelayResponseMazeMultiBeacon1(args);
	}
}
