/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.simulators;

import java.awt.Color;
import java.awt.Polygon;
import java.util.Iterator;
import java.util.Random;
import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.EnvironmentModel;
import de.fhg.iais.outdoor.simba.core.ParameterElement;
import de.fhg.iais.outdoor.simba.core.SimObject;
import de.fhg.iais.outdoor.simba.core.SimObjectSet;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.devices.Beacon;
import de.fhg.iais.outdoor.simba.devices.Box;
import de.fhg.iais.outdoor.simba.devices.DifferentialDriveUnit;
import de.fhg.iais.outdoor.simba.devices.DistanceScanner;
import de.fhg.iais.outdoor.simba.devices.DistanceToPointScanner;
import de.fhg.iais.outdoor.simba.devices.LightSensor;
import de.fhg.iais.outdoor.simba.devices.PathDrawer;
import de.fhg.iais.outdoor.simba.devices.SimpleBumpDetector;
import de.fhg.iais.outdoor.simba.fitnessfunctions.DelayedResponseMazeT1WarpCheckPoint2;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetAgentInterface;
import de.fhg.iais.outdoor.simba.neuralnet.NeuronWrapper;
import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.DoubleValue;
import de.fhg.iais.outdoor.simba.value.IntValue;
import de.fhg.iais.outdoor.simba.value.ValueManager;
import de.fhg.iais.outdoor.simba.visualization.SimbaDefaultFrame;

public class DelayResponzeMaze1TMiddleCorridor1 extends SimbaDefaultFrame {

	public NeuralNetAgentInterface mMainController;
	
	public DelayResponzeMaze1TMiddleCorridor1(String args[]) {
		super("DelayResponzeMaze1TMiddleCorridor1", 700, 700);
		
		setDefaultOffset(-660.0, -680.0);
		setDefaultZoom(0.5);
		setVisualizationInterval(1);
		setSimulationDelay(15);
		
		mMainController = Simulator.getInstance().getNeuralNetManager().getNeuralNetInterface();
		
		setFitnessFunction(new DelayedResponseMazeT1WarpCheckPoint2(), 10000.0);
		setMasterEnvironment(new MyEnvironment(10));
		
		mMainController.setNeuronRange(-1.0, 1.0);
	
		
		Simulator.getInstance().getNeuralNetManager().setCurrentNet(NeuralNet.createMinimalNet(mMainController));
		
		startSystem(mMainController, args);
	}
	
	

	
	public class MyEnvironment extends EnvironmentModel {
		
		//Arrays have to be cloned deeply in a custom clone() method.
		private int[] mOuterWidth;
		private int[] mInnerWidth;
		private int[] mOuterHeight;
		private int[] mInnerHeight;
		private double[] mRobotPositionX;
		private double[] mRobotPositionY;
		private double[] mInitAngle;
		private double[] mBeaconSize;
		private boolean[] mLightIsLeft;
		private double[] mBeaconOffset;
		private double[] mAngleBoardAngle;
		
		//SimObjects used in the environments (clones should use the identical objects (flat copy)
		private DifferentialDriveUnit mRobot;
		private SimObject mLowerT;
		private Box mUpperConnector;
		private Box mLeftConnector;
		private Box mRightConnector;
		private Box mTargetAreaLeft;
		private Box mTargetAreaRight;
		private Box mAngleBoard;
		private Beacon mBeaconLeft;
		private Beacon mBeaconRight;
		private DistanceToPointScanner mDistanceToLeftArea;
		private DistanceToPointScanner mDistanceToRightArea;
		private SimpleBumpDetector mTargetAreaLeftBumper;
		private SimpleBumpDetector mTargetAreaRightBumper;
		private boolean mLightOnTheLeft;
		private NeuronWrapper mCorrectTargetLeft;
		private PathDrawer mPathDrawer;
		private Box mLeftTrap;
		private Box mRightTrap;
		private BooleanValue mUseAngleBoard;
		private DoubleValue mTrapFloorWidth;
		private DoubleValue mScannerMinRange;
		private DoubleValue mScannerMaxRange;
		private DoubleValue mDistanceScannerNoise;
		private DistanceScanner mScanLeft;
		private DistanceScanner mScanRight;
		private DistanceScanner mScanMid;
		
		private Random mRandom = new Random();
		
		private IntValue mMaxNumberOfSectors;
		private int mNumberOfSectors;
		private int mCurrentSector;
		private int mDesiredSector;
		private IntValue mCurrentSectorValue;
		private double mSectorStartAngle;
		
		public MyEnvironment(int numberOfSectors) {
			super("DelayResponzeMaze1TMiddleCorridor1");
			mMaxNumberOfSectors = new IntValue(1);
			Simulator.getInstance().getValueManager().addValue("/Environment/NumberOfSectors", mMaxNumberOfSectors);
			mNumberOfSectors = numberOfSectors;
			
			mCurrentSectorValue = new IntValue();
			Simulator.getInstance().getValueManager().addValue("/Environment/CurrentSector", mCurrentSectorValue);
			
			mTrapFloorWidth = new DoubleValue(30.0);
			Simulator.getInstance().getValueManager().addValue("/Environment/TrapFloorWidth", mTrapFloorWidth);
			
			mDistanceScannerNoise = new DoubleValue(0.02);
			Simulator.getInstance().getValueManager().addValue("/Environment/DistanceScannerNoise", mDistanceScannerNoise);
			
			mScannerMaxRange = new DoubleValue(100.0);
			Simulator.getInstance().getValueManager().addValue("/Environment/ScannerMaxRange", mScannerMaxRange);
			
			mScannerMinRange = new DoubleValue(-100);
			Simulator.getInstance().getValueManager().addValue("/Environment/ScannerMinRange", mScannerMinRange);
			
			mUseAngleBoard = new BooleanValue(true);
			Simulator.getInstance().getValueManager().addValue("/Environment/UseAngleBoard", mUseAngleBoard);
			
			
			mOuterWidth = new int[numberOfSectors];
			mInnerWidth = new int[numberOfSectors];
			mOuterHeight = new int[numberOfSectors];
			mInnerHeight = new int[numberOfSectors];
			mRobotPositionX = new double[numberOfSectors];
			mRobotPositionY = new double[numberOfSectors];
			mInitAngle = new double[numberOfSectors];
			mBeaconSize = new double[numberOfSectors];
			mLightIsLeft = new boolean[numberOfSectors];
			mLightOnTheLeft = true;
			mBeaconOffset = new double[numberOfSectors];
			mAngleBoardAngle = new double[numberOfSectors];
		}
		
		
		public void toggleLightSource() {
			mLightOnTheLeft = !mLightOnTheLeft;
		}

		
		/**
		 * Overridden for this special case: allow sector changes at the begining
		 * of each step.
		 * 
		 * Updates the environment, then the actuators and finally the sensors.
		 */
		public void executeSimulationStep() {

			if(mDesiredSector != mCurrentSector) {
				mCurrentSector = mDesiredSector;
				mCurrentSectorValue.set(mCurrentSector); //only for observation purposes, is never used for control.
				changeEnvironmentToSector(mCurrentSector);
			}
			
			super.executeSimulationStep();
		}
		
		/**
		 * Overridden for this special case: each created variation environment 
		 * is available twice, each with the opposite light distribution.
		 * 
		 * Creates the given number of environment variation clones 
		 * based on this Environment object and returns all these
		 * EnvironmentModels in a Vector.
		 * 
		 * This version creates two scenarios for each randomization pattern,
		 * one for a light on the left and one for a light on the right.
		 * 
		 * @param numberOfEnvironments the number of EnvironmentModels to create.
		 * @return a Vector with the newly created EnvironmentModels.
		 */
		public Vector createEnvironments(int numberOfEnvironments) {
			
			if(numberOfEnvironments % 2 == 1) {
				numberOfEnvironments++;
			}
			Vector clones = super.createEnvironments(numberOfEnvironments / 2);
		
			Vector environments = new Vector();
			for(Iterator i = clones.iterator(); i.hasNext();) {
				MyEnvironment env = (MyEnvironment) i.next();
				if(env != null) {
					environments.add(env);
				
					MyEnvironment envClone = null;
					try {
						envClone = (MyEnvironment) env.clone();
						envClone.toggleLightSource();
					} catch (CloneNotSupportedException e) {
						e.printStackTrace();
					}
					if(envClone != null) {
						environments.add(envClone);
					}
				}
			}
			return environments;
		}
		
		
		

		protected Object clone() throws CloneNotSupportedException {
			//Clone the arrays manually.
			MyEnvironment obj = (MyEnvironment) super.clone();

			obj.mOuterWidth = new int[mNumberOfSectors];
			obj.mInnerWidth = new int[mNumberOfSectors];
			obj.mOuterHeight = new int[mNumberOfSectors];
			obj.mInnerHeight = new int[mNumberOfSectors];
			obj.mRobotPositionX = new double[mNumberOfSectors];
			obj.mRobotPositionY = new double[mNumberOfSectors];
			obj.mInitAngle = new double[mNumberOfSectors];
			obj.mBeaconSize = new double[mNumberOfSectors];
			obj.mLightIsLeft = new boolean[mNumberOfSectors];
			obj.mBeaconOffset = new double[mNumberOfSectors];
			obj.mAngleBoardAngle = new double[mNumberOfSectors];
			
			for(int i = 0; i < mNumberOfSectors; ++i) {
				obj.mOuterWidth[i] = mOuterWidth[i];
				obj.mInnerWidth[i] = mInnerWidth[i];
				obj.mOuterHeight[i] = mOuterHeight[i];
				obj.mInnerHeight[i] = mInnerHeight[i];
				obj.mRobotPositionX[i] = mRobotPositionX[i];
				obj.mRobotPositionY[i] = mRobotPositionY[i];
				obj.mInitAngle[i] = mInitAngle[i];
				obj.mBeaconSize[i] = mBeaconSize[i];
				obj.mLightIsLeft[i] = mLightIsLeft[i];
				obj.mBeaconOffset[i] = mBeaconOffset[i];
				obj.mAngleBoardAngle[i] = mAngleBoardAngle[i];
			}
			return obj;
		}


		public void updateRandomizationParameters() {
			
			for(int i = 0; i < mNumberOfSectors; ++i) {
				mOuterWidth[i] = 270 + (int) (250.0 * mRandom.nextDouble());
				mInnerWidth[i] = mOuterWidth[i] - 40;
				mOuterHeight[i] = 80 + (int) (40.0 * mRandom.nextDouble());;
				mInnerHeight[i] = mOuterHeight[i] - 40;
				mRobotPositionX[i] = 0.0; //((-1.0 * (mInnerHeight[i] - 40.0)) + (2.0 * mRandom.nextDouble() * (mInnerHeight[i] - 40.0)));
				mRobotPositionY[i] = 0.0 + (10.0 - (mRandom.nextDouble() * 20.0));
				mInitAngle[i] = 0.0; //-25.0 + (mRandom.nextDouble() * 50.0);
				mBeaconSize[i] = 90.0 - (mRandom.nextDouble() * 55.0);
				if(i == 0) {
					mLightIsLeft[i] = mRandom.nextBoolean();
				}
				else {
					mLightIsLeft[i] = !mLightIsLeft[i - 1];
				}
				mBeaconOffset[i] = (mRandom.nextDouble() - 0.5) * (((mOuterWidth[i] - mInnerHeight[i]) / 2.0) - 15.0);
				mAngleBoardAngle[i] = ((mRandom.nextDouble() - 0.5) * 30.0);
			}
			
		}
		
		public String createRandomizationParameterLog() {
			StringBuffer entry = new StringBuffer();
			entry.append(ParameterElement.getParamEntry("OuterWidth", mOuterWidth));
			entry.append(ParameterElement.getParamEntry("InnerWidth", mInnerWidth));
			entry.append(ParameterElement.getParamEntry("OuterHeight", mOuterHeight));
			entry.append(ParameterElement.getParamEntry("InnerHeight", mInnerHeight));
			entry.append(ParameterElement.getParamEntry("RobotPosX", mRobotPositionX));
			entry.append(ParameterElement.getParamEntry("RobotPosY", mRobotPositionY));
			entry.append(ParameterElement.getParamEntry("InitAngle", mInitAngle));
			entry.append(ParameterElement.getParamEntry("BeaconSize", mBeaconSize));
			entry.append(ParameterElement.getParamEntry("BeaconOffset", mBeaconOffset));
			entry.append(ParameterElement.getParamEntry("AngleBoardAngle", mAngleBoardAngle));
			entry.append(ParameterElement.getParamEntry("LightIsLeft", mLightIsLeft));
			entry.append(ParameterElement.getParamEntry("LightOnTheLeft", Boolean.toString(mLightOnTheLeft)));
			entry.append(ParameterElement.getParamEntry("MaxSectors", mMaxNumberOfSectors.getValueAsString()));
			entry.append(ParameterElement.getParamEntry("TrapFloorWidth", mTrapFloorWidth.getValueAsString()));
			entry.append(ParameterElement.getParamEntry("DistanceScannerNoise", mDistanceScannerNoise.getValueAsString()));
			entry.append(ParameterElement.getParamEntry("MinScannerRange", mScannerMinRange.getValueAsString()));
			entry.append(ParameterElement.getParamEntry("MaxScannerRange", mScannerMaxRange.getValueAsString()));
			entry.append(ParameterElement.getParamEntry("UseAngleBoard", mUseAngleBoard.getValueAsString()));
			
			return entry.toString();
		}
		

		public void resetEnvironmentObjects() {
			
			mDesiredSector = 0;
			mCurrentSector = 0;
			mCurrentSectorValue.set(0);
			changeEnvironmentToSector(0);
			
		}
		
		public void changeEnvironmentToSector(int sector) {
			
			if(sector == 0) {
				mSectorStartAngle = mInitAngle[0];
			}
			
			mPathDrawer.reset();
			
			mRobot.setPosition(mRobotPositionX[sector], mRobotPositionY[sector]);
			mRobot.setAngle(mSectorStartAngle);

			
			Polygon shape = new Polygon();
			shape.addPoint(-mInnerHeight[sector], mOuterHeight[sector]);
			shape.addPoint(-mInnerHeight[sector] - 30, mOuterHeight[sector]);
			shape.addPoint(-mInnerHeight[sector] - 30, -mOuterHeight[sector]);
			shape.addPoint(mInnerHeight[sector] + 30, -mOuterHeight[sector]);
			shape.addPoint(mInnerHeight[sector] + 30, mOuterHeight[sector]);
			
			shape.addPoint(mInnerHeight[sector], mOuterHeight[sector]);
			shape.addPoint(mInnerHeight[sector], mInnerHeight[sector]);
			shape.addPoint(mInnerHeight[sector], mInnerHeight[sector]);
			shape.addPoint(mInnerHeight[sector], -mInnerHeight[sector]);
			shape.addPoint(-mInnerHeight[sector], -mInnerHeight[sector]);
			shape.addPoint(-mInnerHeight[sector], mInnerHeight[sector]);
			
			shape.addPoint(-mInnerHeight[sector], mInnerHeight[sector]);
			
			mLowerT.setShape(new Polygon(shape.xpoints, shape.ypoints, shape.npoints));
			mUpperConnector.setPosition(0.0, mOuterHeight[sector] + mOuterWidth[sector] + mInnerHeight[sector] + ((mOuterHeight[sector] - mInnerHeight[sector]) / 2));
			mUpperConnector.setSize(mOuterWidth[sector] * 2, mOuterHeight[sector] - mInnerHeight[sector]);
			mLeftConnector.setPosition((mInnerHeight[sector] + ((mOuterWidth[sector] - mInnerHeight[sector]) / 2)) * -1, (mOuterHeight[sector] + ((mOuterWidth[sector] - mInnerHeight[sector]) / 2)));
			mLeftConnector.setSize(mOuterWidth[sector] - mInnerHeight[sector], mOuterWidth[sector] - mInnerHeight[sector]);
			mRightConnector.setPosition(mInnerHeight[sector] + ((mOuterWidth[sector] - mInnerHeight[sector]) / 2), (mOuterHeight[sector] + ((mOuterWidth[sector] - mInnerHeight[sector]) / 2)));
			mRightConnector.setSize(mOuterWidth[sector] - mInnerHeight[sector], mOuterWidth[sector] - mInnerHeight[sector]);
			mTargetAreaLeft.setSize((mOuterWidth[sector] - mInnerHeight[sector]), 2 * mInnerHeight[sector]);
			mTargetAreaLeft.setPosition(((mOuterWidth[sector] - mInnerHeight[sector]) + mInnerHeight[sector]) * -1, (mOuterWidth[sector] - mInnerHeight[sector]) +  mOuterHeight[sector] + mInnerHeight[sector]);
			mTargetAreaRight.setSize((mOuterWidth[sector] - mInnerHeight[sector]), 2 * mInnerHeight[sector]);
			mTargetAreaRight.setPosition(((mOuterWidth[sector] - mInnerHeight[sector]) + mInnerHeight[sector]), (mOuterWidth[sector] - mInnerHeight[sector]) +  mOuterHeight[sector] + mInnerHeight[sector]);
			mAngleBoard.setSize(mInnerHeight[sector] * 2, 40);
			mAngleBoard.setPosition(0.0, mOuterHeight[sector] + mOuterWidth[sector] + mInnerHeight[sector] + ((mOuterHeight[sector] - mInnerHeight[sector]) / 2) - 15);
			mAngleBoard.setAngle(mAngleBoardAngle[sector]);
			if(mUseAngleBoard.get()) {
				mAngleBoard.setActive(true);
			}
			else {
				mAngleBoard.setActive(false);
			}
			
			if((mTrapFloorWidth.get() / 2.0) > mInnerHeight[sector]) {
				mLeftTrap.setActive(false);
				mRightTrap.setActive(false);
			}
			else {
				mLeftTrap.setPosition((mTrapFloorWidth.get() / -2.0) + ((mInnerHeight[sector] - (mTrapFloorWidth.get() / 2.0)) / -2.0) - 1, (mOuterHeight[sector] + ((mOuterWidth[sector] - mInnerHeight[sector]) / 2)) - mInnerHeight[sector]);
				mLeftTrap.setSize(mInnerHeight[sector] - (mTrapFloorWidth.get() / 2.0) + 1, mOuterWidth[sector]);
				mLeftTrap.setActive(true);
				mRightTrap.setPosition((mTrapFloorWidth.get() / 2.0) + ((mInnerHeight[sector] - (mTrapFloorWidth.get() / 2.0)) / 2.0) + 1, (mOuterHeight[sector] + ((mOuterWidth[sector] - mInnerHeight[sector]) / 2)) - mInnerHeight[sector]);
				mRightTrap.setSize(mInnerHeight[sector] - (mTrapFloorWidth.get() / 2.0) + 1, mOuterWidth[sector]);
				mRightTrap.setActive(true);
			}
			
			boolean leftLight = mLightOnTheLeft ? mLightIsLeft[sector] : !mLightIsLeft[sector];
//			System.out.println("chageenv. to secotr: " + sector + " leftLIght " + leftLight
//					+ " mLightIsLeft[" + sector + "] " + mLightIsLeft[sector] + " mLightOnTheLeft " + mLightOnTheLeft + "Obj: " + this);
			
			mBeaconLeft.setPosition((mOuterWidth[sector] - mInnerHeight[sector]) / 2.0,  mBeaconOffset[sector] - 50.0);
			mBeaconLeft.getLightArea().setRadius(mInnerHeight[sector] * 2.0);
			mBeaconLeft.setInitialBrightness(leftLight ? 1.0 : 0.0);
			mBeaconLeft.getLightArea().setMaximalAngle(mBeaconSize[sector]);
			mBeaconLeft.getLightArea().setAngle(((90.0 - mBeaconSize[sector]) / 2.0));
			mBeaconRight.setPosition(-1 * ((mOuterWidth[sector] - mInnerHeight[sector]) / 2.0), mBeaconOffset[sector] - 50.0);
			mBeaconRight.getLightArea().setRadius(mInnerHeight[sector] * 2.0);
			mBeaconRight.setInitialBrightness(leftLight ? 0.0 : 1.0);
			mBeaconRight.getLightArea().setMaximalAngle(mBeaconSize[sector]);
			mBeaconRight.getLightArea().setAngle(((90.0 - mBeaconSize[sector]) / 2.0));
			
			mCorrectTargetLeft.set(leftLight ? 1.0 : 0.0);
			
			mDistanceToLeftArea.setVisible(leftLight);
			mDistanceToRightArea.setVisible(!leftLight);
			
			mScanLeft.setNoiseVariance(mDistanceScannerNoise.get());
			mScanLeft.setMinScanDistance(mScannerMinRange.get());
			mScanLeft.setMaxScanDistance(mScannerMaxRange.get());
			mScanRight.setNoiseVariance(mDistanceScannerNoise.get());
			mScanRight.setMinScanDistance(mScannerMinRange.get());
			mScanRight.setMaxScanDistance(mScannerMaxRange.get());
			mScanMid.setNoiseVariance(mDistanceScannerNoise.get());
			mScanMid.setMinScanDistance(mScannerMinRange.get());
			mScanMid.setMaxScanDistance(mScannerMaxRange.get());
		}

		public void createEnvironment() {

			//Set up scenario.
			
			mRobot = new DifferentialDriveUnit("Robot", 0.0, 10.0, 0.0, 20.0, 5.0, -1.5, 1.5);
			Box robotBody = new Box("Robot/Body", 0.0, 0.0, 90.0, 14.0, 20.0);
			robotBody.setReferenceObject(mRobot);
			robotBody.setShapeColor(Color.GREEN);
			addSimObject(mRobot);
			addSimObject(robotBody);
			
			mPathDrawer = new PathDrawer("RobotTrajectory", 0.0, 0.0, 2, 100, 1000);
			mPathDrawer.setReferenceObject(mRobot);
			mPathDrawer.setLayer(1);
			mPathDrawer.setShapeColor(Color.blue);
			mPathDrawer.setSecondColor(Color.cyan);
			addSimObject(mPathDrawer);
			
//			TextLabel label = new TextLabel("Name", 0.0, -10.0, 0.0, "HalloWelt", 10, true);
//			label.setReferenceObject(mRobot);
//			label.setLayer(100);
//			label.setShapeColor(Color.blue);
//			addSimObject(label);
			
			
			mLowerT = new SimObject("LowerT", 0.0, 0.0, 0.0, new Polygon());
			addSimObject(mLowerT);
	
			mUpperConnector = new Box("UpperConnector", 0.0, 0.0, 0.0, 0.0, 0.0);
			addSimObject(mUpperConnector);
			
			mLeftConnector = new Box("LeftConnector", 0.0, 0.0, 0.0, 0.0, 0.0);
			addSimObject(mLeftConnector);
			
			mRightConnector = new Box("RightConnector", 0.0, 0.0, 0.0, 0.0, 0.0);
			addSimObject(mRightConnector);
			
			mTargetAreaLeft = new Box("TargetAreaLeft", 0.0, 0.0, 0.0, 0.0, 0.0);
			mTargetAreaLeft.setShapeColor(Color.lightGray);
			mTargetAreaLeft.setLayer(-5);
			addSimObject(mTargetAreaLeft);
			
			mTargetAreaRight = new Box("TargetAreaRight", 0.0, 0.0, 0.0, 0.0, 0.0);
			mTargetAreaRight.setShapeColor(Color.lightGray);
			mTargetAreaRight.setLayer(-5);
			addSimObject(mTargetAreaRight);
			
			mAngleBoard = new Box("AngleBoard", 0.0, 0.0, 0.0, 0.0, 0.0);
			addSimObject(mAngleBoard);
			
			SimObjectSet obstacleList = new SimObjectSet();
			obstacleList.add(mLowerT);
			obstacleList.add(mLeftConnector);
			obstacleList.add(mRightConnector);
			obstacleList.add(mUpperConnector);
			obstacleList.add(mAngleBoard);
			
			mScanLeft = new DistanceScanner("DistanceScannerLeft", 0.0, 10.0, 60.0, -100.0, 100.0);
			mScanLeft.setReferenceObject(robotBody);
			mScanLeft.addDependencies(obstacleList);
			mScanLeft.addObstacleSet(obstacleList);
			mScanLeft.setNoiseVariance(1.0);
			addSimObject(mScanLeft);
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForDistanceScanner(mScanLeft));
			
			mScanRight = new DistanceScanner("DistanceScannerRight", 0.0, -10.0, -60.0, -100.0, 100.0);
			mScanRight.setReferenceObject(robotBody);
			mScanRight.addDependencies(obstacleList);
			mScanRight.addObstacleSet(obstacleList);
			mScanRight.setNoiseVariance(1.0);
			addSimObject(mScanRight);
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForDistanceScanner(mScanRight));
			
			mScanMid = new DistanceScanner("DistanceScannerFront", 7.0, 0.0, 0.0, -80.0, 80.0);
			mScanMid.setReferenceObject(robotBody);
			mScanMid.addObstacleSet(obstacleList);
			mScanMid.addObstacleSet(obstacleList);
			mScanMid.setNoiseVariance(1.0);
			addSimObject(mScanMid);
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForDistanceScanner(mScanMid));
			
			
			//Beacons
			mBeaconLeft = new Beacon("BeaconLeft", 0.0, 0.0, 315.0, 0.0, 20, 90.0, 1.0, 1.0);
			mBeaconLeft.setReferenceObject(mLeftConnector);
			mBeaconLeft.setLayer(1);
			mBeaconLeft.getLightArea().setLayer(-1);
			mBeaconLeft.getLightArea().setShapeColor(Color.ORANGE);
			addSimObject(mBeaconLeft);

			mBeaconRight = new Beacon("BeaconRight", 0.0, 0.0, 135.0, 0.0, 20, 90.0, 1.0, 1.0);
			mBeaconRight.setReferenceObject(mRightConnector);
			mBeaconRight.setLayer(1);
			mBeaconRight.getLightArea().setLayer(-1);
			mBeaconRight.getLightArea().setShapeColor(Color.GREEN);
			addSimObject(mBeaconRight);
			
			
			//Light sensors
			LightSensor leftLightSensor = new LightSensor("LeftLightSensor", 0.0, 0.0, 0.0, -1.0, 1.0, true);
			leftLightSensor.addLightSource(mBeaconLeft);
			leftLightSensor.setReferenceObject(robotBody);
			leftLightSensor.setShapeColor(Color.BLUE);
			leftLightSensor.setNoiseVariance(0.1);
			addSimObject(leftLightSensor);
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForLightSensor(leftLightSensor));
			
			LightSensor rightLightSensor = new LightSensor("ReftLightSensor", 0.0, 0.0, 0.0, -1.0, 1.0, true);
			rightLightSensor.addLightSource(mBeaconRight);
			rightLightSensor.setReferenceObject(robotBody);
			rightLightSensor.setShapeColor(Color.CYAN);
			rightLightSensor.setNoiseVariance(0.1);
			addSimObject(rightLightSensor);
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForLightSensor(rightLightSensor));
			
			//Add motor neurons
			ValueManager vm = Simulator.getInstance().getValueManager();
			String name = "/SimObjects/" + mRobot.getName() + "/VelocityLeft";
			NeuronWrapper motorLeft = new NeuronWrapper(name, 0.0, mRobot.getMinVelocity(), mRobot.getMaxVelocity());
			vm.replaceValue(name, motorLeft);
			mMainController.addOutputNeuron(motorLeft);
			
			name = "/SimObjects/" + mRobot.getName() + "/VelocityRight";
			NeuronWrapper motorRight = new NeuronWrapper(name, 0.0, mRobot.getMinVelocity(), mRobot.getMaxVelocity());
			vm.replaceValue(name, motorRight);
			mMainController.addOutputNeuron(motorRight);
			
			
			mLeftTrap = new Box("LeftTrap", 0.0, 0.0, 0.0, 0.0, 0.0);
			mLeftTrap.setLayer(-5);
			mLeftTrap.setShapeColor(Color.YELLOW);
			mRightTrap = new Box("RightTrap", 0.0, 0.0, 0.0, 0.0, 0.0);
			mRightTrap.setLayer(-5);
			mRightTrap.setShapeColor(Color.YELLOW);
			addSimObject(mLeftTrap);
			addSimObject(mRightTrap);
			
			SimpleBumpDetector bump = new SimpleBumpDetector("BumpObstacles", 0.0, 0.0, 0.0);
			Polygon p = robotBody.getShape();
			bump.setShape(new Polygon(p.xpoints, p.ypoints, p.npoints));
			bump.setReferenceObject(robotBody);
			bump.addDependencies(obstacleList);
			bump.addObstacleSet(obstacleList);
			bump.addObstacle(mLeftTrap);
			bump.addObstacle(mRightTrap);
			addSimObject(bump);
			Simulator.getInstance().getScheduler().addTerminateTryValue(bump.getSensorValueObject());
			
			////Used in fitness function
			
			mTargetAreaLeftBumper = new SimpleBumpDetector("HitTargetLeft", 0.0, 0.0, 0.0);
			mTargetAreaLeftBumper.setReferenceObject(robotBody);
			mTargetAreaLeftBumper.addObstacle(mTargetAreaLeft);
			addSimObject(mTargetAreaLeftBumper);
//			mMainController.addBumpValue(mTargetAreaLeftBumper.getSensorValueObject());
			
			mTargetAreaRightBumper = new SimpleBumpDetector("HitTargetRight", 0.0, 0.0, 0.0);
			mTargetAreaRightBumper.setReferenceObject(robotBody);
			mTargetAreaRightBumper.addObstacle(mTargetAreaRight);
			addSimObject(mTargetAreaRightBumper);
//			mMainController.addBumpValue(mTargetAreaRightBumper.getSensorValueObject());
			
			mDistanceToLeftArea = new DistanceToPointScanner("DistanceToLeftArea",0.0, 0.0, 0.0, 0.0, 5000.0);
			mDistanceToLeftArea.addObstacle(mTargetAreaLeft);
			mDistanceToLeftArea.setReferenceObject(robotBody);
			addSimObject(mDistanceToLeftArea);
			
			mDistanceToRightArea = new DistanceToPointScanner("DistanceToRightArea",0.0, 0.0, 0.0, 0.0, 5000.0);
			mDistanceToRightArea.addObstacle(mTargetAreaRight);
			mDistanceToRightArea.setReferenceObject(robotBody);
			addSimObject(mDistanceToRightArea);
			
			mLightOnTheLeft = false;
			mCorrectTargetLeft = new NeuronWrapper("/FitnessInfo/TargetZoneLeft", 0.0, 0.0, 1.0);
			vm.addValue(mCorrectTargetLeft.getName(), mCorrectTargetLeft);
				
			
			//Create and add Warp controller
			WarpController warpController = new WarpController();
			warpController.addDependency(mRobot);
			warpController.addDependency(bump);
			warpController.addDependency(mDistanceToLeftArea);
			warpController.addDependency(mDistanceToRightArea);
			warpController.addDependency(mTargetAreaLeftBumper);
			warpController.addDependency(mTargetAreaRightBumper);
			warpController.addDependency(leftLightSensor);
			warpController.addDependency(rightLightSensor);
			addSimObject(warpController);
		}
		
		/**
		 * The warp controller warps the robot back to the start postion when it reached the 
		 * correct target area. This simulates multiple sectors of concatenated T junctions
		 * without the need to build and simulate a huge maze.
		 * 
		 * @author Chris
		 *
		 */
		public class WarpController extends SimObject {

			private BooleanValue mBumped;
			private int mCurrentSector;
			
			public WarpController() {
				mBumped = new BooleanValue(false);
				Simulator.getInstance().getScheduler().addTerminateTryValue(mBumped);
			}
			
			public boolean init() {
				boolean ok = super.init();		
				if(mBumped.get()) {
					mBumped.set(false);
				}
				mCurrentSector = 0;
				return ok;
			}

			public void reset() {
				super.reset();
				if(mBumped.get()) {
					mBumped.set(false);
				}
				mCurrentSector = 0;
			}

			public boolean update() {
				boolean ok =  super.update();

				if(mCorrectTargetLeft.get() > 0.0) {
					if(mTargetAreaRightBumper.getSensorReading() > 0.0) {
						//bumped on wrong target area.
						mBumped.set(true);
					}
					else if(mTargetAreaLeftBumper.getSensorReading() > 0.0) {
						//bumped on the correct target area.
						mCurrentSector++;
						if(mCurrentSector >= Math.min(mNumberOfSectors, mMaxNumberOfSectors.get())) {
							mBumped.set(true); //End after max number of sectors
						}
						else {
							//Modify the current EvironmentModel object
							MyEnvironment env = (MyEnvironment) Simulator.getInstance().getScheduler().getCurrentEnvironment();
							
							env.mSectorStartAngle = mRobot.getAngle() - 90.0;
							env.mDesiredSector = mCurrentSector;
						}
					}
				}
				else {
					if(mTargetAreaLeftBumper.getSensorReading() > 0.0) {
						//bumped on wrong target area.
						mBumped.set(true);
					}
					else if(mTargetAreaRightBumper.getSensorReading() > 0.0) {
						//bumped on the correct target area.
						mCurrentSector++;
						if(mCurrentSector >= Math.min(mNumberOfSectors, mMaxNumberOfSectors.get())) {
							mBumped.set(true); //End after max number of sectors
						}
						else {
							//Modify the current EvironmentModel object
							MyEnvironment env = (MyEnvironment) Simulator.getInstance().getScheduler().getCurrentEnvironment();
							
							env.mSectorStartAngle = mRobot.getAngle() + 90.0;
							env.mDesiredSector = mCurrentSector;
						}
					}
				}
				return ok;
			}
			
		}
		
	}
	
	
	
	public static void main(String[] args) {
		new DelayResponzeMaze1TMiddleCorridor1(args);
	}
	
}
