/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.simulators;

import java.awt.Color;
import java.awt.Polygon;
import java.util.Random;

import de.fhg.iais.outdoor.simba.core.EnvironmentModel;
import de.fhg.iais.outdoor.simba.core.SimObject;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.devices.Box;
import de.fhg.iais.outdoor.simba.devices.DifferentialDriveUnit;
import de.fhg.iais.outdoor.simba.devices.DistanceScanner;
import de.fhg.iais.outdoor.simba.devices.SimpleBumpDetector;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetAgentInterface;
import de.fhg.iais.outdoor.simba.neuralnet.NeuronWrapper;
import de.fhg.iais.outdoor.simba.value.ValueManager;
import de.fhg.iais.outdoor.simba.visualization.SimbaDefaultFrame;


public class SimpleMazeScenario extends SimbaDefaultFrame {

	public NeuralNetAgentInterface mMainController;
	
	public SimpleMazeScenario(String args[]) {
		super("TestMazeScenario2", 700, 500);
		
		setDefaultOffset(-300.0, -400.0);
		setDefaultZoom(0.5);
		setVisualizationInterval(1);
		setSimulationDelay(15);
		
		mMainController = new NeuralNetAgentInterface("Maze1Controller");
		
		setMasterEnvironment(new MyEnvironment());
		Simulator.getInstance().init();
		mMainController.setNeuronRange(-1.0, 1.0);
		
		startSystem(mMainController, args);
	}
	
	
	
	
	//SimObjects used in the environments. 
	private DifferentialDriveUnit mRobot;
	private SimObject mLowerT;
	private SimObject mRightT;
	private SimObject mLeftT;
	private Box mUpperConnector;
	private Box mLeftConnector;
	private Box mRightConnector;

	
	public class MyEnvironment extends EnvironmentModel {
		
		private int mOuterWidth;
		private int mInnerWidth;
		private int mOuterHeight;
		private int mInnerHeight;
		private double mRobotPositionX;
		private double mRobotPositionY;
		private double mInitAngle;
		
		public MyEnvironment() {
			super("SimpleMazeScenario");
		}

		public void updateRandomizationParameters() {
			
			Random rand = new Random();
			
			mOuterWidth = 200 + (int) (50.0 * rand.nextDouble());
			mInnerWidth = mOuterWidth - 20;
			mOuterHeight = 70 + (int) (50.0 * rand.nextDouble());;
			mInnerHeight = mOuterHeight - 20;
			mRobotPositionX = 0.0 + (10.0 - (rand.nextFloat() * 20.0));
			mRobotPositionY = 0.0 + (10.0 - (rand.nextFloat() * 20.0));
			mInitAngle = 0.0 + (10.0 - (rand.nextFloat() * 20.0));
			
		}
		

		public void resetEnvironmentObjects() {
			mRobot.setPosition(mRobotPositionX, mRobotPositionY);
			mRobot.setAngle(mInitAngle);
			
			Polygon shape = new Polygon();
			shape.addPoint(-mInnerHeight, mOuterHeight);
			shape.addPoint(-mOuterWidth, mOuterHeight);
			shape.addPoint(-mOuterWidth, -mOuterHeight);
			shape.addPoint(mOuterWidth, -mOuterHeight);
			shape.addPoint(mOuterWidth, mOuterHeight);
			shape.addPoint(mInnerHeight, mOuterHeight);
			shape.addPoint(mInnerHeight, mInnerHeight);
			shape.addPoint(mInnerWidth, mInnerHeight);
			shape.addPoint(mInnerWidth, -mInnerHeight);
			shape.addPoint(-mInnerWidth, -mInnerHeight);
			shape.addPoint(-mInnerWidth, mInnerHeight);
			shape.addPoint(-mInnerHeight, mInnerHeight);
			
			mLowerT.setShape(new Polygon(shape.xpoints, shape.ypoints, shape.npoints));
			mRightT.setPosition(mOuterWidth + mOuterHeight, mOuterHeight + mOuterWidth);
			mRightT.setShape(new Polygon(shape.xpoints, shape.ypoints, shape.npoints));
			mLeftT.setPosition(-1 * (mOuterWidth + mOuterHeight - 1), mOuterHeight + mOuterWidth);
			mLeftT.setShape(new Polygon(shape.xpoints, shape.ypoints, shape.npoints));
			mUpperConnector.setPosition(0.0, mOuterHeight + mOuterWidth + mInnerHeight + ((mOuterHeight - mInnerHeight) / 2));
			mUpperConnector.setWidth(mOuterWidth * 2);
			mUpperConnector.setHeight(mOuterHeight - mInnerHeight);
			mLeftConnector.setPosition(mInnerHeight + ((mOuterWidth - mInnerHeight) / 2), (mOuterHeight + ((mOuterWidth - mInnerHeight) / 2)));
			mLeftConnector.setWidth(mOuterWidth - mInnerHeight);
			mLeftConnector.setHeight(mOuterWidth - mInnerHeight);
			mRightConnector.setPosition((mInnerHeight + ((mOuterWidth - mInnerHeight) / 2)) * -1, (mOuterHeight + ((mOuterWidth - mInnerHeight) / 2)));
			mRightConnector.setWidth(mOuterWidth - mInnerHeight);
			mRightConnector.setHeight(mOuterWidth - mInnerHeight);
			
		}

		public void createEnvironment() {

			//Set up scenario.
			
			mRobot = new DifferentialDriveUnit("Robot", 0.0, 10.0, 0.0, 20.0, 5.0, -1.5, 1.5);
			Box robotBody = new Box("Robot/Body", 0.0, 0.0, 90.0, 14.0, 20.0);
			robotBody.setReferenceObject(mRobot);
			robotBody.setShapeColor(Color.GREEN);
			addSimObject(mRobot);
			addSimObject(robotBody);
			
			
			mLowerT = new SimObject("LowerT", 0.0, 0.0, 0.0, new Polygon());
			addSimObject(mLowerT);
			
			mRightT = new SimObject("mRightT", 0.0, 0.0, 90.0, new Polygon());
			addSimObject(mRightT);
			
			mLeftT = new SimObject("LeftT", 0.0, 0.0, -90.0, new Polygon());
			addSimObject(mLeftT);
	
			mUpperConnector = new Box("UpperConnector", 0.0, 0.0, 0.0, 0.0, 0.0);
			addSimObject(mUpperConnector);
			
			mLeftConnector = new Box("LeftConnector", 0.0, 0.0, 0.0, 0.0, 0.0);
			addSimObject(mLeftConnector);
			
			mRightConnector = new Box("RightConnector", 0.0, 0.0, 0.0, 0.0, 0.0);
			addSimObject(mRightConnector);
			
			DistanceScanner scanLeft = new DistanceScanner("DistanceScannerLeft", 0.0, 10.0, 60.0, 0.0, 50.0);
			scanLeft.setReferenceObject(robotBody);
			scanLeft.addObstacle(mLowerT);
			scanLeft.addObstacle(mLeftT);
			scanLeft.addObstacle(mRightT);
			scanLeft.addObstacle(mUpperConnector);
			scanLeft.addObstacle(mLeftConnector);
			scanLeft.addObstacle(mRightConnector);
			addSimObject(scanLeft);
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForDistanceScanner(scanLeft));
			
			DistanceScanner scanRight = new DistanceScanner("DistanceScannerRight", 0.0, -10.0, -60.0, 0.0, 50.0);
			scanRight.setReferenceObject(robotBody);
			scanRight.addObstacle(mLowerT);
			scanRight.addObstacle(mLeftT);
			scanRight.addObstacle(mRightT);
			scanRight.addObstacle(mUpperConnector);
			scanRight.addObstacle(mLeftConnector);
			scanRight.addObstacle(mRightConnector);
			addSimObject(scanRight);
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForDistanceScanner(scanRight));
			
			DistanceScanner scanMid = new DistanceScanner("DistanceScannerFront", 7.0, 0.0, 0.0, 0.0, 40.0);
			scanMid.setReferenceObject(robotBody);
			scanMid.addObstacle(mLowerT);
			scanMid.addObstacle(mLeftT);
			scanMid.addObstacle(mRightT);
			scanMid.addObstacle(mUpperConnector);
			scanMid.addObstacle(mLeftConnector);
			scanMid.addObstacle(mRightConnector);
			addSimObject(scanMid);
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForDistanceScanner(scanMid));
			
			
			//Add motor neurons
			ValueManager vm = Simulator.getInstance().getValueManager();
			String name = "/SimObjects/" + mRobot.getName() + "/VelocityLeft";
			NeuronWrapper motorLeft = new NeuronWrapper(name, 0.0, mRobot.getMinVelocity(), mRobot.getMaxVelocity());
			vm.replaceValue(name, motorLeft);
			mMainController.addOutputNeuron(motorLeft);
			name = "/SimObjects/" + mRobot.getName() + "/VelocityRight";
			NeuronWrapper motorRight = new NeuronWrapper(name, 0.0, mRobot.getMinVelocity(), mRobot.getMaxVelocity());
			vm.replaceValue(name, motorRight);
			mMainController.addOutputNeuron(motorRight);
			
			
			SimpleBumpDetector bump = new SimpleBumpDetector("BumpRight", 0.0, 0.0, 0.0);
			Polygon p = robotBody.getShape();
			bump.setShape(new Polygon(p.xpoints, p.ypoints, p.npoints));
			bump.setReferenceObject(robotBody);
			bump.setReferenceObject(robotBody);
			bump.addObstacle(mLowerT);
			bump.addObstacle(mLeftT);
			bump.addObstacle(mRightT);
			bump.addObstacle(mUpperConnector);
			bump.addObstacle(mLeftConnector);
			bump.addObstacle(mRightConnector);
			addSimObject(bump);
			Simulator.getInstance().getScheduler().addTerminateTryValue(bump.getSensorValueObject());
			
		
			
		}
	}
	
	public static void main(String[] args) {
		new SimpleMazeScenario(args);
	}
}
