/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.simulators;

import java.awt.Color;
import java.awt.Polygon;
import java.util.Random;

import de.fhg.iais.outdoor.simba.core.EnvironmentModel;
import de.fhg.iais.outdoor.simba.core.SimObject;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.devices.Beacon;
import de.fhg.iais.outdoor.simba.devices.Box;
import de.fhg.iais.outdoor.simba.devices.DifferentialDriveUnit;
import de.fhg.iais.outdoor.simba.devices.DistanceScanner;
import de.fhg.iais.outdoor.simba.devices.LightSensor;
import de.fhg.iais.outdoor.simba.devices.SimpleBumpDetector;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetAgentInterface;
import de.fhg.iais.outdoor.simba.neuralnet.NeuronWrapper;
import de.fhg.iais.outdoor.simba.visualization.SimbaDefaultFrame;

public class TestMazeScenario2 extends SimbaDefaultFrame {

	NeuralNetAgentInterface mMainController;
	
	TestMazeScenario2(String args[]) {
		super("TestMazeScenario2", 700, 700);
		
		setDefaultZoom(0.6);
		setDefaultOffset(-580.0, -580.0);
		setVisualizationInterval(1);
		setSimulationDelay(15);
		
		mMainController = new NeuralNetAgentInterface("Maze1Controller");
		
		setMasterEnvironment(new MyEnvironment());
		Simulator.getInstance().init();
		
		startSystem(mMainController, args);
	}
	
	
	
	
	
	
	public class MyEnvironment extends EnvironmentModel {
		
		//Randomization Parameters
		private int mOuterWidth;
		private int mInnerWidth;
		private int mOuterHeight;
		private int mInnerHeight;
		private double mRobotPositionX;
		private double mRobotPositionY;
		private double mInitAngle;
		
//		SimObjects used in the environments. 
		private DifferentialDriveUnit mRobot;
		private SimObject mLowerT;
		private SimObject mRightT;
		private SimObject mLeftT;
		private Box mUpperConnector;
		private Box mLeftConnector;
		private Box mRightConnector;
		private Beacon mBeacon1;
		private Beacon mBeacon2;
		private Beacon mBeacon3;
		private Beacon mBeacon4;
		
		
		public MyEnvironment() {
			super("TestMazeScenario2");
		}
		
		public void  updateRandomizationParameters() {
			
			Random rand = new Random();
			
			mOuterWidth = 200 + (int) (50.0 * rand.nextDouble());
			mInnerWidth = mOuterWidth - 20;
			mOuterHeight = 70 + (int) (50.0 * rand.nextDouble());;
			mInnerHeight = mOuterHeight - 20;
			mRobotPositionX = 0.0 + (10.0 - (rand.nextFloat() * 20.0));
			mRobotPositionY = 0.0 + (10.0 - (rand.nextFloat() * 20.0));
			mInitAngle = 0.0 + (10.0 - (rand.nextFloat() * 20.0));
			
//			System.out.println("CalculatedRandomParameters");
		}
		

		public void resetEnvironmentObjects() {
			mRobot.setPosition(mRobotPositionX, mRobotPositionY);
			mRobot.setAngle(mInitAngle);
			
			Polygon shape = new Polygon();
			shape.addPoint(-mInnerHeight, mOuterHeight);
			shape.addPoint(-mOuterWidth, mOuterHeight);
			shape.addPoint(-mOuterWidth, -mOuterHeight);
			shape.addPoint(mOuterWidth, -mOuterHeight);
			shape.addPoint(mOuterWidth, mOuterHeight);
			shape.addPoint(mInnerHeight, mOuterHeight);
			shape.addPoint(mInnerHeight, mInnerHeight);
			shape.addPoint(mInnerWidth, mInnerHeight);
			shape.addPoint(mInnerWidth, -mInnerHeight);
			shape.addPoint(-mInnerWidth, -mInnerHeight);
			shape.addPoint(-mInnerWidth, mInnerHeight);
			shape.addPoint(-mInnerHeight, mInnerHeight);
			
			mLowerT.setShape(new Polygon(shape.xpoints, shape.ypoints, shape.npoints));
			mRightT.setPosition(mOuterWidth + mOuterHeight, mOuterHeight + mOuterWidth);
			mRightT.setShape(new Polygon(shape.xpoints, shape.ypoints, shape.npoints));
			mLeftT.setPosition(-1 * (mOuterWidth + mOuterHeight - 1), mOuterHeight + mOuterWidth);
			mLeftT.setShape(new Polygon(shape.xpoints, shape.ypoints, shape.npoints));
			mUpperConnector.setPosition(0.0, mOuterHeight + mOuterWidth + mInnerHeight + ((mOuterHeight - mInnerHeight) / 2));
			mUpperConnector.setWidth(mOuterWidth * 2);
			mUpperConnector.setHeight(mOuterHeight - mInnerHeight);
			mLeftConnector.setPosition(mInnerHeight + ((mOuterWidth - mInnerHeight) / 2), (mOuterHeight + ((mOuterWidth - mInnerHeight) / 2)));
			mLeftConnector.setWidth(mOuterWidth - mInnerHeight);
			mLeftConnector.setHeight(mOuterWidth - mInnerHeight);
			mRightConnector.setPosition((mInnerHeight + ((mOuterWidth - mInnerHeight) / 2)) * -1, (mOuterHeight + ((mOuterWidth - mInnerHeight) / 2)));
			mRightConnector.setWidth(mOuterWidth - mInnerHeight);
			mRightConnector.setHeight(mOuterWidth - mInnerHeight);
			
			mBeacon1.setPosition(-1 * ((mOuterWidth - mInnerHeight) / 2.0), 0.0);
			mBeacon1.getLightArea().setRadius(mInnerHeight * 2.0);
			mBeacon2.setPosition((mOuterWidth - mInnerHeight) / 2.0, 0.0);
			mBeacon2.getLightArea().setRadius(mInnerHeight * 2.0);
			mBeacon3.setPosition(-20.0, (mOuterWidth - mInnerHeight) / 2.0);
			mBeacon3.getLightArea().setRadius(mInnerHeight * 2.0);
			mBeacon4.setPosition(((mOuterWidth - mInnerHeight) / 2.0) + mInnerHeight, (mOuterHeight - mInnerHeight) / -2.0);
			mBeacon4.getLightArea().setRadius(mInnerHeight * 2.0);
		}


		public void createEnvironment() {

			//Set up scenario.
			
			
			mRobot = new DifferentialDriveUnit("Robot", 0.0, 10.0, 0.0, 20.0, 5.0, -1.5, 1.5);
			Box robotBody = new Box("Robot/Body", 0.0, 0.0, 90.0, 14.0, 20.0);
			robotBody.setReferenceObject(mRobot);
			robotBody.setShapeColor(Color.GREEN);
			robotBody.setLayer(20);
			addSimObject(mRobot);
			addSimObject(robotBody);
			
			
			mLowerT = new SimObject("LowerT", 0.0, 0.0, 0.0, new Polygon());
			addSimObject(mLowerT);
			
			mRightT = new SimObject("mRightT", 0.0, 0.0, 90.0, new Polygon());
			addSimObject(mRightT);
			
			mLeftT = new SimObject("LeftT", 0.0, 0.0, -90.0, new Polygon());
			addSimObject(mLeftT);
	
			mUpperConnector = new Box("UpperConnector", 0.0, 0.0, 0.0, 0.0, 0.0);
			addSimObject(mUpperConnector);
			
			mLeftConnector = new Box("LeftConnector", 0.0, 0.0, 0.0, 0.0, 0.0);
			addSimObject(mLeftConnector);
			
			mRightConnector = new Box("RightConnector", 0.0, 0.0, 0.0, 0.0, 0.0);
			addSimObject(mRightConnector);
			
			DistanceScanner scanLeft = new DistanceScanner("DistanceScannerLeft", 0.0, 10.0, 60.0, 0.0, 50.0);
			scanLeft.setReferenceObject(robotBody);
			scanLeft.addObstacle(mLowerT);
			scanLeft.addObstacle(mLeftT);
			scanLeft.addObstacle(mRightT);
			scanLeft.addObstacle(mUpperConnector);
			scanLeft.addObstacle(mLeftConnector);
			scanLeft.addObstacle(mRightConnector);
			addSimObject(scanLeft);
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForDistanceScanner(scanLeft));
			
			DistanceScanner scanRight = new DistanceScanner("DistanceScannerRight", 0.0, -10.0, -60.0, 0.0, 50.0);
			scanRight.setReferenceObject(robotBody);
			scanRight.addObstacle(mLowerT);
			scanRight.addObstacle(mLeftT);
			scanRight.addObstacle(mRightT);
			scanRight.addObstacle(mUpperConnector);
			scanRight.addObstacle(mLeftConnector);
			scanRight.addObstacle(mRightConnector);
			addSimObject(scanRight);
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForDistanceScanner(scanRight));
			
			DistanceScanner scanMid = new DistanceScanner("DistanceScannerFront", 7.0, 0.0, 0.0, 0.0, 40.0);
			scanMid.setReferenceObject(robotBody);
			scanMid.addObstacle(mLowerT);
			scanMid.addObstacle(mLeftT);
			scanMid.addObstacle(mRightT);
			scanMid.addObstacle(mUpperConnector);
			scanMid.addObstacle(mLeftConnector);
			scanMid.addObstacle(mRightConnector);
			addSimObject(scanMid);
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForDistanceScanner(scanMid));
			
			
			
			RobotController controller = new RobotController(2.0);
			controller.setReferenceObject(mRobot);
			controller.addDependency(robotBody);
			controller.addDependency(scanLeft);
			controller.addDependency(scanRight);
			controller.addDependency(scanMid);
			addSimObject(controller);
			
			
			
			SimpleBumpDetector bump = new SimpleBumpDetector("BumpRight", 0.0, 0.0, 0.0);
			Polygon p = robotBody.getShape();
			bump.setShape(new Polygon(p.xpoints, p.ypoints, p.npoints));
			bump.setReferenceObject(robotBody);
			bump.setReferenceObject(robotBody);
			bump.addObstacle(mLowerT);
			bump.addObstacle(mLeftT);
			bump.addObstacle(mRightT);
			bump.addObstacle(mUpperConnector);
			bump.addObstacle(mLeftConnector);
			bump.addObstacle(mRightConnector);
			addSimObject(bump);
			Simulator.getInstance().getScheduler().addTerminateTryValue(bump.getSensorValueObject());
			
			mBeacon1 = new Beacon("Beacon1", 0.0, 0.0, 135.0, 0.0, 20, 90.0, 0.02, 0.00);
			mBeacon1.setReferenceObject(mLeftConnector);
			mBeacon1.setLayer(5);
			mBeacon1.getLightArea().setLayer(4);
			int[] sequence = new int[1];
			sequence[0] = 200;
			mBeacon1.setBlinkPattern(sequence);
			mBeacon1.getLightArea().setShapeColor(Color.ORANGE);
			addSimObject(mBeacon1);
			
			mBeacon2 = new Beacon("Beacon", 0.0, 0.0, 315.0, 0.0, 20, 90.0, 0.02, 1.0);
			mBeacon2.setReferenceObject(mRightConnector);
			mBeacon2.setLayer(5);
			mBeacon2.getLightArea().setLayer(4);
			mBeacon2.setBlinkPattern(sequence);
			addSimObject(mBeacon2);
			
			mBeacon3 = new Beacon("Beacon3", -20.0, 0.0, 45.0, 0.0, 20, 90.0, 0.9, 0.00);
			int[] sequence2 = new int[1];
			sequence2[0] = 150;
			mBeacon3.setReferenceObject(mLeftConnector);
			mBeacon3.setLayer(5);
			mBeacon3.getLightArea().setLayer(4);
			mBeacon3.setBlinkPattern(sequence2);
			mBeacon3.getLightArea().setShapeColor(Color.BLUE);
			addSimObject(mBeacon3);
			
			mBeacon4 = new Beacon("Beacon4", 0.0, 0.0, 225.0, 0.0, 20, 90.0, 0.9, 1.0);
			mBeacon4.setBlinkPattern(sequence2);
			mBeacon4.setReferenceObject(mUpperConnector);
			mBeacon4.setLayer(5);
			mBeacon4.getLightArea().setLayer(4);
			mBeacon4.getLightArea().setShapeColor(Color.cyan);
			addSimObject(mBeacon4);
			
			LightSensor lightSensor1 = new LightSensor("LightSensor1", 0.0, 0.0, 0.0, 0.0, 1.0);
			lightSensor1.addLightSource(mBeacon1);
			lightSensor1.addLightSource(mBeacon3);
			lightSensor1.setReferenceObject(robotBody);
			addSimObject(lightSensor1);
			
			
		}
	}
	
	
	public class RobotController extends SimObject {
		
		private DistanceScanner mDistScannerLeft;
		private DistanceScanner mDistScannerRight;
		private DistanceScanner mDistScannerMid;
		private DifferentialDriveUnit mDrive;
		private int mCounter;
		
		private double mVelocity;
		
		public RobotController(double velocity) {
			mVelocity = velocity;
			mCounter = 0;
		}
		
		public boolean init() {
			boolean ok = true;
			
			SimObject obj1 = Simulator.getInstance().getValueManager().getSimObjectValue("/SimObjects/DistanceScannerLeft").get();
			SimObject obj2 = Simulator.getInstance().getValueManager().getSimObjectValue("/SimObjects/DistanceScannerRight").get();
			SimObject obj3 = Simulator.getInstance().getValueManager().getSimObjectValue("/SimObjects/DistanceScannerFront").get();
			SimObject obj4 = Simulator.getInstance().getValueManager().getSimObjectValue("/SimObjects/Robot").get();

			
			if(obj1 ==null || obj2 == null || obj3 == null || obj4 == null ||
				!(obj1 instanceof DistanceScanner) || !(obj2 instanceof DistanceScanner)
				|| !(obj3 instanceof DistanceScanner) || !(obj4 instanceof DifferentialDriveUnit)) 
			{
				ok = false;
			}
			else {
				mDistScannerLeft = (DistanceScanner) obj1;
				mDistScannerRight = (DistanceScanner) obj2;
				mDistScannerMid = (DistanceScanner) obj3;
				mDrive = (DifferentialDriveUnit) obj4;
			}
			return ok;
		}
		
		public boolean cleanUp() {
			return true;
		}
		
		public boolean update() {
			double left = mVelocity;
			double right = mVelocity;
			
			if(mCounter > 0) {
				mCounter--;
				return true;
			}
			
			if((mDistScannerLeft.getSensorReading() <= 30.0 
					|| mDistScannerRight.getSensorReading() <= 30.0))
			{
				if(mDistScannerLeft.getSensorReading() - mDistScannerRight.getSensorReading() < 0.0)
				{
					right *= -1;
				}
				else {
					left *= -1;
				}
			}
			if(mDistScannerMid.getSensorReading() <= 10.0) {
				if(Math.abs(mDistScannerLeft.getSensorReading() - mDistScannerRight.getSensorReading()) <= 1.0)
				{
					right *= -1;
					mCounter = 5;
				}
			}
			mDrive.setLeftVelocity(left);
			mDrive.setRightVelocity(right);
			
			return true;
		}
	}
	
	public static void main(String[] args) {
		new TestMazeScenario2(args);
	}
}
