/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.simulators;

import java.awt.Color;
import java.awt.Polygon;
import java.util.Iterator;
import java.util.Random;
import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.EnvironmentModel;
import de.fhg.iais.outdoor.simba.core.SimObjectSet;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.devices.Beacon;
import de.fhg.iais.outdoor.simba.devices.Box;
import de.fhg.iais.outdoor.simba.devices.DifferentialDriveUnit;
import de.fhg.iais.outdoor.simba.devices.DistanceScanner;
import de.fhg.iais.outdoor.simba.devices.DistanceToPointScanner;
import de.fhg.iais.outdoor.simba.devices.LightSensor;
import de.fhg.iais.outdoor.simba.devices.LightSource;
import de.fhg.iais.outdoor.simba.devices.PathDrawer;
import de.fhg.iais.outdoor.simba.devices.SimpleBumpDetector;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetAgentInterface;
import de.fhg.iais.outdoor.simba.neuralnet.NeuronWrapper;
import de.fhg.iais.outdoor.simba.value.IntValue;
import de.fhg.iais.outdoor.simba.value.ValueManager;
import de.fhg.iais.outdoor.simba.visualization.SimbaDefaultFrame;

public class TrapFloor1 extends SimbaDefaultFrame {

public NeuralNetAgentInterface mMainController;
	
	public TrapFloor1(String args[]) {
		super("TrapFloor1", 700, 700);
		
		setDefaultOffset(-660.0, -680.0);
		setDefaultZoom(0.5);
		setVisualizationInterval(1);
		setSimulationDelay(15);
		
		mMainController = Simulator.getInstance().getNeuralNetManager().getNeuralNetInterface();
		
		//setFitnessFunction(new DelayedResponseMazeT1WarpFitnessCheckPoints(), 10000.0);
		setMasterEnvironment(new MyEnvironment(5));
		
		
		Simulator.getInstance().init();
		mMainController.setNeuronRange(-1.0, 1.0);
	
		
		Simulator.getInstance().getNeuralNetManager().setCurrentNet(NeuralNet.createMinimalNet(mMainController));

		startSystem(mMainController, args);
	}
	
	

	
	public class MyEnvironment extends EnvironmentModel {
		
		//Randomized parameters.
		//Arrays have to be cloned deeply in a custom clone() method.
		private double[] mCorridorSegmentLength;
		private double mRobotPositionX;
		private double mInitAngle;
		private double[] mBeaconSize;
		private boolean mLightIsLeft;
		private double[] mBeaconOffset;
		private double[] mTrapOffset;
		private double[] mTrapLength;
		private double[] mTrapWidth;
		
		//SimObjects used in the environments (clones should use the identical objects (flat copy)
		private DifferentialDriveUnit mRobot;
		private PathDrawer mPathDrawer;
		
		private Box mLowerBound;
		private Box mFinalGoalArea;
		private Box mLeftWall;
		private Box mRightWall;
		
		private SimObjectSet mTraps;
		private SimObjectSet mBeacons;
		
		private DistanceToPointScanner mDistanceToTargetArea;
		
		private SimpleBumpDetector mTrapBumper;
		private SimpleBumpDetector mWallBumper;
		
		private boolean mLightOnTheLeft;
		
		private Random mRandom = new Random();
		
		private IntValue mNumberOfTrapsValue;
		private int mMaxNumberOfTraps;
		private double mCorridorWidth;
		
		public MyEnvironment(int numberOfTraps) {
			super("TrapFloor1");
			
			mNumberOfTrapsValue = new IntValue(numberOfTraps);
			Simulator.getInstance().getValueManager().addValue("/Environment/MaxNumberOfTraps", mNumberOfTrapsValue);
			mMaxNumberOfTraps = numberOfTraps;
			
			mCorridorSegmentLength = new double[numberOfTraps];
			mBeaconSize = new double[numberOfTraps];
			mBeaconOffset = new double[numberOfTraps];
			mTrapOffset = new double[numberOfTraps];
			mTrapLength = new double[numberOfTraps];
			mTrapWidth = new double[numberOfTraps];
			
			mTraps = new SimObjectSet();
			mBeacons = new SimObjectSet();
			
			//TODO remove
			if(mLightIsLeft) {
				
			}
			
		}
		
		
		public void toggleLightSource() {
			mLightOnTheLeft = !mLightOnTheLeft;
		}

		
		/**
		 * Overridden for this special case: each created variation environment 
		 * is available twice, each with the opposite light distribution.
		 * 
		 * Creates the given number of environment variation clones 
		 * based on this Environment object and returns all these
		 * EnvironmentModels in a Vector.
		 * 
		 * This version creates two scenarios for each randomization pattern,
		 * one for a light on the left and one for a light on the right.
		 * 
		 * @param numberOfEnvironments the number of EnvironmentModels to create.
		 * @return a Vector with the newly created EnvironmentModels.
		 */
		public Vector createEnvironments(int numberOfEnvironments) {
			
			if(numberOfEnvironments % 2 == 1) {
				numberOfEnvironments++;
			}
			Vector clones = super.createEnvironments(numberOfEnvironments / 2);
		
			Vector environments = new Vector();
			for(Iterator i = clones.iterator(); i.hasNext();) {
				MyEnvironment env = (MyEnvironment) i.next();
				if(env != null) {
					environments.add(env);
				
					MyEnvironment envClone = null;
					try {
						envClone = (MyEnvironment) env.clone();
						envClone.toggleLightSource();
					} catch (CloneNotSupportedException e) {
						e.printStackTrace();
					}
					if(envClone != null) {
						environments.add(envClone);
					}
				}
			}
			return environments;
		}
		
		
		

		protected Object clone() throws CloneNotSupportedException {
			//Clone the arrays manually.
			MyEnvironment obj = (MyEnvironment) super.clone();

			obj.mCorridorSegmentLength = new double[mMaxNumberOfTraps];
			obj.mBeaconSize = new double[mMaxNumberOfTraps];
			obj.mBeaconOffset = new double[mMaxNumberOfTraps];
			obj.mTrapOffset = new double[mMaxNumberOfTraps];
			obj.mTrapLength = new double[mMaxNumberOfTraps];
			obj.mTrapWidth = new double[mMaxNumberOfTraps];
			
			
			for(int i = 0; i < mMaxNumberOfTraps; ++i) {
				obj.mCorridorSegmentLength[i] = mCorridorSegmentLength[i];
				obj.mBeaconSize[i] = mBeaconSize[i];
				obj.mBeaconOffset[i] = mBeaconOffset[i];
				obj.mTrapOffset[i] = mTrapOffset[i];
				obj.mTrapLength[i] = mTrapLength[i];
				obj.mTrapWidth[i] = mTrapWidth[i];
			}
			return obj;
		}


		public void updateRandomizationParameters() {
			
			
			mLightIsLeft = true;
			mRobotPositionX = (mRandom.nextDouble() - 0.5) * (mCorridorWidth / 3.0);
			mInitAngle = (mRandom.nextDouble() - 0.5) * 90.0;
			
			for(int i = 0; i < mMaxNumberOfTraps; ++i) {
				
				//TODO
				mBeaconSize[i] = 45.0;
				mBeaconOffset[i] = 100.0 + (mRandom.nextDouble() * 100.0);
				mTrapOffset[i] = mTrapOffset[i];
				mTrapLength[i] = 100.0 + (mRandom.nextDouble() * 200.0);
				mTrapWidth[i] = (mCorridorWidth / 2.0) + ((mRandom.nextDouble() -0.5) * (mCorridorWidth / 4.0));
				
				mCorridorSegmentLength[i] = mTrapOffset[i] + mTrapLength[i]
				                            + mBeaconOffset[i] + (100.0 + (mRandom.nextDouble() * 50.0));
			}
			
		}
		

		public void resetEnvironmentObjects() {
			
			mPathDrawer.reset();
			
			mRobot.setPosition(mRobotPositionX, 0.0);
			mRobot.setAngle(mInitAngle);
			
			//TODO apply randomized parameters here.
			Iterator traps = mTraps.getIterator();
			Iterator beacons = mBeacons.getIterator();
			
			int numberOfDesiredTraps = Math.max(0, Math.min(mNumberOfTrapsValue.get(), mMaxNumberOfTraps));
			double currentPosition = 0.0;
			
			for(int i = 0; (i < mMaxNumberOfTraps) && traps.hasNext() && beacons.hasNext();) {
				Box trap = (Box) traps.next();
				Beacon beacon = (Beacon) beacons.next();
				
				if(i >= numberOfDesiredTraps) {
					trap.setActive(false);
					beacon.setActive(false);
					beacon.getLightArea().setActive(false);
				}
				else {
					trap.setActive(true);
					beacon.setActive(true);
					beacon.getLightArea().setActive(true);
					
					beacon.setPosition(mCorridorWidth / 2.0, currentPosition + mBeaconOffset[i]);
					
					
//					trap.setPosition(mCorridorWidth / 2.0 )
					
					currentPosition = currentPosition + mBeaconOffset[i] + mTrapOffset[i] + mTrapLength[i];
					
				}
			}
			

		}

		public void createEnvironment() {

			//Set up scenario.
			mCorridorWidth = 200.0;
			
			mRobot = new DifferentialDriveUnit("Robot", 0.0, 10.0, 0.0, 20.0, 5.0, -1.5, 1.5);
			Box robotBody = new Box("Robot/Body", 0.0, 0.0, 90.0, 14.0, 20.0);
			robotBody.setReferenceObject(mRobot);
			robotBody.setShapeColor(Color.GREEN);
			addSimObject(mRobot);
			addSimObject(robotBody);
			
			mPathDrawer = new PathDrawer("RobotTrajectory", 0.0, 0.0, 2, 100, 1000);
			mPathDrawer.setReferenceObject(mRobot);
			mPathDrawer.setLayer(1);
			mPathDrawer.setShapeColor(Color.blue);
			mPathDrawer.setSecondColor(Color.cyan);
			addSimObject(mPathDrawer);
			
			
			//Walls
			mLeftWall = new Box("LeftWall", (-1 * (mCorridorWidth / 2.0)) - 20.0, 1000.0, 0.0, 40.0, 2150.0);
			addSimObject(mLeftWall);
			
			mRightWall = new Box("RightWall", (mCorridorWidth / 2.0) + 20, 1000.0, 0.0, 40.0, 2150.0);
			addSimObject(mRightWall);
			
			mFinalGoalArea = new Box("FinalGoal", 0.0, 2045.0, 0.0, mCorridorWidth, 60.0);
			mFinalGoalArea.setShapeColor(Color.GREEN);
			mFinalGoalArea.setLayer(-5);
			addSimObject(mFinalGoalArea);
			
			mLowerBound = new Box("LowerBound", 0.0, -45.0, 0.0, mCorridorWidth, 60.0);
			mLowerBound.setShapeColor(Color.RED);
			mLowerBound.setLayer(-5);
			addSimObject(mLowerBound);
			
			SimObjectSet obstacleList = new SimObjectSet();
			obstacleList.add(mLeftWall);
			obstacleList.add(mRightWall);
			
			mTraps.add(mFinalGoalArea);
			
			for(int i = 0; i < mMaxNumberOfTraps; ++i) {
				Beacon beacon = new Beacon("Beacon" + i, 0.0, 0.0, 0.0, mCorridorWidth, 10, 45.0, 1.0, 1.0);
				beacon.setLayer(1);
				beacon.getLightArea().setLayer(-1);
				addSimObject(beacon);
				Box box = new Box("Trap" + i, 0.0, 100.0, 0.0, (mCorridorWidth / 2.0), 100.0);
				box.setLayer(-5);
				box.setShapeColor(Color.RED);
				addSimObject(box);
				mTraps.add(box);
				
				mBeacons.add(beacon);
				mTraps.add(box);
			}
			
			
			
			
			
			DistanceScanner scanLeft = new DistanceScanner("DistanceScannerLeft", 0.0, 10.0, 60.0, 0.0, 50.0);
			scanLeft.setReferenceObject(robotBody);
			scanLeft.addDependencies(obstacleList);
			scanLeft.addObstacleSet(obstacleList);
			scanLeft.setNoiseVariance(1.0);
			addSimObject(scanLeft);
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForDistanceScanner(scanLeft));
			
			DistanceScanner scanRight = new DistanceScanner("DistanceScannerRight", 0.0, -10.0, -60.0, 0.0, 50.0);
			scanRight.setReferenceObject(robotBody);
			scanRight.addDependencies(obstacleList);
			scanRight.addObstacleSet(obstacleList);
			scanRight.setNoiseVariance(1.0);
			addSimObject(scanRight);
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForDistanceScanner(scanRight));
			
			DistanceScanner scanMid = new DistanceScanner("DistanceScannerFront", 7.0, 0.0, 0.0, 0.0, 40.0);
			scanMid.setReferenceObject(robotBody);
			scanMid.addObstacleSet(obstacleList);
			scanMid.addObstacleSet(obstacleList);
			scanMid.setNoiseVariance(1.0);
			addSimObject(scanMid);
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForDistanceScanner(scanMid));
			
			
			SimObjectSet leftLightList = new SimObjectSet();
			//..
			SimObjectSet rightLightList = new SimObjectSet();
			//..
			
			//Beacons and traps
			
			
			
//			mBeaconLeft = new Beacon("BeaconLeft", 0.0, 0.0, 315.0, 0.0, 20, 90.0, 1.0, 1.0);
//			mBeaconLeft.setReferenceObject(mLeftConnector);
//			mBeaconLeft.setLayer(1);
//			mBeaconLeft.getLightArea().setLayer(-1);
//			mBeaconLeft.getLightArea().setShapeColor(Color.ORANGE);
//			addSimObject(mBeaconLeft);

			
			
			//Light sensors
			LightSensor leftLightSensor = new LightSensor("LeftLightSensor", 0.0, 0.0, 0.0, 0.0, 1.0, true);
			for(Iterator i = leftLightList.getIterator(); i.hasNext();) {
				leftLightSensor.addLightSource((LightSource) i.next());
			}
			leftLightSensor.setReferenceObject(robotBody);
			leftLightSensor.setShapeColor(Color.BLUE);
			leftLightSensor.setNoiseVariance(0.1);
			addSimObject(leftLightSensor);
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForLightSensor(leftLightSensor));
			
			LightSensor rightLightSensor = new LightSensor("ReftLightSensor", 0.0, 0.0, 0.0, 0.0, 1.0, true);
			for(Iterator i = rightLightList.getIterator(); i.hasNext();) {
				rightLightSensor.addLightSource((LightSource) i.next());
			}
			rightLightSensor.setReferenceObject(robotBody);
			rightLightSensor.setShapeColor(Color.CYAN);
			rightLightSensor.setNoiseVariance(0.1);
			addSimObject(rightLightSensor);
			mMainController.addInputNeuron(NeuronWrapper.createWrapperForLightSensor(rightLightSensor));
			
			
			//Add motor neurons
			ValueManager vm = Simulator.getInstance().getValueManager();
			String name = "/SimObjects/" + mRobot.getName() + "/VelocityLeft";
			NeuronWrapper motorLeft = new NeuronWrapper(name, 0.0, mRobot.getMinVelocity(), mRobot.getMaxVelocity());
			vm.replaceValue(name, motorLeft);
			mMainController.addOutputNeuron(motorLeft);
			
			name = "/SimObjects/" + mRobot.getName() + "/VelocityRight";
			NeuronWrapper motorRight = new NeuronWrapper(name, 0.0, mRobot.getMinVelocity(), mRobot.getMaxVelocity());
			vm.replaceValue(name, motorRight);
			mMainController.addOutputNeuron(motorRight);
			
			
			//Trap and Wall bumpers
			mWallBumper = new SimpleBumpDetector("WallBumper", 0.0, 0.0, 0.0);
			Polygon p = robotBody.getShape();
			mWallBumper.setShape(new Polygon(p.xpoints, p.ypoints, p.npoints));
			mWallBumper.setReferenceObject(robotBody);
			mWallBumper.addDependencies(obstacleList);
			mWallBumper.addObstacleSet(obstacleList);
			addSimObject(mWallBumper);
			Simulator.getInstance().getScheduler().addTerminateTryValue(mWallBumper.getSensorValueObject());
			
			mTrapBumper = new SimpleBumpDetector("TrapBumper", 0.0, 0.0, 0.0);
			mTrapBumper.setShape(new Polygon(p.xpoints, p.ypoints, p.npoints)); //robot shape
			mTrapBumper.setReferenceObject(robotBody);
			mTrapBumper.addDependencies(mTraps);
			mTrapBumper.addObstacleSet(mTraps);
			addSimObject(mTrapBumper);
			Simulator.getInstance().getScheduler().addTerminateTryValue(mTrapBumper.getSensorValueObject());
			
			
			mDistanceToTargetArea = new DistanceToPointScanner("DistanceToTargetArea", 0.0, 0.0, 0.0, 0.0, 10000.0);
			mDistanceToTargetArea.addObstacle(mFinalGoalArea);
			mDistanceToTargetArea.setReferenceObject(robotBody);
			addSimObject(mDistanceToTargetArea);
			

		}
	}
		

	public static void main(String[] args) {
		new TrapFloor1(args);
	}
	
}
