/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.statistic;

import java.util.Iterator;
import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.GenerationListener;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.evolution.EvolutionIndividual;
import de.fhg.iais.outdoor.simba.evolution.EvolutionManager;
import de.fhg.iais.outdoor.simba.value.HistoryValue;

public class EvolutionStatistics extends StatisticCalculator implements GenerationListener {

	
	private HistoryValue mMeanAge;
	private HistoryValue mExecutionTime;
	private HistoryValue mNumberOfIndividuals;
	private long mStartingTime;
	
	
	public EvolutionStatistics() {
		super("EvolutionStatistics");
		
		mMeanAge = new HistoryValue();
		mExecutionTime = new HistoryValue();
		mNumberOfIndividuals = new HistoryValue();
		
		addParameter(mMeanAge, "MeanAge");
		addParameter(mExecutionTime, "ExecutionTime");
		addParameter(mNumberOfIndividuals, "NumberOfIndividuals");
		
		mStartingTime = 0;
		
		Simulator.getInstance().getScheduler().addGenerationListener(this);
		Simulator.getInstance().getStatisticManager().addCalculator(this);
		
	}
	
	public void calculate() {
		
		EvolutionManager em = Simulator.getInstance().getEvolutionManager();
		
		//Execution Time
		generationCompleted(0);
		
		//Mean Age
		Vector individuals = em.getGeneration();
		int ageSum = 0;
		int individualCount = 0;
		for(Iterator i = individuals.iterator(); i.hasNext();) {
			EvolutionIndividual ind = (EvolutionIndividual) i.next();
			ageSum += ind.getAge();
			individualCount++;
		}
		if(individualCount == 0) {
			mMeanAge.set(0.0);
		}
		else {
			mMeanAge.set(((double) ageSum) / ((double) individualCount));
		}
		
		//NumberOfIndividuals
		mNumberOfIndividuals.set((double) individualCount);
		
	}

	public void generationCompleted(int generationId) {
		if(mStartingTime != 0) {
			long endTime = System.currentTimeMillis();
			mExecutionTime.set(((double) (endTime - mStartingTime)) / 1000.0);
			mStartingTime = 0;
		}
	}

	public void generationStarted(int generationId) {
		mStartingTime = System.currentTimeMillis();
	}
	
	

}
