/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.test;

import java.awt.geom.Point2D;

import de.fhg.iais.outdoor.simba.devices.Box;


public class TestActuator extends Box {
	
	public double mVelocity;
	private int mSteps;
	private int mMaxSteps;
	private boolean mUp;
	
	TestActuator(String name, double x, double y, double angle, double width, double height, double vel) {
		super(name, x, y, angle, width, height);
		mVelocity = vel;
		mSteps = 0;
		mUp = true;
		mMaxSteps = 2;
	}
	
	public void setSteps(int steps) {
		mMaxSteps = steps;
	}
	

	public boolean update() {
		boolean ok = super.update();
	
		Point2D.Double position = mPosition.get();
		if(mUp) {
			mSteps++;
			if(mSteps > mMaxSteps) {
				mUp = false;
			}
			mPosition.set(position.x + (Math.cos(Math.toRadians(mAngle.get())) * mVelocity), 
					position.y + (Math.sin(Math.toRadians(mAngle.get())) * mVelocity));
		}
		else {
			mSteps--;
			if(mSteps < -1 * mMaxSteps) {
				mUp = true;
			}
			mPosition.set(position.x - (Math.cos(Math.toRadians(mAngle.get())) * mVelocity),
						position.y + (Math.sin(Math.toRadians(mAngle.get())) * mVelocity));
		}
		
		return ok;
	}
	
	public void reset() {
		super.reset();
	}

}
