/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.test;

import java.awt.Polygon;

import de.fhg.iais.outdoor.simba.core.SimObject;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.devices.AngleToPointScanner;
import de.fhg.iais.outdoor.simba.value.DoubleValue;
import de.fhg.iais.outdoor.simba.value.ValueManager;
import junit.framework.TestCase;

public class TestAngleToPointScanner extends TestCase {

	protected void setUp() throws Exception {
		super.setUp();
		Simulator.getInstance().clearInstance();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
		Simulator.getInstance().clearInstance();
	}

	public void testConstructor() {
		AngleToPointScanner s = new AngleToPointScanner("AngleScanner1", 10.1, 20.0, 30.0, -90.0, 90.0);
		
		assertEquals(s.getName(), "AngleScanner1");
		assertEquals(s.getPosition().x, 10.1, 0.001);
		assertEquals(s.getPosition().y, 20.0, 0.001);
		assertEquals(s.getAngle(), 30.0, 0.001);
		assertEquals(s.getMinScanAngle(), -90.0, 0.001);
		assertEquals(s.getMaxScanAngle(), 90.0, 0.001);
		
	}
	
	public void testUpdate() {
		AngleToPointScanner s = new AngleToPointScanner("AngleScanner1", 0.0, 0.0, 0.0, -180.0, 180.0);
		
		SimObject obs = new SimObject("obs1", 0.0, 3.0, 0.0, new Polygon());
		
		assertTrue(s.addObstacle(obs));
		
		s.update();
		assertEquals(s.getSensorReading(), 90.0, 0.001);
		
		obs.setPosition(0.0, -5.0);
		obs.update();
		s.update();
		assertEquals(s.getSensorReading(), -90.0, 0.001);
		
		s.setPosition(-2.0, -7.0);
		s.setAngle(90.0);
		s.update();
		assertEquals(s.getSensorReading(), -45.0, 0.001);
		
		s.setAngle(180.0);
		s.update();
		assertEquals(s.getSensorReading(), -135.0, 0.001);
		
		s.setAngle(270.0);
		s.update();
		assertEquals(s.getSensorReading(), 135.0, 0.001);
		
		s.setMinScanAngle(-90.0);
		s.setMaxScanAngle(120.0);
		
		s.update();
		assertEquals(s.getSensorReading(), 120.0, 0.001);
		
		s.setAngle(180.0);
		s.update();
		assertEquals(s.getSensorReading(), -90.0, 0.001);
		
		//add a second obstacle
		s.setMinScanAngle(-180.0);
		s.setMaxScanAngle(180.0);
		s.setAngle(0.0);
		s.setPosition(2.0, -3.0);
		
		s.update();
		assertEquals(s.getSensorReading(), -135.0, 0.001);
		
		
		SimObject obs2 = new SimObject("Second", 4.0, -5.0, 123.0, new Polygon());
		s.addObstacle(obs2);
		
		s.update();
		assertEquals(s.getSensorReading(), -45.0, 0.001);
		
		s.setAngle(360.0 - 45.0);
		s.update();
		assertEquals(s.getSensorReading(), 0.0, 0.001);
		
		s.setAngle(180.0);
		s.update();
		assertEquals(s.getSensorReading(), 45.0, 0.001);
		
	}

	public void testValueReplaced() {
		ValueManager vm = Simulator.getInstance().getValueManager();
		
		AngleToPointScanner s = new AngleToPointScanner("AngleScanner1", 0.0, 0.0, 0.0, -100.0, 100.0);
		assertNotNull(vm.getDoubleValue("/SimObjects/AngleScanner1/MinScanAngle"));
		DoubleValue min = vm.getDoubleValue("/SimObjects/AngleScanner1/MinScanAngle");
		assertNotNull(vm.getDoubleValue("/SimObjects/AngleScanner1/MaxScanAngle"));
		DoubleValue max = vm.getDoubleValue("/SimObjects/AngleScanner1/MaxScanAngle");
		assertNotNull(vm.getDoubleValue("/SimObjects/AngleScanner1/Angle"));
		DoubleValue angle = vm.getDoubleValue("/SimObjects/AngleScanner1/Angle");

		
		s.setMinScanAngle(-55.5);
		assertEquals(s.getMinScanAngle(), -55.5, 0.001);
		assertEquals(min.get(), -55.5, 0.001);
		min.set(0.4);
		assertEquals(s.getMinScanAngle(), 0.4, 0.001);
		DoubleValue min2 = new DoubleValue(123.4);
		vm.replaceValue("/SimObjects/AngleScanner1/MinScanAngle", min2);
		assertEquals(s.getMinScanAngle(), 123.4, 0.001);
		
		s.setMaxScanAngle(-55.5);
		assertEquals(s.getMaxScanAngle(), -55.5, 0.001);
		assertEquals(max.get(), -55.5, 0.001);
		max.set(0.4);
		assertEquals(s.getMaxScanAngle(), 0.4, 0.001);
		DoubleValue max2 = new DoubleValue(123.4);
		vm.replaceValue("/SimObjects/AngleScanner1/MaxScanAngle", max2);
		assertEquals(s.getMaxScanAngle(), 123.4, 0.001);
		
		angle.set(567.1);
		assertEquals(s.getSensorReading(), 567.1, 0.001);
		assertEquals(angle.get(), 567.1, 0.001);
		DoubleValue angle2 = new DoubleValue(123.4);
		vm.replaceValue("/SimObjects/AngleScanner1/Angle", angle2);
		assertEquals(s.getSensorReading(), 123.4, 0.001);
		
		
	}

	public void testAddObstacle() {
		
	}

}
