/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.test;

import java.util.Vector;

import de.fhg.iais.outdoor.simba.neuralnet.ConnectivityRating;
import de.fhg.iais.outdoor.simba.neuralnet.Neuron;
import de.fhg.iais.outdoor.simba.neuralnet.Synapse;
import junit.framework.TestCase;

public class TestConnectivityRating extends TestCase {

	public void testIsConnected() {
		Neuron n1 = new Neuron("N1");
		Neuron n2 = new Neuron("N2");
		Neuron n3 = new Neuron("N3");
		Neuron n4 = new Neuron("N4");
		Neuron n5 = new Neuron("N5");
		Neuron n6 = new Neuron("N6");
		Neuron n7 = new Neuron("N7");
		
		Vector actuators = new Vector();
		actuators.add(n5);
		actuators.add(n6);
		
		assertTrue(ConnectivityRating.isConnected(n5, actuators));
		assertTrue(ConnectivityRating.isConnected(n6, actuators));
		
		assertFalse(ConnectivityRating.isConnected(n1, actuators));
		
		Synapse s = new Synapse(n1);
		n5.addSynapse(s);
		
		
		assertTrue(ConnectivityRating.isConnected(n1, actuators));
		assertFalse(ConnectivityRating.isConnected(n2, actuators));
		
		s = new Synapse(n2);
		n1.addSynapse(s);
		assertTrue(ConnectivityRating.isConnected(n2, actuators));
		assertFalse(ConnectivityRating.isConnected(n3, actuators));
		assertFalse(ConnectivityRating.isConnected(n4, actuators));
		
		s = new Synapse(n3);
		n3.addSynapse(s);
		n4.addSynapse(s);
		
		s = new Synapse(n4);
		n3.addSynapse(s);
		n4.addSynapse(s);
		
		assertFalse(ConnectivityRating.isConnected(n3, actuators));
		assertFalse(ConnectivityRating.isConnected(n4, actuators));
		
		n2.addSynapse(s);
		assertTrue(ConnectivityRating.isConnected(n3, actuators));
		assertTrue(ConnectivityRating.isConnected(n4, actuators));
		
		s = new Synapse(n5);
		n5.addSynapse(s);
		n6.addSynapse(s);
		s = new Synapse(n6);
		n6.addSynapse(s);
		
		assertTrue(ConnectivityRating.isConnected(n1, actuators));
		assertTrue(ConnectivityRating.isConnected(n2, actuators));
		assertTrue(ConnectivityRating.isConnected(n3, actuators));
		assertTrue(ConnectivityRating.isConnected(n4, actuators));
		assertTrue(ConnectivityRating.isConnected(n5, actuators));
		assertTrue(ConnectivityRating.isConnected(n6, actuators));
		assertFalse(ConnectivityRating.isConnected(n7, actuators));
		
		s = new Synapse(n5);
		n7.addSynapse(s);
		
		assertFalse(ConnectivityRating.isConnected(n7, actuators));
		
		s = new Synapse(n4);
		n7.addSynapse(s);
		
		assertFalse(ConnectivityRating.isConnected(n7, actuators));
		
		
	}
	
	public void testIsInverseConnected() {
		
		Neuron n1 = new Neuron("N1");
		Neuron n2 = new Neuron("N2");
		Neuron n3 = new Neuron("N3");
		Neuron n4 = new Neuron("N4");
		Neuron n5 = new Neuron("N5");
		Neuron n6 = new Neuron("N6");
		
		Vector sensors = new Vector();
		sensors.add(n3);
		sensors.add(n4);
		
		assertTrue(ConnectivityRating.isInverseConnected(n3, sensors));
		assertTrue(ConnectivityRating.isInverseConnected(n4, sensors));
		
		assertFalse(ConnectivityRating.isInverseConnected(n2, sensors));
		
		Synapse s = new Synapse(n4);
		n2.addSynapse(s);
		
		assertTrue(ConnectivityRating.isInverseConnected(n2, sensors));
		assertFalse(ConnectivityRating.isInverseConnected(n1, sensors));
		
		n1.addSynapse(s);
		assertTrue(ConnectivityRating.isInverseConnected(n1, sensors));
		
		s = new Synapse(n5);
		n5.addSynapse(s);
		n3.addSynapse(s);
		n1.addSynapse(s);
		
		assertFalse(ConnectivityRating.isInverseConnected(n5, sensors));
		
		s = new Synapse(n1);
		n5.addSynapse(s);
		
		assertTrue(ConnectivityRating.isInverseConnected(n5, sensors));
		
		n1.addSynapse(s);
		assertTrue(ConnectivityRating.isInverseConnected(n1, sensors));
		assertTrue(ConnectivityRating.isInverseConnected(n2, sensors));
		assertTrue(ConnectivityRating.isInverseConnected(n3, sensors));
		assertTrue(ConnectivityRating.isInverseConnected(n4, sensors));
		assertTrue(ConnectivityRating.isInverseConnected(n5, sensors));
		assertFalse(ConnectivityRating.isInverseConnected(n6, sensors));
		
	}

}
