/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.test;

import java.awt.Polygon;
import java.awt.geom.Point2D;

import de.fhg.iais.outdoor.simba.core.SimObject;
import de.fhg.iais.outdoor.simba.core.SimObjectSet;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.devices.DistanceScanner;
import de.fhg.iais.outdoor.simba.devices.DistanceToPointScanner;
import junit.framework.TestCase;

public class TestDistanceToPointScanner extends TestCase {

	protected void tearDown() throws Exception {
		super.tearDown();
		Simulator.getInstance().clearInstance();
	}
	
	/*
	 * Test method for 'de.fhg.iais.outdoor.simba.devices.DistanceToPointScanner.DistanceToPointScanner(String, double, double, double, double, double, double, double)'
	 */
	public void testDistanceToPointScanner() {
		DistanceToPointScanner obj = new DistanceToPointScanner(
				"Scanner1", 1.0, 2.0, 0.1, 0.5, 6.5);
		
		assertEquals(obj.getName(), "Scanner1");
		assertEquals(obj.getPosition(), new Point2D.Double(1.0, 2.0));
		assertEquals(obj.getAngle(), 0.1, 0.001);
		assertEquals(obj.getMinScanDistance(), 0.5, 0.001);
		assertEquals(obj.getMaxScanDistance(), 6.5, 0.001);
		assertEquals(obj.getObstacles().size(), 0);
		assertEquals(obj.getDependencies().size(), 0);
		assertTrue(obj.isActive());
		assertTrue(obj.isVisible());
	}
	
	public void testGlobalStuffCalculation() {
		
		DistanceScanner obj = new DistanceScanner("", 
			1.0, 2.0, 0.1, 0.5, 6.5);
		
		assertEquals(obj.getGlobalAngle(), 0.1, 0.001);
		assertEquals(obj.getGlobalPosition().x, 1.0, 0.001);
		assertEquals(obj.getGlobalPosition().y, 2.0, 0.001);
		
		obj.setPosition(10.0, 12.0);
		obj.setAngle(43.21);
		
		assertEquals(obj.getGlobalAngle(), 0.1, 0.001);
		assertEquals(obj.getGlobalPosition().x, 1.0, 0.001);
		assertEquals(obj.getGlobalPosition().y, 2.0, 0.001);
		
		obj.update();
		assertEquals(obj.getGlobalAngle(), 43.21, 0.001);
		assertEquals(obj.getGlobalPosition().x, 10.0, 0.001);
		assertEquals(obj.getGlobalPosition().y, 12.0, 0.001);
	}

	/*
	 * Test method for 'de.fhg.iais.outdoor.simba.devices.DistanceToPointScanner.update()'
	 */
	public void testUpdate() {
		DistanceToPointScanner scanner = new DistanceToPointScanner(
				"Scanner1", 2.0, 3.0, 1.0, 0.0, 10.0);
		
		assertEquals(scanner.getSensorReading(), 0.0, 0.001);
		
		
		SimObject obstacle = new SimObject("Obstacle", 8.0, 3.0, 0.0, new Polygon());
		scanner.addObstacle(obstacle);
		
		assertEquals(scanner.getSensorReading(), 0.0, 0.001);
		
		assertTrue(scanner.update());
		assertEquals(scanner.getSensorReading(), 6.0, 0.001);
		
		obstacle.setPosition(6.0, 3.0);
		obstacle.update();
		assertTrue(scanner.update());
		assertEquals(scanner.getSensorReading(), 4.0, 0.001);
		
		obstacle.setPosition(8.0, 5.0);
		obstacle.update();
		assertTrue(scanner.update());
		assertEquals(scanner.getSensorReading(), Math.sqrt(40.0), 0.001); //Pytagoras: sqrt(6^2 + 2^2).
		
		obstacle.setPosition(0.0, 6.0);
		obstacle.update();
		assertTrue(scanner.update());
		assertEquals(scanner.getSensorReading(), Math.sqrt(13.0), 0.001); 
		
		obstacle.setPosition(-1.0, -6.0);
		obstacle.update();
		assertTrue(scanner.update());
		assertEquals(scanner.getSensorReading(), Math.sqrt(90.0), 0.001); 
		
		//different angles. The distance is not affected by the angle.
		obstacle.setPosition(8.0, 5.0);
		obstacle.update();
		scanner.setAngle(22.6);
		assertTrue(scanner.update());
		assertEquals(scanner.getSensorReading(), Math.sqrt(40.0), 0.001); 
		
		obstacle.setPosition(0.0, 6.0);
		obstacle.update();
		scanner.setAngle(199.2);
		assertTrue(scanner.update());
		assertEquals(scanner.getSensorReading(), Math.sqrt(13.0), 0.001); 
		
		
		scanner.setMinScanDistance(5.0);
		obstacle.setPosition(6.0, 3.0);
		obstacle.update();
		assertTrue(scanner.update());
		assertEquals(scanner.getSensorReading(), 5.0, 0.001); //instead of 4.0
		
		
	}
	
	public void testObstacleSets() {
			
		DistanceToPointScanner scanner = new DistanceToPointScanner(
				"Scanner1", 2.0, 3.0, 1.0, 0.0, 10.0);
		
		assertEquals(scanner.getSensorReading(), 0.0, 0.001);
		
		SimObjectSet set = new SimObjectSet();
		
		scanner.update();
		assertEquals(scanner.getSensorReading(), 10.0, 0.001);
		
		SimObject obs = new SimObject("Obstacle", 8.0, 3.0, 0.0, new Polygon());
		assertTrue(set.add(obs));
		
		assertTrue(scanner.addObstacleSet(set));
		scanner.update();
		assertEquals(scanner.getSensorReading(), 6.0, 0.001);
		
		assertFalse(scanner.addObstacleSet(set)); // already contained.
		scanner.update();
		assertEquals(scanner.getSensorReading(), 6.0, 0.001);
		
		assertFalse(scanner.removeObstacleSet(null));
		scanner.update();
		assertEquals(scanner.getSensorReading(), 6.0, 0.0001);
		
		assertTrue(scanner.removeObstacleSet(set));
		scanner.update();
		assertEquals(scanner.getSensorReading(), 10.0, 0.0001);
		
		assertFalse(scanner.removeObstacleSet(set)); //not contained.
		scanner.update();
		assertEquals(scanner.getSensorReading(), 10.0, 0.0001);
		
		
	}

	

}
