/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.test;

import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.GenerationListener;
import de.fhg.iais.outdoor.simba.core.IndividualListener;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.evolution.EvaluationMethod;
import de.fhg.iais.outdoor.simba.evolution.EvolutionIndividual;
import de.fhg.iais.outdoor.simba.evolution.EvolutionListener;
import de.fhg.iais.outdoor.simba.evolution.EvolutionManager;
import de.fhg.iais.outdoor.simba.evolution.EvolutionOperator;
import de.fhg.iais.outdoor.simba.evolution.NeuralNetEvaluationMethod;
import de.fhg.iais.outdoor.simba.evolution.EvolutionTesterInterface;
import de.fhg.iais.outdoor.simba.evolution.SelectionMethod;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetAgentInterface;
import de.fhg.iais.outdoor.simba.neuralnet.Neuron;
import de.fhg.iais.outdoor.simba.neuralnet.NeuronWrapper;
import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.IntValue;
import junit.framework.TestCase;

public class TestEvolutionManager extends TestCase {

	protected void setUp() throws Exception {
		super.setUp();
		Simulator.getInstance().clearInstance();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
		Simulator.getInstance().clearInstance();
	}
	
	public void testRunEvolution() {
		EvolutionManager em = Simulator.getInstance().getEvolutionManager();
		
		BooleanValue runEvoVal = em.getRunEvolutionValue();
		assertNotNull(runEvoVal);
		assertFalse(runEvoVal.get());
		
		IntValue popSizeVal = em.getMeanPopulationSizeValue();
		assertNotNull(popSizeVal);
		
		BooleanValue evoRunningVal = em.getEvolutionRunningValue();
		assertNotNull(evoRunningVal);
		assertFalse(evoRunningVal.get());
		
		IntValue preserveParents = em.getPreserveParentsValue();
		assertNotNull(preserveParents);
		
		IntValue currentPopSize = em.getCurrentPopulationSizeValue();
		assertNotNull(currentPopSize);
		assertEquals(currentPopSize.get(), 0);
		
		BooleanValue stase = em.getStaseModeValue();
		assertNotNull(stase);
		assertFalse(stase.get());
		
		BooleanValue evoEnabled = em.getEvolutionEnabledValue();
		assertNotNull(evoEnabled);
		assertFalse(evoEnabled.get());
		
		
		assertEquals(em.getGeneration().size(), 0);
		Vector gen = new Vector();
		EvolutionIndividual ei1 = new EvolutionIndividual("Test");
		gen.add(ei1);
		assertTrue(em.replaceGeneration(gen));
		assertEquals(em.getGeneration().size(), 1);
		assertSame(em.getGeneration().firstElement(), ei1);
		
		assertNotNull(em.getEvaluationMethod());
		assertTrue(em.getEvaluationMethod() instanceof  NeuralNetEvaluationMethod);
		assertEquals(em.getEvolutionOperators().size(), 0);
		assertEquals(em.getSelectionMethods().size(), 0);
	}


	public void testAddEvolutionOperator() {
		EvolutionManager em = Simulator.getInstance().getEvolutionManager();
		AdapterEvolutionOperator eo1 = new AdapterEvolutionOperator();
		AdapterEvolutionOperator eo2 = new AdapterEvolutionOperator();
		
		assertEquals(eo1.getLayerValue().get(), 0);
		assertEquals(eo2.getLayerValue().get(), 0);
		
		assertEquals(em.getEvolutionOperators().size(), 0);
		
		assertFalse(em.addEvolutionOperator(null, 20));
		assertTrue(em.addEvolutionOperator(eo1, 20));
		assertFalse(em.addEvolutionOperator(eo1, 20));
		assertEquals(em.getEvolutionOperators().size(), 1);
		assertEquals(eo1.getLayerValue().get(), 20);
		
		assertTrue(em.addEvolutionOperator(eo2, 15));
		assertEquals(em.getEvolutionOperators().size(), 2);
		assertEquals(eo2.getLayerValue().get(), 15);
		
		assertFalse(em.removeEvolutionOperator(null));
		assertTrue(em.removeEvolutionOperator(eo1));
		assertEquals(em.getEvolutionOperators().size(), 1);
		assertEquals(em.getEvolutionOperators().elementAt(0), eo2);
		assertFalse(em.removeEvolutionOperator(eo1)); //already removed
		assertEquals(em.getEvolutionOperators().size(), 1);
		
		assertTrue(em.removeEvolutionOperator(eo2));
		assertEquals(em.getEvolutionOperators().size(), 0);
	}

	public void testAddSelectionMethod() {
		EvolutionManager em = Simulator.getInstance().getEvolutionManager();
		AdapterSelectionMethod sm1 = new AdapterSelectionMethod();
		AdapterSelectionMethod sm2 = new AdapterSelectionMethod();
		AdapterSelectionMethod sm3 = new AdapterSelectionMethod();
		
		sm1.getOffspringPercentageValue().set(20.0);
		sm2.getOffspringPercentageValue().set(50.0);
		sm3.getOffspringPercentageValue().set(90.0);
		
		assertEquals(sm1.getOffspringPercentageValue().get(), 20.0, 0.001);
		assertEquals(sm2.getOffspringPercentageValue().get(), 50.0, 0.001);
		assertEquals(sm3.getOffspringPercentageValue().get(), 90.0, 0.001);
		
		assertEquals(em.getSelectionMethods().size(), 0);
		
		assertFalse(em.addSelectionMethod(null));
		assertTrue(em.addSelectionMethod(sm1));
		assertEquals(em.getSelectionMethods().size(), 1);
		assertFalse(em.addSelectionMethod(sm1));
		
		assertEquals(sm1.getOffspringPercentageValue().get(), 100.0, 0.001);
		sm1.getOffspringPercentageValue().set(11.23); //is changed back to 100%
		assertEquals(sm1.getOffspringPercentageValue().get(), 100.0, 0.001);
		
		
		assertTrue(em.addSelectionMethod(sm2));
		assertEquals(em.getSelectionMethods().size(), 2);
		assertEquals(sm2.getOffspringPercentageValue().get(), 50.0, 0.001);
		assertEquals(sm1.getOffspringPercentageValue().get(), 50.0, 0.001);
		
		//Change offspring percentage. The sum always has to be 100%
		assertTrue(em.addSelectionMethod(sm3));
		assertEquals(em.getSelectionMethods().size(), 3);
		assertEquals(sm1.getOffspringPercentageValue().get(), 10.0, 0.001);
		assertEquals(sm2.getOffspringPercentageValue().get(), 0.0, 0.001);
		assertEquals(sm3.getOffspringPercentageValue().get(), 90.0, 0.001);
		
		sm1.getOffspringPercentageValue().set(50.0);
		assertEquals(sm1.getOffspringPercentageValue().get(), 50.0, 0.001);
		assertEquals(sm2.getOffspringPercentageValue().get(), 0.0, 0.001);
		assertEquals(sm3.getOffspringPercentageValue().get(), 50.0, 0.001);
		
		sm2.getOffspringPercentageValue().set(15.0);
		assertEquals(sm1.getOffspringPercentageValue().get(), 50.0, 0.001);
		assertEquals(sm2.getOffspringPercentageValue().get(), 15.0, 0.001);
		assertEquals(sm3.getOffspringPercentageValue().get(), 35.0, 0.001);
		
		sm1.getOffspringPercentageValue().set(15.0);
		assertEquals(sm1.getOffspringPercentageValue().get(), 15.0, 0.001);
		assertEquals(sm2.getOffspringPercentageValue().get(), 15.0, 0.001);
		assertEquals(sm3.getOffspringPercentageValue().get(), 70.0, 0.001);
		
		sm1.getOffspringPercentageValue().set(30.0);
		assertEquals(sm1.getOffspringPercentageValue().get(), 30.0, 0.001);
		assertEquals(sm2.getOffspringPercentageValue().get(), 15.0, 0.001);
		assertEquals(sm3.getOffspringPercentageValue().get(), 55.0, 0.001);
		
		
		assertFalse(em.removeSelectionMethod(null));
		assertEquals(em.getSelectionMethods().size(), 3);
		assertTrue(em.removeSelectionMethod(sm1));
		assertEquals(em.getSelectionMethods().size(), 2);
		assertFalse(em.removeSelectionMethod(sm1));
		
		assertEquals(sm1.getOffspringPercentageValue().get(), 30.0, 0.001); //not changed.
		assertEquals(sm2.getOffspringPercentageValue().get(), 15.0, 0.001);
		assertEquals(sm3.getOffspringPercentageValue().get(), 85.0, 0.001); //the 30% are added here.
		
		assertTrue(em.removeSelectionMethod(sm3));
		assertEquals(em.getSelectionMethods().size(), 1);
		
		assertEquals(sm1.getOffspringPercentageValue().get(), 30.0, 0.001); //not changed.
		assertEquals(sm2.getOffspringPercentageValue().get(), 100.0, 0.001); //rest added here (only active selection method)
		assertEquals(sm3.getOffspringPercentageValue().get(), 85.0, 0.001); //not changed.
		
		assertTrue(em.removeSelectionMethod(sm2));
		assertEquals(em.getSelectionMethods().size(), 0);
		
	}
	
	public void testExecuteEvolution() {
		EvolutionManager em = Simulator.getInstance().getEvolutionManager();
		
		AdapterSelectionMethod sm1 = new AdapterSelectionMethod();
		AdapterSelectionMethod sm2 = new AdapterSelectionMethod();
		AdapterEvolutionOperator eo1 = new AdapterEvolutionOperator();
		AdapterEvolutionOperator eo2 = new AdapterEvolutionOperator();
		
		BooleanValue runEvoVal = em.getRunEvolutionValue();
		assertNotNull(runEvoVal);
		assertFalse(runEvoVal.get());
		
		IntValue popSizeVal = em.getMeanPopulationSizeValue();
		assertNotNull(popSizeVal);
		
		BooleanValue evoRunningVal = em.getEvolutionRunningValue();
		assertNotNull(evoRunningVal);
		assertFalse(evoRunningVal.get());
		
		IntValue preserveParents = em.getPreserveParentsValue();
		assertNotNull(preserveParents);
		
		IntValue currentPopSize = em.getCurrentPopulationSizeValue();
		assertNotNull(currentPopSize);
		assertEquals(currentPopSize.get(), 0);
		
		BooleanValue stase = em.getStaseModeValue();
		assertNotNull(stase);
		assertFalse(stase.get());
		
		BooleanValue evoEnabled = em.getEvolutionEnabledValue();
		assertNotNull(evoEnabled);
		assertFalse(evoEnabled.get());
		
		AdapterEvaluationMethod evm = new AdapterEvaluationMethod();
		em.setEvaluationMethod(evm);
		assertSame(em.getEvaluationMethod(), evm);
		
		NeuralNetAgentInterface agent = new NeuralNetAgentInterface("");
		agent.addInputNeuron(new NeuronWrapper("N1", 0.0, 0.0, 1.0));
		agent.addOutputNeuron(new NeuronWrapper("N2", 0.0, 0.0, 1.0));
		Simulator.getInstance().getNeuralNetManager().setNeuralNetInterface(agent);
		
		assertTrue(em.addEvolutionOperator(eo1, 50));
		assertTrue(em.addEvolutionOperator(eo2, 20));
		assertTrue(em.addSelectionMethod(sm1));
		assertTrue(em.addSelectionMethod(sm2));
		
		eo1.mNumberOfFalseExecutions = 7;
		eo1.mLeftNumberOfFalseExecutions = 7;
		eo1.getMaxNumberOfApplicationsValue().set(10);
		
		eo2.mNumberOfFalseExecutions = 2;
		eo2.mLeftNumberOfFalseExecutions = 2;
		eo2.getMaxNumberOfApplicationsValue().set(5);
		
		Simulator.getInstance().getScheduler().getNumberOfStepsValue().set(123);
		Simulator.getInstance().getScheduler().getNumberOfTriesValue().set(43);
		
		TestEvolutionExecution tee = new TestEvolutionExecution();
		em.addEvolutionListener(tee);
		Simulator.getInstance().getScheduler().addGenerationListener(tee);
		Simulator.getInstance().getScheduler().addIndividualListener(tee);
		em.setEvolutionTester(tee);
		em.getMeanPopulationSizeValue().set(10);
		assertTrue(Simulator.getInstance().init());
		
		em.runEvolution();
		
		
		
	}
	
	
	public class TestEvolutionExecution implements GenerationListener, EvolutionListener,
							IndividualListener, EvolutionTesterInterface  {
	
		private EvolutionManager mEvo;
		private int mCountRunEvolutionPausedCalls;
		private int mCountPreparationsCompletedCalls;
		private int mCountParentsSelectedCalls;
		private int mCountChildrenEvaluated;
		private int mCountGenerationStartedCalls;
		private int mCountGenerationCompletedCalls;
		private int mCountIndividualsCreated;
		private int mCountIndividualsStarted;
		private int mCountIndividualsCompleted;
		
		private AdapterEvolutionOperator mOp1;
		private AdapterEvolutionOperator mOp2;
		private AdapterSelectionMethod mSm1;
		private AdapterSelectionMethod mSm2;
		
		private int mExecMarker;
		
		
		public TestEvolutionExecution() {
			mCountRunEvolutionPausedCalls = 0;
			mCountPreparationsCompletedCalls = 0;
			mCountParentsSelectedCalls = 0;
			mEvo = Simulator.getInstance().getEvolutionManager();
			
			assertEquals(mEvo.getEvolutionOperators().size(), 2);
			mOp1 = (AdapterEvolutionOperator) mEvo.getEvolutionOperators().firstElement();
			mOp2 = (AdapterEvolutionOperator) mEvo.getEvolutionOperators().lastElement();
			
			assertEquals(mEvo.getSelectionMethods().size(), 2);
			mSm1 = (AdapterSelectionMethod) mEvo.getSelectionMethods().firstElement();
			mSm2 = (AdapterSelectionMethod) mEvo.getSelectionMethods().lastElement();
			
			assertNotNull(mOp1);
			assertNotNull(mOp2);
			assertNotNull(mSm1);
			assertNotNull(mSm2);
			
			mSm1.getOffspringPercentageValue().set(60);
			assertEquals(mSm1.getOffspringPercentageValue().get(), 60.0, 0.001);
			assertEquals(mSm2.getOffspringPercentageValue().get(), 40.0, 0.001);
			
			mExecMarker = 11;
		}
		
		

		public void runEvolutionPaused() {
			if(mCountRunEvolutionPausedCalls == 0) {
				assertTrue(mEvo.getEvolutionEnabledValue().get());
				assertFalse(mEvo.getEvolutionRunningValue().get());
				assertEquals(mEvo.getGeneration().size(), 0);
				assertNull(Simulator.getInstance().getNeuralNetManager().getCurrentNet());
				assertEquals(mExecMarker, 11);
				mExecMarker = 12;
			}
			if(mCountRunEvolutionPausedCalls == 1) {
				mEvo.getRunEvolutionValue().set(true);
				assertEquals(mExecMarker, 12);
				mExecMarker = 15;
			}
			else if(mCountRunEvolutionPausedCalls == 2) {
				//after the generation
				assertEquals(mExecMarker, 160);
				mExecMarker = 170;
			}
			if(mCountRunEvolutionPausedCalls >= 10) {
				mEvo.getEvolutionEnabledValue().set(false);
			}
			
			mCountRunEvolutionPausedCalls++;
		}
		
		

		public void preparationsCompleted() {
			if(mCountPreparationsCompletedCalls == 0) {
				assertEquals(mExecMarker, 15);
				mExecMarker = 20;
				
				mEvo.getRunEvolutionValue().set(false);
				
				assertTrue(mEvo.getEvolutionEnabledValue().get());
				assertTrue(mEvo.getEvolutionRunningValue().get());
				assertEquals(mEvo.getGeneration().size(), 1);
				assertNull(Simulator.getInstance().getNeuralNetManager().getCurrentNet());
				
				//Check evoIndividual and neural net.
				EvolutionIndividual ind = (EvolutionIndividual) mEvo.getGeneration().firstElement();
				assertNotNull(ind.getGenom());
				assertTrue(ind.getGenom() instanceof NeuralNet);
				NeuralNet net = (NeuralNet) ind.getGenom();
				
				assertEquals(net.getNeurons().size(), 2);
				assertEquals(net.getSynapses().size(), 0);
				
				assertEquals(((Neuron) net.getNeurons().firstElement()).getActivationValue().getName(), "N1");
				assertEquals(((Neuron) net.getNeurons().lastElement()).getActivationValue().getName(), "N2");
				
				
			}
			
			mCountPreparationsCompletedCalls++;
		}



		public void generationCompleted(int generationId) {
			if(mCountGenerationCompletedCalls == 0) {
				assertEquals(mExecMarker, 108);
				assertEquals(generationId, 0);
				mExecMarker = 110;
			}
			else if(mCountGenerationCompletedCalls > 0) {
				fail("");
			}
			
			mCountGenerationCompletedCalls++;
		}

		public void generationStarted(int generationId) {
			if(mCountGenerationStartedCalls == 0) {
				assertEquals(mExecMarker, 30);
				mExecMarker = 40;
				assertEquals(generationId, 0);
				assertEquals(Simulator.getInstance().getScheduler().getNumberOfSteps(), 123);
				assertEquals(Simulator.getInstance().getScheduler().getNumberOfTries(), 43);
				
				assertEquals(mEvo.getPreserveParentsValue().get(), 2);
			}
			mCountGenerationStartedCalls++;
		}

		public void notifyIndividualsCreated(Vector individuals, int generation) {
			if(mCountIndividualsCreated == 0) {
				assertEquals(mExecMarker, 40);
				mExecMarker = 50;
				
				assertTrue(individuals.size() == 9);
				assertEquals(mEvo.getCurrentPopulationSizeValue().get(), 9);
				
				assertEquals(mOp1.mExecutionCount, 8 * 8); //8 new individuals times 7 false executions + 1 true execution.
				assertEquals(mOp2.mExecutionCount, 5 * 8); //only 5 times executed
				
				//first individual is not changed by operators (was a parent)
				EvolutionIndividual ind = (EvolutionIndividual) individuals.firstElement();
				assertNotNull(ind);
				assertTrue(ind.getGenom() instanceof NeuralNet);
				
				for(int i = 1; i < individuals.size(); ++i) {
					ind = (EvolutionIndividual) individuals.elementAt(i);
					assertSame(ind.getGenom(), mGenom);
				}
			}
			
			mCountIndividualsCreated++;
		}

		public void notifyIndividualsEvaluated(Vector individuals, int generation) {
			if(mCountChildrenEvaluated == 0) {
				assertEquals(mExecMarker, 150);
				mExecMarker = 160;
				assertEquals(individuals.size(), 9);
				assertEquals(generation, 0);
			}
		}

		public void notifyParentsSelected(Vector parents, int generation) {
			if(mCountParentsSelectedCalls == 0) {
				assertEquals(mExecMarker, 20);
				mExecMarker = 30;
				assertEquals(parents.size(), 1);
				assertEquals(generation, 0);
				
				assertEquals(mSm1.mCountSelectParentsCalls, 1);
				assertEquals(mSm2.mCountSelectParentsCalls, 1);
				assertEquals(mSm1.mCountCreateChildrenStubsCalls, 1);
				assertEquals(mSm2.mCountCreateChildrenStubsCalls, 1);
				
				assertEquals(mSm1.mCreateChildrenNumberOfChildren, 5);
				assertEquals(mSm2.mCreateChildrenNumberOfChildren, 3);
				
				assertEquals(mSm1.mSelectParentsIndivs.size(), 1);
				assertEquals(mSm2.mSelectParentsIndivs.size(), 1);
				assertEquals(mSm1.mCreateChildrenParents.size(), 1);
				
				assertEquals(mEvo.getGeneration().size(), 1);
				
			}
			
			mCountParentsSelectedCalls++;
		}

		public void individualCompleted(int individualId) {
			if(mCountIndividualsCompleted >= 0 && mCountIndividualsCompleted < 5) {
				assertEquals(individualId, mCountIndividualsCompleted);
				assertEquals(mExecMarker, 50 + mCountIndividualsCompleted);
				mExecMarker = 50 + mCountIndividualsCompleted + 1;
			}
			mCountIndividualsCompleted++;
		}

		public void individualStarted(int individualId) {
			if(mCountIndividualsStarted >= 0 && mCountIndividualsStarted < 5) {
				assertEquals(individualId, mCountIndividualsStarted);
			}
			//Stase long
			else if(mCountIndividualsStarted == 5) {
				assertEquals(individualId, 5);
				assertEquals(mExecMarker, 50 + mCountIndividualsStarted);
				mExecMarker = 100;
				mEvo.getStaseModeValue().set(true);
			}
			else if(mCountIndividualsStarted == 6) {
				assertEquals(individualId, 5);
				assertEquals(mExecMarker, 100);
				mExecMarker = 101;
			}
			else if(mCountIndividualsStarted == 7) {
				assertEquals(individualId, 5);
				assertEquals(mExecMarker, 101);
				mExecMarker = 102;
			}
			else if(mCountIndividualsStarted == 8) {
				assertEquals(individualId, 5);
				assertEquals(mExecMarker, 102);
				mExecMarker = 103;
				mEvo.getStaseModeValue().set(false);
			}
			else if(mCountIndividualsStarted == 9) {
				assertEquals(individualId, 5);
				assertEquals(mExecMarker, 103);
				mExecMarker = 104;
			}
			else if(mCountIndividualsStarted == 10) {
				assertEquals(individualId, 6);
				assertEquals(mExecMarker, 104);
				mExecMarker = 105;
			}
			//Stase instantanous
			else if(mCountIndividualsStarted == 11) {
				assertEquals(individualId, 7);
				assertEquals(mExecMarker, 105);
				mExecMarker = 106;
				mEvo.getStaseModeValue().set(true);
				mEvo.getStaseModeValue().set(false);
			}
			else if(mCountIndividualsStarted == 12) {
				assertEquals(individualId, 7);
				assertEquals(mExecMarker, 106);
				mExecMarker = 107;
			}
			else if(mCountIndividualsStarted == 13) {
				assertEquals(individualId, 8);
				assertEquals(mExecMarker, 107);
				mExecMarker = 108;
				mEvo.restartCurrentGeneration();
			}
			//restartGeneration
			else if(mCountIndividualsStarted == 14) {
				assertEquals(individualId, 0);
				assertEquals(mExecMarker, 110);
				mExecMarker = 120;
			}
			else if(mCountIndividualsStarted >= 15 && mCountIndividualsStarted < 22) {
				assertEquals(mExecMarker, 120 + individualId - 1);
				mExecMarker = 120 + individualId;
			}
			else if(mCountIndividualsStarted == 22) {
				assertEquals(mExecMarker, 120 + individualId - 1);
				mExecMarker = 150;
			}
			
			
			
			mCountIndividualsStarted++;
		}
		
		
	
	}
	
	
	
	
	public static int mExecutionCounter = 0;
	public static Object mGenom = "";
	
	public class AdapterEvolutionOperator extends EvolutionOperator {

		public int mExecutionOrder;
		public int mExecutionCount;
		public int mNumberOfFalseExecutions;
		public int mLeftNumberOfFalseExecutions;
		
		public AdapterEvolutionOperator() {
			super("AdapterEvolutionOperator", 5);
			mExecutionOrder = 0;
			mGenom = "Genom";
			mExecutionCounter = 0;
		}
		
		public boolean executeOperation(EvolutionIndividual individual) {
			mExecutionCount++;
			mExecutionOrder = mExecutionCounter++;
			individual.setGenom(mGenom);
			if(mLeftNumberOfFalseExecutions > 0) {
				mLeftNumberOfFalseExecutions--;
				
				return false;
			}
			mLeftNumberOfFalseExecutions = mNumberOfFalseExecutions;
			return true;
		}
	}
	
	public class AdapterSelectionMethod extends SelectionMethod {

		public int mExecutionOrder;
		public int mCountCreateChildrenStubsCalls;
		public int mCountSelectParentsCalls;
		public Vector mChildren;
		public Vector mParents;
		public int mCreateChildrenNumberOfChildren;
		public Vector mSelectParentsIndivs;
		public Vector mCreateChildrenParents;
		
		public AdapterSelectionMethod() {
			super("AdapterSelectionMethod");
		}
		
		public Vector createChildrenStubs(Vector parents, int numberOfChildren) {
			mExecutionOrder = mExecutionCounter++;
			mCountCreateChildrenStubsCalls++;
			mCreateChildrenNumberOfChildren = numberOfChildren;
			mCreateChildrenParents = parents;
			if(mChildren != null) {
				return mChildren;
			}
			else {
				Vector children = new Vector();
				for(int i = 0; i < numberOfChildren; ++i) {
					EvolutionIndividual ind = new EvolutionIndividual(null);
					ind.addParent((EvolutionIndividual) parents.firstElement()); 
					children.add(ind);
					
				}
				return children;
			}
		}

		public Vector selectParents(Vector individuals) {
			mExecutionOrder = mExecutionCounter++;
			mCountSelectParentsCalls++;
			mSelectParentsIndivs = individuals;
			if(mParents != null) {
				return mParents;
			}
			else {
				Vector parents = new Vector();
				for(int i = 0; i < individuals.size() && i < 2; ++i) {
					parents.add(individuals.elementAt(i));
				}
				return parents;
			}
		}
	}
	
	public class AdapterEvaluationMethod extends EvaluationMethod {
		
		public int mCountEvaluations;
		public Vector mIndividuals = new Vector();
		
		public AdapterEvaluationMethod() {
			super("");
		}

		public void evaluateIndividual(EvolutionIndividual individual, int individualId) {
			mCountEvaluations++;
			mIndividuals.add(individual);
			Simulator.getInstance().getScheduler().prepareNextIndividual(individualId);
		}

		public void prepareNextGeneration(int generationId) {
		}
		
		
		
		
	}

}
