/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.test;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.value.HistoryValue;
import junit.framework.TestCase;

public class TestHistoryValue extends TestCase {

	protected void setUp() throws Exception {
		super.tearDown();
		Simulator.getInstance().clearInstance();
	}
	
	protected void tearDown() throws Exception {
		super.tearDown();
		Simulator.getInstance().clearInstance();
	}
	
	public void testHistory() {
		HistoryValue value = new HistoryValue();
		
		assertEquals(value.get(), 0.0, 0.0001);
		assertEquals(value.getHistory().size(), 1);
		assertEquals(((Double) value.getHistory().elementAt(0)).doubleValue(), 0.0, 0.0001);
		
		value.set(99.23);
		assertEquals(value.get(), 99.23, 0.0001);
		assertEquals(value.getHistory().size(), 1);
		assertEquals(((Double) value.getHistory().elementAt(0)).doubleValue(), 99.23, 0.0001);
		
		value.generationStarted(3);
		assertEquals(value.getHistory().size(), 4);
		assertEquals(((Double) value.getHistory().elementAt(0)).doubleValue(), 99.23, 0.0001);
		assertNull(value.getHistory().elementAt(1));
		assertNull(value.getHistory().elementAt(2));
		assertEquals(((Double) value.getHistory().elementAt(3)).doubleValue(), 0.0, 0.0001);
		
		
		value.set(11.22);
		assertEquals(value.get(), 11.22, 0.0001);
		assertEquals(value.getHistory().size(), 4);
		assertEquals(((Double) value.getHistory().elementAt(0)).doubleValue(), 99.23, 0.0001);
		assertNull(value.getHistory().elementAt(1));
		assertNull(value.getHistory().elementAt(2));
		assertEquals(((Double) value.getHistory().elementAt(3)).doubleValue(), 11.22, 0.0001);
		
		value.set(22.11);
		assertEquals(value.get(), 22.11, 0.0001);
		assertEquals(value.getHistory().size(), 4);
		assertEquals(((Double) value.getHistory().elementAt(0)).doubleValue(), 99.23, 0.0001);
		assertNull(value.getHistory().elementAt(1));
		assertNull(value.getHistory().elementAt(2));
		assertEquals(((Double) value.getHistory().elementAt(3)).doubleValue(), 22.11, 0.0001);
		
		value.generationStarted(0);
		value.set(5.55);
		assertEquals(value.get(), 5.55, 0.0001);
		assertEquals(value.getHistory().size(), 4);
		assertEquals(((Double) value.getHistory().elementAt(0)).doubleValue(), 5.55, 0.0001);
		assertNull(value.getHistory().elementAt(1));
		assertNull(value.getHistory().elementAt(2));
		assertEquals(((Double) value.getHistory().elementAt(3)).doubleValue(), 22.11, 0.0001);
		
		value.set(654.765, 2);
		assertEquals(value.get(), 5.55, 0.0001);
		assertEquals(value.getHistory().size(), 4);
		assertEquals(((Double) value.getHistory().elementAt(0)).doubleValue(), 5.55, 0.0001);
		assertNull(value.getHistory().elementAt(1));
		assertEquals(((Double) value.getHistory().elementAt(2)).doubleValue(), 654.765, 0.0001);
		assertEquals(((Double) value.getHistory().elementAt(3)).doubleValue(), 22.11, 0.0001);
		
	}
	
	public void testGenerationListener() {
		HistoryValue value = new HistoryValue(100.5);
		assertEquals(value.getHistory().size(), 1);
		assertEquals(((Double) value.getHistory().elementAt(0)).doubleValue(), 100.5, 0.0001);
		
		Simulator.getInstance().getScheduler().prepareNextGeneration(4, 10, 10);
		
		value.set(11.22);
		assertEquals(value.get(), 11.22, 0.0001);
		assertEquals(value.getHistory().size(), 5);
		assertEquals(((Double) value.getHistory().elementAt(0)).doubleValue(), 100.5, 0.0001);
		assertNull(value.getHistory().elementAt(1));
		assertNull(value.getHistory().elementAt(2));
		assertNull(value.getHistory().elementAt(3));
		assertEquals(((Double) value.getHistory().elementAt(4)).doubleValue(), 11.22, 0.0001);
		
	}
}
