/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.test;

import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.neuralnet.NetworkLibrary;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetAgentInterface;
import de.fhg.iais.outdoor.simba.neuralnet.Neuron;
import de.fhg.iais.outdoor.simba.neuralnet.NeuronWrapper;
import de.fhg.iais.outdoor.simba.neuralnet.Synapse;
import de.fhg.iais.outdoor.simba.neuralnet.TransferFunctionRamp;
import junit.framework.TestCase;

public class TestNetworkLibrary extends TestCase {

	protected void setUp() throws Exception {
		super.tearDown();
		Simulator.getInstance().clearInstance();
	}
	
	protected void tearDown() throws Exception {
		super.tearDown();
		Simulator.getInstance().clearInstance();
	}
	
	public void testSortNeurons() {
		
		Vector neurons = new Vector();
		
		Neuron n1 = new Neuron("n1", new TransferFunctionRamp(), Neuron.OUTPUT);
		Neuron n2 = new Neuron("n2", new TransferFunctionRamp(), Neuron.HIDDEN);
		Neuron n3 = new Neuron("n3", new TransferFunctionRamp(), Neuron.HIDDEN);
		Neuron n4 = new Neuron("n4", new TransferFunctionRamp(), Neuron.INPUT);
		Neuron n5 = new Neuron("n5", new TransferFunctionRamp(), Neuron.HIDDEN);
		Neuron n6 = new Neuron("n6", new TransferFunctionRamp(), Neuron.INPUT);
		Neuron n7 = new Neuron("n7", new TransferFunctionRamp(), Neuron.OUTPUT);
		
		NeuralNetAgentInterface agentInterface = new NeuralNetAgentInterface("");
		NeuronWrapper wi1 = new NeuronWrapper("", 0.0, 0.0, 1.0);
		NeuronWrapper wi2 = new NeuronWrapper("", 0.0, 0.0, 1.0);
		NeuronWrapper wo1 = new NeuronWrapper("", 0.0, 0.0, 1.0);
		NeuronWrapper wo2 = new NeuronWrapper("", 0.0, 0.0, 1.0);
		agentInterface.addInputNeuron(wi1);
		agentInterface.addInputNeuron(wi2);
		agentInterface.addOutputNeuron(wo1);
		agentInterface.addOutputNeuron(wo2);
		
		n4.setActivationValue(wi1);
		n6.setActivationValue(wi2);
		n7.setActivationValue(wo1);
		n1.setActivationValue(wo2);
		
		Synapse s1 = new Synapse(n2, 0.7);
		n5.addSynapse(s1);
		
		Synapse s2 = new Synapse(n2, 0.3);
		Synapse s3 = new Synapse(n4, 1.2);
		n3.addSynapse(s2);
		n3.addSynapse(s3);
		
		neurons.add(n1);
		neurons.add(n2);
		neurons.add(n3);
		neurons.add(n4);
		neurons.add(n5);
		neurons.add(n6);
		neurons.add(n7);
		
		Vector sorted = NetworkLibrary.sortNeurons(neurons, agentInterface);
		
		assertEquals(sorted.size(), 7);
		assertSame(sorted.elementAt(0), n4);
		assertSame(sorted.elementAt(1), n6);
		assertSame(sorted.elementAt(2), n3);
		assertSame(sorted.elementAt(3), n5);
		assertSame(sorted.elementAt(4), n2);
		assertSame(sorted.elementAt(5), n7);
		assertSame(sorted.elementAt(6), n1);
		
		Synapse s4 = new Synapse(n6, 1.2);
		n5.addSynapse(s4);
		
		assertEquals(n5.getSynapses().size(), n3.getSynapses().size());
		
		sorted = NetworkLibrary.sortNeurons(neurons, agentInterface);
		
		assertEquals(sorted.size(), 7);
		assertSame(sorted.elementAt(0), n4);
		assertSame(sorted.elementAt(1), n6);
		assertSame(sorted.elementAt(2), n5);
		assertSame(sorted.elementAt(3), n3);
		assertSame(sorted.elementAt(4), n2);
		assertSame(sorted.elementAt(5), n7);
		assertSame(sorted.elementAt(6), n1);
		
		s2.setStrength(0.9);
		sorted = NetworkLibrary.sortNeurons(neurons, agentInterface);
		
		assertEquals(sorted.size(), 7);
		assertSame(sorted.elementAt(0), n4);
		assertSame(sorted.elementAt(1), n6);
		assertSame(sorted.elementAt(2), n3);
		assertSame(sorted.elementAt(3), n5);
		assertSame(sorted.elementAt(4), n2);
		assertSame(sorted.elementAt(5), n7);
		assertSame(sorted.elementAt(6), n1);
		
		
		
	}
	
	
	public void testSortSynapses() {
		Synapse s1 = new Synapse(null, 0.1);
		Synapse s2 = new Synapse(null, 2.5);
		Synapse s3 = new Synapse(null, -2.5);
		Synapse s4 = new Synapse(null, -0.5);
		
		Vector synapses = new Vector();
		synapses.add(s1);
		synapses.add(s2);
		synapses.add(s3);
		synapses.add(s4);
		
		Vector sorted = NetworkLibrary.sortSynapses(synapses);
		
		assertEquals(4, sorted.size());
		assertSame(sorted.elementAt(0), s2);
		assertSame(sorted.elementAt(1), s1);
		assertSame(sorted.elementAt(2), s4);
		assertSame(sorted.elementAt(3), s3);
		
	}
	
	public void testFootprint() {
		NeuralNet net = new NeuralNet();
		
		Neuron n1 = new Neuron("n1", new TransferFunctionRamp(), Neuron.OUTPUT);
		Neuron n2 = new Neuron("n2", new TransferFunctionRamp(), Neuron.HIDDEN);
		Neuron n3 = new Neuron("n3", new TransferFunctionRamp(), Neuron.HIDDEN);
		Neuron n4 = new Neuron("n4", new TransferFunctionRamp(), Neuron.INPUT);
		Neuron n5 = new Neuron("n5", new TransferFunctionRamp(), Neuron.HIDDEN);
		Neuron n6 = new Neuron("n6", new TransferFunctionRamp(), Neuron.INPUT);
		Neuron n7 = new Neuron("n7", new TransferFunctionRamp(), Neuron.OUTPUT);
		
		NeuralNetAgentInterface agentInterface = new NeuralNetAgentInterface("");
		NeuronWrapper wi1 = new NeuronWrapper("", 0.0, 0.0, 1.0);
		NeuronWrapper wi2 = new NeuronWrapper("", 0.0, 0.0, 1.0);
		NeuronWrapper wo1 = new NeuronWrapper("", 0.0, 0.0, 1.0);
		NeuronWrapper wo2 = new NeuronWrapper("", 0.0, 0.0, 1.0);
		agentInterface.addInputNeuron(wi1);
		agentInterface.addInputNeuron(wi2);
		agentInterface.addOutputNeuron(wo1);
		agentInterface.addOutputNeuron(wo2);
		
		n4.setActivationValue(wi1);
		n6.setActivationValue(wi2);
		n7.setActivationValue(wo1);
		n1.setActivationValue(wo2);
		
		Synapse s1 = new Synapse(n2, 0.7);
		n5.addSynapse(s1);
		
		Synapse s2 = new Synapse(n2, 0.3);
		Synapse s3 = new Synapse(n4, 1.2);
		n3.addSynapse(s2);
		n3.addSynapse(s3);
		
		net.addNeuron(n1);
		net.addNeuron(n2);
		net.addNeuron(n3);
		net.addNeuron(n4);
		net.addNeuron(n5);
		net.addNeuron(n6);
		net.addNeuron(n7);
		
		System.out.println("SystemFootprint: " + NetworkLibrary.getNetworkFingerPrint(net));
		
		Synapse s4 = new Synapse(n6, 1.2);
		n5.addSynapse(s4);
		
		System.out.println("SystemFootprint: " + NetworkLibrary.getNetworkFingerPrint(net));
		
		
		
		NeuralNet net2 = new NeuralNet();
		
		Neuron na1 = new Neuron("n1", new TransferFunctionRamp(), Neuron.OUTPUT);
		Neuron na2 = new Neuron("n2", new TransferFunctionRamp(), Neuron.HIDDEN);
		Neuron na3 = new Neuron("n3", new TransferFunctionRamp(), Neuron.HIDDEN);
		Neuron na4 = new Neuron("n4", new TransferFunctionRamp(), Neuron.INPUT);
		Neuron na5 = new Neuron("n5", new TransferFunctionRamp(), Neuron.HIDDEN);
		Neuron na6 = new Neuron("n6", new TransferFunctionRamp(), Neuron.INPUT);
		Neuron na7 = new Neuron("n7", new TransferFunctionRamp(), Neuron.OUTPUT);
			
		na4.setActivationValue(wi1);
		na6.setActivationValue(wi2);
		na7.setActivationValue(wo1);
		na1.setActivationValue(wo2);
		
		Synapse sa1 = new Synapse(na5, 0.7);
		Synapse sa4 = new Synapse(n6, 1.2);
		na2.addSynapse(sa1);
		na2.addSynapse(sa4);
		
		Synapse sa2 = new Synapse(na5, 0.3);
		Synapse sa3 = new Synapse(na4, 1.2);
		na3.addSynapse(sa2);
		na3.addSynapse(sa3);
		
		
		net2.addNeuron(na1);
		net2.addNeuron(na2);
		net2.addNeuron(na3);
		net2.addNeuron(na4);
		net2.addNeuron(na5);
		net2.addNeuron(na6);
		net2.addNeuron(na7);
		
		System.out.println("SystemFootprint: " + NetworkLibrary.getNetworkFingerPrint(net2));
	}
	
}
