/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.test;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetAgentInterface;
import de.fhg.iais.outdoor.simba.neuralnet.NeuronWrapper;
import junit.framework.TestCase;

public class TestNeuralNetAgentInterface extends TestCase {

	protected void setUp() throws Exception {
		super.setUp();
		Simulator.getInstance().clearInstance();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
		Simulator.getInstance().clearInstance();
	}
	
	public void testConstructor() {
		NeuralNetAgentInterface controller = new NeuralNetAgentInterface("NeuralNetController1");
		assertEquals(controller.getName(), "NeuralNetController1");
		assertEquals(controller.getInputNeurons().size(), 0);
		assertEquals(controller.getOutputNeurons().size(), 0);
	}

	public void testAddInputOutputNeuron() {
		NeuralNetAgentInterface controller = new NeuralNetAgentInterface("");
		
		assertEquals(controller.getInputNeurons().size(), 0);
		assertEquals(controller.getOutputNeurons().size(), 0);
		
		NeuronWrapper w1 = new NeuronWrapper("Name", 1.0, 0.5, 1.5);
		NeuronWrapper w2 = new NeuronWrapper("Name2", 50.0, 25.0, 125.0);
		
		assertFalse(controller.addInputNeuron(null));
		assertTrue(controller.addInputNeuron(w1));
		assertEquals(controller.getInputNeurons().size(), 1);
		assertFalse(controller.addInputNeuron(w1));
		
		assertTrue(controller.addOutputNeuron(w1));
		assertEquals(controller.getInputNeurons().size(), 1);
		assertEquals(controller.getOutputNeurons().size(), 1);
		
		assertTrue(controller.addInputNeuron(w2));
		assertEquals(controller.getInputNeurons().size(), 2);
		
		assertTrue(controller.addOutputNeuron(w2));
		assertEquals(controller.getOutputNeurons().size(), 2);
		
	}

	

	public void testSetNeuronRange() {
		NeuralNetAgentInterface controller = new NeuralNetAgentInterface("");
		NeuronWrapper w1 = new NeuronWrapper("Name", 1.0, 0.5, 1.5);
		NeuronWrapper w2 = new NeuronWrapper("Name2", 50.0, 25.0, 125.0);
		
		controller.addInputNeuron(w1);
		controller.addOutputNeuron(w2);
		
		assertEquals(w1.get(), 1.0, 0.001);
		assertEquals(w1.getNormalized(), 0.5, 0.001);
		assertEquals(w1.getMinNormalizedValue(), 0.0, 0.001);
		assertEquals(w1.getMaxNormalizedValue(), 1.0, 0.001);
		
		assertEquals(w2.get(), 50.0, 0.001);
		assertEquals(w2.getNormalized(), 0.25, 0.001);
		assertEquals(w2.getMinNormalizedValue(), 0.0, 0.001);
		assertEquals(w2.getMaxNormalizedValue(), 1.0, 0.001);
		
		controller.setNeuronRange(-2.0, 2.0);
		
		assertEquals(w1.get(), 1.0, 0.001);
		assertEquals(w1.getNormalized(), 0.0, 0.001);
		assertEquals(w1.getMinNormalizedValue(), -2.0, 0.001);
		assertEquals(w1.getMaxNormalizedValue(), 2.0, 0.001);
		
		assertEquals(w2.get(), 50.0, 0.001);
		assertEquals(w2.getNormalized(), -1.0, 0.001);
		assertEquals(w2.getMinNormalizedValue(), -2.0, 0.001);
		assertEquals(w2.getMaxNormalizedValue(), 2.0, 0.001);
		
		
	}

}
