/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.test;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.value.NormalizedDoubleValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueChangedListener;
import junit.framework.TestCase;

public class TestNormalizedDoubleValue extends TestCase {

	protected void tearDown() throws Exception {
		super.tearDown();
		Simulator.getInstance().clearInstance();
	}

	public void testNormalizedDoubleValue() {
		NormalizedDoubleValue ndv = new NormalizedDoubleValue(30.5, -10.0, 40.0, -0.5, 0.5);
		assertEquals(ndv.get(), 30.5, 0.0001);
		assertEquals(ndv.getMinValue(), -10.0, 0.001);
		assertEquals(ndv.getMaxValue(), 40.0, 0.001);
		assertEquals(ndv.getMinNormalizedValue(), -0.5, 0.001);
		assertEquals(ndv.getMaxNormalizedValue(), 0.5, 0.001);
		
		ndv = new NormalizedDoubleValue(-50434.123, -10.0, 40.0, -0.5, 0.5);
		assertEquals(ndv.get(), -10.0, 0.0001);
		
		ndv = new NormalizedDoubleValue(23432, -10.0, 40.0, -0.5, 0.5);
		assertEquals(ndv.get(), 40.0, 0.0001);
		
		assertFalse(ndv.isInverted());
	}

	public void testGetAndSetNormalized() {
		NormalizedDoubleValue ndv = new NormalizedDoubleValue(15.0, -10.0, 40.0, -0.5, 0.5);
		
		assertEquals(ndv.get(), 15.0, 0.001);
		assertEquals(ndv.getNormalized(), 0.0, 0.001);
		
		//set the value
		
		ndv.set(-10.0);
		assertEquals(ndv.get(), -10.0, 0.001);
		assertEquals(ndv.getNormalized(), -0.5, 0.001);
		
		ndv.set(-50.0);
		assertEquals(ndv.get(), -10.0, 0.001);
		assertEquals(ndv.getNormalized(), -0.5, 0.001);
		
		ndv.set(40.0);
		assertEquals(ndv.get(),40.0, 0.001);
		assertEquals(ndv.getNormalized(), 0.5, 0.001);
		
		ndv.set(145.0);
		assertEquals(ndv.get(), 40.0, 0.001);
		assertEquals(ndv.getNormalized(), 0.5, 0.001);
		
		ndv.set(2.5);
		assertEquals(ndv.get(), 2.5, 0.001);
		assertEquals(ndv.getNormalized(), -0.25, 0.001);
		
		ndv.set(-5.0);
		assertEquals(ndv.get(), -5.0, 0.001);
		assertEquals(ndv.getNormalized(), -0.4, 0.001);
		
		//set the normalized value
		
		ndv.setNormalized(-0.5);
		assertEquals(ndv.getNormalized(), -0.5, 0.001);
		assertEquals(ndv.get(), -10, 0.001);
		
		ndv.setNormalized(-1.8);
		assertEquals(ndv.getNormalized(), -0.5, 0.001);
		assertEquals(ndv.get(), -10, 0.001);
		
		ndv.setNormalized(0.5);
		assertEquals(ndv.getNormalized(), 0.5, 0.001);
		assertEquals(ndv.get(), 40, 0.001);
		
		ndv.setNormalized(1.5);
		assertEquals(ndv.getNormalized(), 0.5, 0.001);
		assertEquals(ndv.get(), 40, 0.001);
		
		ndv.setNormalized(0.0);
		assertEquals(ndv.getNormalized(), 0.0, 0.001);
		assertEquals(ndv.get(), 15.0, 0.001);
		
		ndv.setNormalized(0.1);
		assertEquals(ndv.getNormalized(), 0.1, 0.001);
		assertEquals(ndv.get(), 20.0, 0.001);
		
		//change ranges
		
		ndv.set(0.0);
		ndv.setValueRange(-25.0, 75.0);
		assertEquals(ndv.getNormalized(), -0.25, 0.001);
		assertEquals(ndv.get(), 0.0, 0.001);
		
		ndv.setValueRange(50.0, 100.0);
		assertEquals(ndv.getNormalized(), -0.5, 0.001);
		assertEquals(ndv.get(), 50.0, 0.001);
		
		ndv.setValueRange(-100.0, -50);
		assertEquals(ndv.getNormalized(), 0.5, 0.001);
		assertEquals(ndv.get(), -50.0, 0.001);
		
		ndv.setValueRange(1000.0, 999.0); //min > max => no change.
		assertEquals(ndv.getNormalized(), 0.5, 0.001);
		assertEquals(ndv.get(), -50.0, 0.001);
		
		ndv.setNormalizedValueRange(-1.0, 0.0);
		assertEquals(ndv.getNormalized(), 0.0, 0.001);
		assertEquals(ndv.get(), -50.0, 0.001);
		
		ndv.setNormalized(-0.5);
		assertEquals(ndv.getNormalized(), -0.5, 0.001);
		assertEquals(ndv.get(), -75.0, 0.001);
		
		ndv.setNormalizedValueRange(100.0, 200.0);
		assertEquals(ndv.getNormalized(), 150.0, 0.001);
		assertEquals(ndv.get(), -75.0, 0.001);
		
		ndv.setNormalizedValueRange(1000.0, 999.9); //min > max => no change.
		assertEquals(ndv.getNormalized(), 150.0, 0.001);
		assertEquals(ndv.get(), -75.0, 0.001);
	}
	
	public void testNofification() {
		NormalizedDoubleValue ndv = new NormalizedDoubleValue(30.5, -10.0, 40.0, -0.5, 0.5);
		
		TestValueChangedListener l1 = new TestValueChangedListener();
		assertNull(l1.mLastReportedValue);
		
		ndv.addValueChangedListener(l1);
		
		ndv.set(50.0);
		assertSame(l1.mLastReportedValue, ndv);
		l1.mLastReportedValue = null;
		
		ndv.get();
		assertNull(l1.mLastReportedValue);
		
		ndv.getNormalized();
		assertNull(l1.mLastReportedValue);
		
		ndv.setNormalized(0.5);
		assertSame(l1.mLastReportedValue, ndv);
		
		l1.mLastReportedValue = null;
		assertNull(l1.mLastReportedValue);
		ndv.setValueRange(10000.0, 20000.0);
		assertSame(l1.mLastReportedValue, ndv); //because the value was changed.
		
		
	}

	public void testInversion() {
		NormalizedDoubleValue ndv = new NormalizedDoubleValue(-10.0, -30.0, 10.0, -0.5, 0.5);
		
		assertEquals(ndv.get(), -10.0, 0.001);
		assertEquals(ndv.getNormalized(), 0.0, 0.001);
		
		//set the value
		ndv.set(10.0);
		assertEquals(ndv.get(), 10.0, 0.001);
		assertEquals(ndv.getNormalized(), 0.5, 0.001);
		
		ndv.setInverted(true);
		assertTrue(ndv.isInverted());
		assertEquals(ndv.get(), 10.0, 0.001);
		assertEquals(ndv.getNormalized(), -0.5, 0.001);
		
		ndv.set(0);
		assertEquals(ndv.get(), 0.0, 0.001);
		assertEquals(ndv.getNormalized(), -0.25, 0.001);
		ndv.setInverted(false);
		assertEquals(ndv.getNormalized(), 0.25, 0.001);
		
		ndv.setInverted(true);
		assertEquals(ndv.getNormalized(), -0.25, 0.001);
		ndv.setNormalized(0.1);
		assertEquals(ndv.getNormalized(), 0.1, 0.001);
		assertEquals(ndv.get(), -14.0, 0.001);
	}
	
	private class TestValueChangedListener implements ValueChangedListener {
		public Value mLastReportedValue;
		public Value mLastOldValue;
		
		public void valueChanged(Value value) {
			mLastReportedValue = value;
		}
		
		/**
		 * Called if the one of the registered Value objects were replaced in the ValueManager.
		 */
		 public void valueReplaced(Value oldValue, Value newValue) {
			mLastOldValue = oldValue;
			mLastReportedValue = newValue;
		 }
	}
	

}
