/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.test;

import java.awt.Polygon;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.SimObject;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.DoubleValue;
import de.fhg.iais.outdoor.simba.value.PointValue;
import de.fhg.iais.outdoor.simba.value.SimObjectValue;
import de.fhg.iais.outdoor.simba.value.ValueManager;
import junit.framework.TestCase;

public class TestSimObject extends TestCase {

	
	protected void tearDown() throws Exception {
		super.tearDown();
		Simulator.getInstance().clearInstance();
	}
	
	/*
	 * Test method for 'de.fhg.iais.outdoor.simba.core.SimObject.SimObject()'
	 */
	public void testSimObject() {
		
		Simulator.getInstance().clearInstance();
		ValueManager vm = Simulator.getInstance().getValueManager();
		
		int countValues = vm.getValues().size();
		
		//Standard constructor.
		SimObject obj = new SimObject();
		assertEquals(obj.getAngle(), 0.0, 0.001);
		assertEquals(obj.getDependencies().size(), 0);
		assertEquals(obj.getLayer(), 0);
		assertTrue(obj.isActive());
		assertTrue(obj.isVisible());
		assertNull(obj.getReferenceObject());
		assertEquals(obj.getPosition(), new Point2D.Double(0.0, 0.0));
		assertEquals(obj.getName(), "SimObject");
		assertEquals(obj.getShape().npoints, 0);
		
		int numberOfValueAttributes = 8;
		assertEquals(vm.getValues().size(), 2 + numberOfValueAttributes + countValues);
		assertNotNull(vm.getSimObjectValue("/SimObjects/SimObject"));
		assertNotNull(vm.getDoubleValue("/SimObjects/SimObject/Transparency"));
		assertSame(vm.getSimObjectValue("/SimObjects/SimObject").get(), obj);
		
		
		//Setter and getter methods.
		obj.setAngle(34.0);
		assertEquals(obj.getAngle(), 34.0, 0.001);
		obj.addDependency(new SimObject());
		assertEquals(obj.getDependencies().size(), 1);
		obj.setLayer(15);
		assertEquals(obj.getLayer(), 15);
		obj.setActive(false);
		assertFalse(obj.isActive());
		obj.setVisible(false);
		assertFalse(obj.isVisible());
		SimObject o1 = new SimObject();
		obj.setReferenceObject(o1);
		assertSame(obj.getReferenceObject(), o1);
		obj.setPosition(44.5, 66.77);
		assertEquals(obj.getPosition(), new Point2D.Double(44.5, 66.77));
		obj.setPosition(new Point2D.Double(122.5, 233.77));
		assertEquals(obj.getPosition(), new Point2D.Double(122.5, 233.77));
		
		//Other constructor.
		obj = new SimObject(44.5, 66.77, 34.0, new Polygon());
		assertEquals(obj.getAngle(), 34.0, 0.001);
		assertEquals(obj.getDependencies().size(), 0);
		assertEquals(obj.getLayer(), 0);
		assertTrue(obj.isActive());
		assertTrue(obj.isVisible());
		assertNull(obj.getReferenceObject());
		assertEquals(obj.getPosition(), new Point2D.Double(44.5, 66.77));
		//there were 3 other simobjects created before.
		assertEquals(obj.getName(), "SimObject3");
		
		assertEquals(vm.getValues().size(), 4 * (numberOfValueAttributes + 2) + countValues);
		assertNotNull(vm.getSimObjectValue("/SimObjects/SimObject3"));
		assertNotNull(vm.getSimObjectValue("/SimObjects/SimObject2"));
		assertNotNull(vm.getSimObjectValue("/SimObjects/SimObject1"));
		assertNotNull(vm.getSimObjectValue("/SimObjects/SimObject"));
		assertSame(vm.getSimObjectValue("/SimObjects/SimObject3").get(), obj);
		
		obj = new SimObject("Obj");
		assertEquals(obj.getName(), "Obj");
		
		assertEquals(vm.getValues().size(), 5 * (numberOfValueAttributes + 2) + countValues);
		assertNotNull(vm.getSimObjectValue("/SimObjects/Obj"));
		assertSame(vm.getSimObjectValue("/SimObjects/Obj").get(), obj);
		
		obj = new SimObject("AnotherObject", 44.5, 66.77, 34.0, new Polygon());
		assertEquals(obj.getAngle(), 34.0, 0.001);
		assertEquals(obj.getDependencies().size(), 0);
		assertEquals(obj.getLayer(), 0);
		assertTrue(obj.isActive());
		assertTrue(obj.isVisible());
		assertNull(obj.getReferenceObject());
		assertEquals(obj.getPosition(), new Point2D.Double(44.5, 66.77));
		assertEquals(obj.getName(), "AnotherObject");
		
		assertEquals(vm.getValues().size(), 6 * (numberOfValueAttributes + 2) + countValues);
		assertNotNull(vm.getSimObjectValue("/SimObjects/AnotherObject"));
		assertSame(vm.getSimObjectValue("/SimObjects/AnotherObject").get(), obj);
		
	}


	/*
	 * Test method for 'de.fhg.iais.outdoor.simba.core.SimObject.reset()'
	 */
	public void testReset() {

	}

	/*
	 * Test method for 'de.fhg.iais.outdoor.simba.core.SimObject.getGlobalPosition()'
	 */
	public void testGetGlobalPosition() {
		SimObject o1 = new SimObject("", 2.0, 5.0, 90.0, new Polygon());
		SimObject o2 = new SimObject("", 1.0, 1.0, 90.0, new Polygon());
		SimObject o3 = new SimObject("", 2.0, 1.0, 90.0, new Polygon());
		SimObject o4 = new SimObject("", 1.0, -2.0, -540.0, new Polygon());
		
		assertEquals(o1.getGlobalPosition(), new Point2D.Double(2.0, 5.0));
		
		//without a call to update, there is no change!
		o1.setReferenceObject(o2);
		assertEquals(o1.getGlobalPosition(), new Point2D.Double(2.0, 5.0));
		
		//now with an update.
		o1.update();
		o2.update();
		
		assertEquals(o2.getGlobalPosition(), new Point2D.Double(1.0, 1.0));
		assertEquals(o1.getGlobalPosition(), new Point2D.Double(-4.0, 3.0));
		
		o1.update();
		o2.update();
		
		o2.setReferenceObject(o3);
		assertEquals(o3.getGlobalPosition(), new Point2D.Double(2.0, 1.0));
		assertEquals(o1.getGlobalPosition(), new Point2D.Double(-1.0, -3.0));
		
		o1.update();
		o2.update();
		o3.update();
		o4.update();
		
		o3.setReferenceObject(o4); //too large angles are truncated to 0 - 360
		assertEquals(o4.getGlobalPosition(), new Point2D.Double(1.0, -2.0));
		assertEquals(o1.getGlobalPosition(), new Point2D.Double(2.0, 1.0));
	}

	
	/*
	 * Test method for 'de.fhg.iais.outdoor.simba.core.SimObject.getGlobalAngle()'
	 */
	public void testGetGlobalAngle() {
		SimObject o1 = new SimObject("", 2.0, 1.0, 45.0, new Polygon());
		SimObject o2 = new SimObject("", 5.0, 3.0, 45.0, new Polygon());
		SimObject o3 = new SimObject("", -9.0, -6.0, 90.0, new Polygon());
		SimObject o4 = new SimObject("", 0.0, 0.0, 225.0, new Polygon());
		
		assertEquals(o1.getGlobalAngle(), 45.0, 0.001);
		
		o1.setReferenceObject(o2);
		
		//without an update, nothing changed
		assertEquals(o1.getGlobalAngle(), 45.0, 0.001);
		
		//now with update.
		o1.update();
		o2.update();
		assertEquals(o2.getGlobalAngle(), 45.0, 0.001);
		assertEquals(o1.getGlobalAngle(), 90.0, 0.001);
		
		o2.setReferenceObject(o3);
		o1.update();
		o2.update();
		o3.update();
		assertEquals(o3.getGlobalAngle(), 90.0, 0.001);
		assertEquals(o1.getGlobalAngle(), 180.0, 0.001);
		
		o3.setReferenceObject(o4);
		o1.update();
		o2.update();
		o3.update();
		o4.update();
		assertEquals(o4.getGlobalAngle(), 225.0, 0.001);
		assertEquals(o1.getGlobalAngle(), 45.0, 0.001);
		

	}

	/*
	 * Test method for 'de.fhg.iais.outdoor.simba.core.SimObject.getDependencies()'
	 */
	public void testAddAndGetDependencies() {
		SimObject o1 = new SimObject();
		SimObject o2 = new SimObject();
		SimObject o3 = new SimObject();
		
		assertEquals(o1.getDependencies().size(), 0);
		assertTrue(o1.addDependency(o2));
		assertEquals(o1.getDependencies().size(), 1);
		assertFalse(o1.addDependency(o2));
		assertEquals(o1.getDependencies().size(), 1);
		assertTrue(o1.addDependency(o3));
		assertEquals(o1.getDependencies().size(), 2);
		
	}

	/*
	 * Test method for 'de.fhg.iais.outdoor.simba.core.SimObject.setActive(boolean)'
	 */
	public void testSetActive() {
		SimObject obj = new SimObject();
		assertTrue(obj.isActive());
		
		obj.setActive(false);
		assertFalse(obj.isActive());
		
		obj.setActive(true);
		assertTrue(obj.isActive());
	}
	
	public void testValueReplacement() {
		ValueManager vm = Simulator.getInstance().getValueManager();
		
		SimObject obj = new SimObject("MyObject", 1.0, 2.0, 3.0, new Polygon());
		assertNotNull(vm.getDoubleValue("/SimObjects/MyObject/Transparency"));
		DoubleValue trans1 = vm.getDoubleValue("/SimObjects/MyObject/Transparency");
		PointValue pos = vm.getPointValue("/SimObjects/MyObject/LocalPosition");
		DoubleValue angle = vm.getDoubleValue("/SimObjects/MyObject/LocalAngle");
		PointValue initPos = vm.getPointValue("/SimObjects/MyObject/InitPosition");
		DoubleValue initAngle = vm.getDoubleValue("/SimObjects/MyObject/InitAngle");
		BooleanValue active = vm.getBooleanValue("/SimObjects/MyObject/Active");
		BooleanValue visible = vm.getBooleanValue("/SimObjects/MyObject/Visible");
		SimObjectValue reference = vm.getSimObjectValue("/SimObjects/MyObject/ReferenceObject");
		
		obj.setShapeTransparency(0.2);
		assertEquals(obj.getShapeTransparency(), 0.2, 0.001);
		assertEquals(trans1.get(), 0.2, 0.001);
		trans1.set(0.4);
		assertEquals(obj.getShapeTransparency(), 0.4, 0.001);
		assertEquals(trans1.get(), 0.4, 0.001);
		
		//Change the DoubleValue at the ValueManager
		DoubleValue trans2 = new DoubleValue(0.99);
		assertTrue(vm.replaceValue("/SimObjects/MyObject/Transparency", trans2));
		assertEquals(obj.getShapeTransparency(), 0.99, 0.001);
		assertEquals(trans1.get(), 0.4, 0.001);
		assertEquals(trans2.get(), 0.99, 0.001);
		
		trans1.set(0.2);
		trans2.set(0.7);
		assertEquals(obj.getShapeTransparency(), 0.7, 0.001);
		assertEquals(trans1.get(), 0.2, 0.001);
		assertEquals(trans2.get(), 0.7, 0.001);
		
		obj.setShapeTransparency(0.11);
		assertEquals(trans1.get(), 0.2, 0.001);
		assertEquals(trans2.get(), 0.11, 0.001);
		
		obj.setPosition(34.5, 67.8);
		assertEquals(pos.getX(), 34.5, 0.001);
		assertEquals(pos.getY(), 67.8, 0.001);
		PointValue pos2 = new PointValue();
		assertTrue(vm.replaceValue("/SimObjects/MyObject/LocalPosition", pos2));
		obj.setPosition(1.2, 3.4);
		assertEquals(pos.getX(), 34.5, 0.001);
		assertEquals(pos.getY(), 67.8, 0.001);
		assertEquals(pos2.getX(), 1.2, 0.001);
		assertEquals(pos2.getY(), 3.4, 0.001);
		
		obj.reset();
		assertEquals(obj.getPosition(), new Point2D.Double(1.0, 2.0));
		assertEquals(initPos.get(), new Point2D.Double(1.0, 2.0));
		PointValue initPos2 = new PointValue(9.0, 8.0);
		assertTrue(vm.replaceValue("/SimObjects/MyObject/InitPosition", initPos2));
		obj.reset();
		assertEquals(initPos.get(), new Point2D.Double(1.0, 2.0));
		assertEquals(obj.getPosition(), new Point2D.Double(9.0, 8.0));
		assertEquals(initPos2.get(), new Point2D.Double(9.0, 8.0));
		
		obj.setAngle(30.0);
		assertEquals(angle.get(), 30.0, 0.001);
		DoubleValue angle2 = new DoubleValue(15.0);
		assertTrue(vm.replaceValue("/SimObjects/MyObject/LocalAngle", angle2));
		assertEquals(obj.getAngle(), 15.0, 0.001);
		assertEquals(angle.get(), 30.0, 0.001);
		assertEquals(angle2.get(), 15.0, 0.001);
		
		obj.reset();
		assertEquals(initAngle.get(), 3.0, 0.001);
		assertEquals(obj.getAngle(), 3.0, 0.001);
		DoubleValue initAngle2 = new DoubleValue(25.0);
		assertTrue(vm.replaceValue("/SimObjects/MyObject/InitAngle", initAngle2));
		obj.reset();
		assertEquals(obj.getAngle(), 25.0, 0.001);
		assertEquals(initAngle.get(), 3.0, 0.001);
		assertEquals(initAngle2.get(), 25.0, 0.001);
		
		obj.setActive(false);
		assertEquals(active.get(), false);
		BooleanValue active2 = new BooleanValue(true);
		assertTrue(vm.replaceValue("/SimObjects/MyObject/Active", active2));
		assertTrue(obj.isActive());
		assertTrue(active2.get());
		assertFalse(active.get());
		
		obj.setVisible(false);
		assertEquals(visible.get(), false);
		BooleanValue visible2 = new BooleanValue(true);
		assertTrue(vm.replaceValue("/SimObjects/MyObject/Visible", visible2));
		assertTrue(obj.isVisible());
		assertTrue(visible2.get());
		assertFalse(visible.get());
		
		SimObject obj2 = new SimObject();
		SimObject obj3 = new SimObject();
		obj.setReferenceObject(obj2);
		assertSame(reference.get(), obj2);
		SimObjectValue reference2 = new SimObjectValue(obj3);
		assertTrue(vm.replaceValue("/SimObjects/MyObject/ReferenceObject", reference2));
		assertSame(obj.getReferenceObject(), obj3);
		assertSame(reference.get(), obj2);
		assertSame(reference2.get(), obj3);
		
	}
	
	
	public void testGetGlobalShape() {
		Polygon p = new Polygon();
		p.addPoint(5, 2);
		p.addPoint(1, -1);
		p.addPoint(-1, 3);
		
		assertEquals(p.npoints, 3);
		
		SimObject obj = new SimObject(0.0, 0.0, 0.0, p);
		
		Polygon p2 = obj.getGlobalShape();
		assertEquals(p2.npoints, 3);
		assertEquals(p2.xpoints[0], 5);
		assertEquals(p2.ypoints[0], 2);
		assertEquals(p2.xpoints[1], 1);
		assertEquals(p2.ypoints[1], -1);
		assertEquals(p2.xpoints[2], -1);
		assertEquals(p2.ypoints[2], 3);
		
		obj.setPosition(2.0, 1.0);
		
		//without update nothing changes
		p2 = obj.getGlobalShape();
		assertEquals(p2.npoints, 3);
		assertEquals(p2.xpoints[0], 5);
		assertEquals(p2.ypoints[0], 2);
		assertEquals(p2.xpoints[1], 1);
		assertEquals(p2.ypoints[1], -1);
		assertEquals(p2.xpoints[2], -1);
		assertEquals(p2.ypoints[2], 3);
		
		obj.update();
		
		p2 = obj.getGlobalShape();
		assertEquals(p2.npoints, 3);
		assertEquals(p2.xpoints[0], 5 + 2);
		assertEquals(p2.ypoints[0], 2 + 1);
		assertEquals(p2.xpoints[1], 1 + 2);
		assertEquals(p2.ypoints[1], -1 + 1);
		assertEquals(p2.xpoints[2], -1 + 2);
		assertEquals(p2.ypoints[2], 3 + 1);
		
		obj.setAngle(90.0);
		obj.update();
		
		p2 = obj.getGlobalShape();
		assertEquals(p2.npoints, 3);
		assertEquals(p2.xpoints[0], 0);
		assertEquals(p2.ypoints[0], 6);
		assertEquals(p2.xpoints[1], 3);
		assertEquals(p2.ypoints[1], 2);
		assertEquals(p2.xpoints[2], -1);
		assertEquals(p2.ypoints[2], 0);
		
		SimObject obj2 = new SimObject("", -2.0, -2.0, 180.0, new Polygon());
		obj.setReferenceObject(obj2);
		
		obj.update();
		
		p2 = obj.getGlobalShape();
		assertEquals(p2.npoints, 3);
		assertEquals(p2.xpoints[0], -2);
		assertEquals(p2.ypoints[0], -8);
		assertEquals(p2.xpoints[1], -5);
		assertEquals(p2.ypoints[1], -4);
		assertEquals(p2.xpoints[2], -1);
		assertEquals(p2.ypoints[2], -2);
		
	}
	
	public void testGetGlobalShapeAsLineSegments() {
		
		Polygon p = new Polygon();
		p.addPoint(5, 2);
		p.addPoint(1, -1);
		p.addPoint(-1, 3);
		
		assertEquals(p.npoints, 3);
		
		SimObject obj = new SimObject(0.0, 0.0, 0.0, p);
		Vector lines = obj.getShapeAsGlobalLineSegments();
		assertEquals(lines.size(), 3);
		
		Line2D.Double l1 = new Line2D.Double();
		Line2D.Double l2 = new Line2D.Double(5.0, 2.0, 1.0, -1.0);
		l1 = (Line2D.Double) lines.elementAt(0);
		assertTrue(l1.x1 == l2.x1 && l1.y1 == l2.y1 && l1.x2 == l2.x2 && l1.y2 == l2.y2);
		l1 = (Line2D.Double) lines.elementAt(1);
		l2 = new Line2D.Double(1.0, -1.0, -1.0, 3.0);
		assertTrue(l1.x1 == l2.x1 && l1.y1 == l2.y1 && l1.x2 == l2.x2 && l1.y2 == l2.y2);
		l1 = (Line2D.Double) lines.elementAt(2);
		l2 = new Line2D.Double(-1.0, 3.0, 5.0, 2.0);
		assertTrue(l1.x1 == l2.x1 && l1.y1 == l2.y1 && l1.x2 == l2.x2 && l1.y2 == l2.y2);
		
		obj.setPosition(2.0, 1.0);
		
		//without update there is no change.
		lines = obj.getShapeAsGlobalLineSegments();
		assertEquals(lines.size(), 3);
		
		l1 = (Line2D.Double) lines.elementAt(0);
		l2 = new Line2D.Double(5.0, 2.0, 1.0, -1.0);
		assertTrue(l1.x1 == l2.x1 && l1.y1 == l2.y1 && l1.x2 == l2.x2 && l1.y2 == l2.y2);
		l1 = (Line2D.Double) lines.elementAt(1);
		l2 = new Line2D.Double(1.0, -1.0, -1.0, 3.0);
		assertTrue(l1.x1 == l2.x1 && l1.y1 == l2.y1 && l1.x2 == l2.x2 && l1.y2 == l2.y2);
		l1 = (Line2D.Double) lines.elementAt(2);
		l2 = new Line2D.Double(-1.0, 3.0, 5.0, 2.0);
		assertTrue(l1.x1 == l2.x1 && l1.y1 == l2.y1 && l1.x2 == l2.x2 && l1.y2 == l2.y2);
		
		//with update
		obj.update();
		lines = obj.getShapeAsGlobalLineSegments();
		assertEquals(lines.size(), 3);
		l1 = (Line2D.Double) lines.elementAt(0);
		l2 = new Line2D.Double(5.0 + 2.0, 2.0 + 1.0, 1.0 + 2.0, -1.0 + 1.0);
		assertTrue(l1.x1 == l2.x1 && l1.y1 == l2.y1 && l1.x2 == l2.x2 && l1.y2 == l2.y2);
		l1 = (Line2D.Double) lines.elementAt(1);
		l2 = new Line2D.Double(1.0 + 2.0, -1.0 + 1.0, -1.0 + 2.0, 3.0 + 1.0);
		assertTrue(l1.x1 == l2.x1 && l1.y1 == l2.y1 && l1.x2 == l2.x2 && l1.y2 == l2.y2);
		l1 = (Line2D.Double) lines.elementAt(2);
		l2 = new Line2D.Double(-1.0 + 2.0, 3.0 + 1.0, 5.0 + 2.0, 2.0 + 1.0);
		assertTrue(l1.x1 == l2.x1 && l1.y1 == l2.y1 && l1.x2 == l2.x2 && l1.y2 == l2.y2);
		
		obj.setAngle(90.0);
		obj.update();
		lines = obj.getShapeAsGlobalLineSegments();
		assertEquals(lines.size(), 3);
		l1 = (Line2D.Double) lines.elementAt(0);
		l2 = new Line2D.Double(0.0, 6.0, 3.0, 2.0);
		assertTrue(l1.x1 == l2.x1 && l1.y1 == l2.y1 && l1.x2 == l2.x2 && l1.y2 == l2.y2);
		l1 = (Line2D.Double) lines.elementAt(1);
		l2 = new Line2D.Double(3.0, 2.0, -1.0, 0.0);
		assertTrue(l1.x1 == l2.x1 && l1.y1 == l2.y1 && l1.x2 == l2.x2 && l1.y2 == l2.y2);
		l1 = (Line2D.Double) lines.elementAt(2);
		l2 = new Line2D.Double(-1.0, 0.0, 0.0, 6.0);
		assertTrue(l1.x1 == l2.x1 && l1.y1 == l2.y1 && l1.x2 == l2.x2 && l1.y2 == l2.y2);
		
		SimObject obj2 = new SimObject("", -2.0, -2.0, 180.0, new Polygon());
		obj.setReferenceObject(obj2);
		
		obj.update();
		lines = obj.getShapeAsGlobalLineSegments();
		assertEquals(lines.size(), 3);
		l1 = (Line2D.Double) lines.elementAt(0);
		l2 = new Line2D.Double(-2.0, -8.0, -5.0, -4.0);
		assertTrue(l1.x1 == l2.x1 && l1.y1 == l2.y1 && l1.x2 == l2.x2 && l1.y2 == l2.y2);
		l1 = (Line2D.Double) lines.elementAt(1);
		l2 = new Line2D.Double(-5.0, -4.0, -1.0, -2.0);
		assertTrue(l1.x1 == l2.x1 && l1.y1 == l2.y1 && l1.x2 == l2.x2 && l1.y2 == l2.y2);
		l1 = (Line2D.Double) lines.elementAt(2);
		l2 = new Line2D.Double(-1.0, -2.0, -2.0, -8.0);
		assertTrue(l1.x1 == l2.x1 && l1.y1 == l2.y1 && l1.x2 == l2.x2 && l1.y2 == l2.y2);
		

	}
	

}
