/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.test;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.neuralnet.Neuron;
import de.fhg.iais.outdoor.simba.neuralnet.NeuronWrapper;
import de.fhg.iais.outdoor.simba.neuralnet.Synapse;
import de.fhg.iais.outdoor.simba.value.DoubleValue;
import junit.framework.TestCase;

public class TestSynapse extends TestCase {

	protected void setUp() throws Exception {
		super.tearDown();
		Simulator.getInstance().clearInstance();
	}
	
	protected void tearDown() throws Exception {
		super.tearDown();
		Simulator.getInstance().clearInstance();
	}
	
	public void testConstructor() {
		Synapse s = new Synapse(null);
		assertNull(s.getSourceNeuron());
		assertNull(s.getTargetNeuron());
		assertEquals(s.getStrength(), 0.0, 0.0001);
		assertNotNull(s.getEnabledValue());
		assertTrue(s.getEnabledValue().get());
		try {
			assertEquals(s.getActivation(), 0.0, 0.0001);
			fail("Was allowed to throw a NullPointerException");
		}
		catch(NullPointerException e) {
		}
		
		s.setStrength(-25.6);
		assertEquals(s.getStrength(), -25.6, 0.0001);
		
		Neuron n = new Neuron("N1");
		NeuronWrapper activation = new NeuronWrapper("", 0.0, 0.0, 0.0);
		n.setActivationValue(activation);
		activation.setNormalized(0.5);
		assertEquals(n.getActivation(), 0.5, 0.0001);
		
		s = new Synapse(n);
		s.setStrength(-25.6);
		assertSame(s.getSourceNeuron(), n);
		n.storeLastActivity();
		assertEquals(s.getActivation(), -25.6 * 0.5, 0.0001);
		
		s.setTargetNeuron(n);
		assertSame(s.getTargetNeuron(), n);
		
		s.setSourceNeuron(null);
		assertNull(s.getSourceNeuron());
		
	}
	
	public void testClone() {
		Neuron n = new Neuron("Neuron1");
		Neuron target = new Neuron("Neuron1");

		Synapse s = new Synapse(n);
		s.setStrength(100.0);
		s.setTargetNeuron(target);
		DoubleValue strengthValue = s.getStrengthValue();
		assertSame(s.getSourceNeuron(), n);
		assertEquals(s.getStrength(), 100.0, 0.001);
		assertSame(s.getTargetNeuron(), target);
		
		Synapse s2 = null;
		try {
			s2 = (Synapse) s.clone();
		}
		catch(CloneNotSupportedException e) {
			fail("CloneNotSupportedException");
		}
		assertNotNull(s2);
		assertNotSame(s, s2);
		assertNotSame(strengthValue, s2.getStrengthValue());
		assertSame(s.getSourceNeuron(), s2.getSourceNeuron());
		assertSame(s.getTargetNeuron(), s2.getTargetNeuron());
		s.setStrength(40.0);
		assertEquals(s.getStrength(), 40.0, 0.001);
		assertEquals(s2.getStrength(), 100.0, 0.0001);
		
		assertNotNull(s2.getEnabledValue());
		assertNotSame(s.getEnabledValue(), s2.getEnabledValue());
		
		
	}
}
