/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.test;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetSocketAddress;

public class TestUDPNeuroConnectionClient {

	public static final int FAILURE = -1;
	public static final int COMMAND_COMPLETE = 1;
	public static final int COMMAND_PREPARE_GENERATION = 2;
	public static final int COMMAND_PREPARE_INDIVIDUAL = 3;
	public static final int COMMAND_PREPARE_TRY = 4;
	public static final int COMMAND_EXECUTE_STEP = 5;
	public static final int COMMAND_TERMINATE = 6;
	public static final int COMMAND_RESET_SYSTEM = 7;
	public static final int COMMAND_GET_SYSTEM_INFO = 8;
	public static final int COMMAND_INFO_ACK = 9;
	
	DatagramSocket mSocket;
	
	private int mCurrentTry;
	private int mCurrentGeneration;
	private int mCurrentIndividual;
	private int mCurrentStep;
	private int mNumberOfTries;
	private int mNumberOfCycles;
	private int mNumberOfInputNeurons;
	private int mNumberOfOutputNeurons;
	private int mNumberOfInfoNeurons;
	private int mPort;
	private ByteArrayInputStream mInputStream;
	private ByteArrayOutputStream mOutputStream;
	private DataInputStream mInput;
	private DataOutputStream mOutput;
	byte[] mInputBytes;
	DatagramPacket mInputDatagram;
	InetSocketAddress mAdress;
	private boolean mBumped;
	
	public TestUDPNeuroConnectionClient() {
		mCurrentTry = 0;
		mCurrentGeneration = 0;
		mCurrentIndividual = 0;
		mCurrentStep = 0;
		mNumberOfTries = 8;
		mNumberOfCycles = 500;
		mInputBytes = new byte[1000];
		mInputDatagram = new DatagramPacket(mInputBytes, mInputBytes.length);
	}
	
	public void start(int port) {
		mPort = port;
		mAdress = new InetSocketAddress("localhost", mPort);
		
		connect();
		
		for(int g = 0; g < 10; ++g) {
			if(g != 0) {
				setNewStartPosition();
			}
			for(int i = 0; i < 10; ++i) {
				if(i != 0) {
					reset();
				}
				for(int t = 0; t < mNumberOfTries; ++t) {
					if(t != 0) {
						nextTry();
					}
					for(int s = 0; s < mNumberOfCycles; ++s) {
						send();
						if(mBumped) {
							break;
						}
					}
				}
			}
		}

		disconnect();
		
	}
	

	public String getConfigFileName() {
		return "xml/simulator/yars.xml";
	}

	public String getSimName() {
		return "Simba:Maze";
	}

	public void send() {
		mBumped = false;
		
//		System.out.println("Send");
		
		if(mCurrentGeneration == 0) {
			setNewStartPosition();
		}
		if(mCurrentIndividual == 0) {
			reset();
		}
		if(mCurrentTry == 0) {
			nextTry();
		}
			
		if(mInput != null && mOutput != null) {
			try {
				mOutputStream.reset();
				mOutput.writeInt(COMMAND_EXECUTE_STEP);
				mOutput.writeInt(mCurrentStep);

				for(int i = 0; i < mNumberOfOutputNeurons; ++i) {
					mOutput.writeDouble(0.5 - (((double) i) * 0.1));
				}
				mOutput.flush();
				DatagramPacket outputDatagram = new DatagramPacket(mOutputStream.toByteArray(), mOutputStream.toByteArray().length, mAdress);
				mSocket.send(outputDatagram);
				
//				System.out.println("written");
				
				mInputDatagram = new DatagramPacket(mInputBytes, mInputBytes.length);
				mSocket.receive(mInputDatagram);
				mInput.reset();
				
				//check if bumped.
				mBumped = mInput.readBoolean();
				
				for(int i = 0; i < mNumberOfInputNeurons; ++i) {
					mInput.readDouble();
				}
				for(int i = 0; i < mNumberOfInfoNeurons; ++i) { //TODO new
					mInput.readDouble();
				}
				
//				System.out.println("Read");
				
				if(!(mInput.readInt() == COMMAND_COMPLETE)) {
					System.out.println("Communication problem: Did not receive CommandComplete signal for executeStep command.");
				}
//				System.out.println("Complete");
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		mCurrentStep++;
	}

	public void connect() {
		
//		System.out.println("Conecting");
		try {
			mSocket = new DatagramSocket();
			mSocket.connect(new InetSocketAddress("127.0.0.1", mPort));
			mInputStream = new ByteArrayInputStream(mInputBytes);
			mOutputStream = new ByteArrayOutputStream();
			mInput = new DataInputStream(mInputStream);
			mOutput = new DataOutputStream(mOutputStream);
			
			mOutputStream.reset();
			mOutput.writeInt(COMMAND_GET_SYSTEM_INFO);
			mOutput.flush();
			DatagramPacket outputDatagram = new DatagramPacket(mOutputStream.toByteArray(), mOutputStream.toByteArray().length, mAdress);
			mSocket.send(outputDatagram);
			
//			System.out.println("Sent info");
			
			mInputDatagram = new DatagramPacket(mInputBytes, mInputBytes.length);
			mSocket.receive(mInputDatagram);
			mInput.reset();
			mNumberOfInputNeurons = mInput.readInt();
			mNumberOfOutputNeurons = mInput.readInt();
			mNumberOfInfoNeurons = mInput.readInt(); //TODO new
			if(!(mInput.readInt() == COMMAND_COMPLETE)) {
				System.out.println("Communication problem: Did not receive CommandComplete signal for info command.");
			}
			
			
			for(int i = 0; i < mNumberOfInputNeurons; ++i) {
				mOutputStream.reset();
				mOutput.writeInt(COMMAND_INFO_ACK);
				mOutput.flush();
				outputDatagram = new DatagramPacket(mOutputStream.toByteArray(), mOutputStream.toByteArray().length, mAdress);
				mSocket.send(outputDatagram);
				
				mInputStream.reset();
				mSocket.receive(mInputDatagram);
				mInput.reset();
				String name =  mInput.readUTF();
				System.out.println("Received Input: " + name);
			}
			
			for(int i = 0; i < mNumberOfOutputNeurons; ++i) {
				mOutputStream.reset();
				mOutput.writeInt(COMMAND_INFO_ACK);
				mOutput.flush();
				outputDatagram = new DatagramPacket(mOutputStream.toByteArray(), mOutputStream.toByteArray().length, mAdress);
				mSocket.send(outputDatagram);
				
				mInputStream.reset();
				mSocket.receive(mInputDatagram);
				mInput.reset();
				String name =  mInput.readUTF();
				System.out.println("Received Output: " + name);
			}
			
			for(int i = 0; i < mNumberOfInfoNeurons; ++i) { //TODO new
				mOutputStream.reset();
				mOutput.writeInt(COMMAND_INFO_ACK);
				mOutput.flush();
				outputDatagram = new DatagramPacket(mOutputStream.toByteArray(), mOutputStream.toByteArray().length, mAdress);
				mSocket.send(outputDatagram);
				
				mInputStream.reset();
				mSocket.receive(mInputDatagram);
				mInput.reset();
				String name =  mInput.readUTF();
				System.out.println("Received Info: " + name);
			}
			
			mInputStream.reset();
			mSocket.receive(mInputDatagram);
			mInput.reset();
			if(!(mInput.readInt() == COMMAND_COMPLETE)) {
				System.out.println("Communication problem: Did not receive CommandComplete signal for info command.");
			}
//			System.out.println("Have: Input: " + mNumberOfInputNeurons + "  Output: " + mNumberOfOutputNeurons);
			
		} catch (IOException e) {
			System.out.println("Could not connect.");
			System.exit(0);
		}
	}

	public void disconnect() {
		if(mInput != null && mOutput != null) {
			try {
				mOutputStream.reset();
				mOutput.writeInt(COMMAND_TERMINATE);
				mOutput.flush();
				DatagramPacket outputDatagram = new DatagramPacket(mOutputStream.toByteArray(), mOutputStream.toByteArray().length, mAdress);
				mSocket.send(outputDatagram);
				
				mInputDatagram = new DatagramPacket(mInputBytes, mInputBytes.length);
				mSocket.receive(mInputDatagram);
				mInput.reset();
				if(!(mInput.readInt() == COMMAND_COMPLETE)) {
					System.out.println("Communication problem: Did not receive CommandComplete signal for terminate command.");
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		mSocket.close();
		mInput = null;
		mOutput = null;
	}

	public void nextTry() {
//		System.out.println("next try");
		if(mInput != null && mOutput != null) {
			try {
				mOutputStream.reset();
				mOutput.writeInt(COMMAND_PREPARE_TRY);
				mOutput.writeInt(mCurrentTry);
				mOutput.flush();
				DatagramPacket outputDatagram = new DatagramPacket(mOutputStream.toByteArray(), mOutputStream.toByteArray().length, mAdress);
				mSocket.send(outputDatagram);
				
//				System.out.println("written try");
				
				mInputDatagram = new DatagramPacket(mInputBytes, mInputBytes.length);
				mSocket.receive(mInputDatagram);
				mInput.reset();
				if(!(mInput.readInt() == COMMAND_COMPLETE)) {
					System.out.println("Communication problem: Did not receive CommandComplete signal for nextTry command.");
				}
//				System.out.println("Complete");
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		mCurrentStep = 0;
		mCurrentTry++;
	}

	public void reset() {
//		System.out.println("Reset");
		if(mInput != null && mOutput != null) {
			try {
				mOutputStream.reset();
				mOutput.writeInt(COMMAND_PREPARE_INDIVIDUAL);
				mOutput.writeInt(mCurrentIndividual);
				mOutput.flush();
				DatagramPacket outputDatagram = new DatagramPacket(mOutputStream.toByteArray(), mOutputStream.toByteArray().length, mAdress);
				mSocket.send(outputDatagram);
				
				mInputDatagram = new DatagramPacket(mInputBytes, mInputBytes.length);
				mSocket.receive(mInputDatagram);
				mInput.reset();
				if(!(mInput.readInt() == COMMAND_COMPLETE)) {
					System.out.println("Communication problem: Did not receive CommandComplete signal for nextIndividual command.");
				}
//				System.out.println("Complete");
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		mCurrentStep = 0;
		mCurrentTry = 0;
		mCurrentIndividual++;
	}

	public void setNewStartPosition() {
		
//		System.out.println("Set start pos");
		if(mInput != null && mOutput != null) {
			try {
				mOutputStream.reset();
				mOutput.writeInt(COMMAND_PREPARE_GENERATION);
				mOutput.writeInt(mCurrentGeneration);
				mOutput.writeInt(mNumberOfTries);
				mOutput.writeInt(mNumberOfCycles);
				mOutput.flush();
				DatagramPacket outputDatagram = new DatagramPacket(mOutputStream.toByteArray(), mOutputStream.toByteArray().length, mAdress);
				mSocket.send(outputDatagram);
				
				mInputDatagram = new DatagramPacket(mInputBytes, mInputBytes.length);
				mSocket.receive(mInputDatagram);
				mInput.reset();
				if(!(mInput.readInt() == COMMAND_COMPLETE)) {
					System.out.println("Communication problem: Did not receive CommandComplete signal for nextGeneration command.");
				}
//				System.out.println("Command complete");
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		mCurrentStep = 0;
		mCurrentTry = 0;
		mCurrentIndividual = 0;
		mCurrentGeneration++;
	}

	public void stop() {
		//nothing to do.
	}

	
	
	public static void main(String[] arg) {
		TestUDPNeuroConnectionClient client = new TestUDPNeuroConnectionClient();
		client.start(50100);
	}
	
}
