/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.test;

import java.awt.geom.Point2D;

import de.fhg.iais.outdoor.simba.core.SimObject;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.DoubleValue;
import de.fhg.iais.outdoor.simba.value.IntValue;
import de.fhg.iais.outdoor.simba.value.ObjectValue;
import de.fhg.iais.outdoor.simba.value.PointValue;
import de.fhg.iais.outdoor.simba.value.SimObjectValue;
import de.fhg.iais.outdoor.simba.value.StringValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueChangedListener;
import de.fhg.iais.outdoor.simba.value.ValueManager;
import junit.framework.TestCase;

public class TestValue extends TestCase {

	protected void setUp() throws Exception {
		super.setUp();
		Simulator.getInstance().clearInstance();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
		Simulator.getInstance().clearInstance();
	}

	public void testDoubleValue() {
		TestValueChangedListener listener = new TestValueChangedListener();
		
		DoubleValue d = new DoubleValue();
		d.addValueChangedListener(listener);
		
		assertNull(listener.mLastReportedValue);
		assertEquals(d.get(), 0.0, 0.001);
		System.out.println(d.getValueAsString());
		assertEquals(d.getValueAsString(), "0.0");
		
		d.set(55.1);
		assertSame(listener.mLastReportedValue, d);
		assertEquals(d.get(), 55.1, 0.001);
		assertEquals(d.getValueAsString(), "55.1");
		
		listener.mLastReportedValue = null;
		d = new DoubleValue(19.28);
		assertNull(listener.mLastReportedValue);
		assertEquals(d.get(), 19.28, 0.001);
		
		ValueManager vm = new ValueManager();
		vm.addValue("test", d);
		assertSame(vm.getDoubleValue("test"), d);
		
		assertTrue(d.setValueFromString("1.2345"));
		assertEquals(d.get(), 1.2345, 0.001);
		
		assertFalse(d.setValueFromString("EinsZweiDrei"));
		assertEquals(d.get(), 1.2345, 0.001);
		

	}
	
	public void testIntValue() {
		TestValueChangedListener listener = new TestValueChangedListener();
		
		IntValue i = new IntValue();
		i.addValueChangedListener(listener);
		
		assertNull(listener.mLastReportedValue);
		assertEquals(i.get(), 0);
		
		i.set(46);
		assertSame(listener.mLastReportedValue, i);
		assertEquals(i.get(), 46);
		assertEquals(i.getValueAsString(), "46");
		
		listener.mLastReportedValue = null;
		i = new IntValue(16);
		assertNull(listener.mLastReportedValue);
		assertEquals(i.get(), 16);
		assertEquals(i.getValueAsString(), "16");
		
		ValueManager vm = new ValueManager();
		vm.addValue("test", i);
		assertSame(vm.getIntValue("test"), i);
		
		assertTrue(i.setValueFromString("4567"));
		assertEquals(i.get(), 4567);
		
		assertFalse(i.setValueFromString("EinsZwei"));
		assertEquals(i.get(), 4567);
	}
	
	public void testBooleanValue() {
		TestValueChangedListener listener = new TestValueChangedListener();
		
		BooleanValue v = new BooleanValue();
		v.addValueChangedListener(listener);
		
		assertNull(listener.mLastReportedValue);
		assertEquals(v.get(), true);
		assertEquals(v.getValueAsString(), "T");
		
		v.set(false);
		assertSame(listener.mLastReportedValue, v);
		assertEquals(v.get(), false);
		assertEquals(v.getValueAsString(), "F");
		
		listener.mLastReportedValue = null;
		v = new BooleanValue(false);
		assertNull(listener.mLastReportedValue);
		assertEquals(v.get(), false);
		
		ValueManager vm = new ValueManager();
		vm.addValue("test", v);
		assertSame(vm.getBooleanValue("test"), v);
		
		assertTrue(v.setValueFromString("true"));
		assertEquals(v.get(), true);
		assertTrue(v.setValueFromString("FALSE"));
		assertEquals(v.get(), false);
		assertTrue(v.setValueFromString("tRue"));
		assertEquals(v.get(), true);
		assertTrue(v.setValueFromString("F"));
		assertEquals(v.get(), false);
		assertTrue(v.setValueFromString("1"));
		assertEquals(v.get(), true);
		assertTrue(v.setValueFromString("0"));
		assertEquals(v.get(), false);
	}
	
	public void testObjectValue() {
		TestValueChangedListener listener = new TestValueChangedListener();
		
		ObjectValue v = new ObjectValue();
		v.addValueChangedListener(listener);
		
		assertNull(listener.mLastReportedValue);
		assertNull(v.get());
		
		String obj = "";
		
		v.set(obj);
		assertSame(listener.mLastReportedValue, v);
		assertSame(v.get(), obj);
		assertEquals(v.getValueAsString(), obj.toString());
		
		listener.mLastReportedValue = null;
		v = new ObjectValue(obj);
		assertNull(listener.mLastReportedValue);
		assertSame(v.get(), obj);
		
		ValueManager vm = new ValueManager();
		vm.addValue("test", v);
		assertSame(vm.getObjectValue("test"), v);
		
	}
	
	public void testPointValue() {
		TestValueChangedListener listener = new TestValueChangedListener();
		
		Point2D.Double point = new Point2D.Double(12.3, 45.6);
		
		PointValue v = new PointValue();
		v.addValueChangedListener(listener);
		
		assertNull(listener.mLastReportedValue);
		assertEquals(v.get().x, 0.0, 0.001);
		assertEquals(v.get().y, 0.0, 0.001);
		assertEquals(v.getX(), 0.0, 0.001);
		assertEquals(v.getY(), 0.0, 0.001);
		assertEquals(v.getValueAsString(), "(0.0, 0.0)");
		
		v.set(point);
		assertSame(listener.mLastReportedValue, v);
		assertNotSame(v.get(), point);
		assertEquals(v.get().x, 12.3, 0.001);
		assertEquals(v.get().y, 45.6, 0.001);
		assertEquals(v.getX(), 12.3, 0.001);
		assertEquals(v.getY(), 45.6, 0.001);
		assertEquals(v.getValueAsString(), "(12.3, 45.6)");
		listener.mLastReportedValue = null;
		
		v.set(10.5, 20.6);
		assertSame(listener.mLastReportedValue, v);
		assertEquals(v.get().x, 10.5, 0.001);
		assertEquals(v.get().y, 20.6, 0.001);
		assertEquals(v.getX(), 10.5, 0.001);
		assertEquals(v.getY(), 20.6, 0.001);
		
		listener.mLastReportedValue = null;
		v = new PointValue(45.7, 67.8);
		assertNull(listener.mLastReportedValue);
		assertEquals(v.getX(), 45.7, 0.001);
		assertEquals(v.getY(), 67.8, 0.001);
		
		listener.mLastReportedValue = null;
		v = new PointValue(point);
		assertNull(listener.mLastReportedValue);
		assertEquals(v.getX(), 12.3, 0.001);
		assertEquals(v.getY(), 45.6, 0.001);
		
		ValueManager vm = new ValueManager();
		vm.addValue("test", v);
		assertSame(vm.getPointValue("test"), v);
		
		assertTrue(v.setValueFromString("(100.0,  30.3)"));
		assertEquals(v.getX(), 100.0, 0.001);
		assertEquals(v.getY(), 30.3, 0.001);
		assertTrue(v.setValueFromString("50.1,60.2"));
		assertEquals(v.getX(), 50.1, 0.001);
		assertEquals(v.getY(), 60.2, 0.001);
		assertFalse(v.setValueFromString("Point"));
		assertEquals(v.getX(), 50.1, 0.001);
		assertEquals(v.getY(), 60.2, 0.001);
		
	}
	
	public void testSimObjectValue() {
		TestValueChangedListener listener = new TestValueChangedListener();
		
		SimObjectValue v = new SimObjectValue();
		v.addValueChangedListener(listener);
		
		assertNull(listener.mLastReportedValue);
		assertNull(v.get());
		
		SimObject obj = new SimObject("ObjectName");
		
		v.set(obj);
		assertSame(listener.mLastReportedValue, v);
		assertSame(v.get(), obj);
		assertEquals(v.getValueAsString(), "ObjectName");
		
		listener.mLastReportedValue = null;
		v = new SimObjectValue(obj);
		assertNull(listener.mLastReportedValue);
		assertSame(v.get(), obj);
		
		ValueManager vm = new ValueManager();
		vm.addValue("test", v);
		assertSame(vm.getSimObjectValue("test"), v);
		
		//TODO implement and test setFromString.
	}
	
	public void testStringValue() {
		TestValueChangedListener listener = new TestValueChangedListener();
		
		StringValue v = new StringValue();
		v.addValueChangedListener(listener);
		
		assertNull(listener.mLastReportedValue);
		assertEquals(v.get(), "");
		
		v.set("Hello");
		assertSame(listener.mLastReportedValue, v);
		assertEquals(v.get(), "Hello");
		assertEquals(v.getValueAsString(), "Hello");
		
		listener.mLastReportedValue = null;
		v = new StringValue("World");
		assertNull(listener.mLastReportedValue);
		assertSame(v.get(), "World");
		
		ValueManager vm = new ValueManager();
		vm.addValue("test", v);
		assertSame(vm.getStringValue("test"), v);
		
		assertTrue(v.setValueFromString("Galaxy"));
		assertEquals(v.get(), "Galaxy");
	}
	
	
	public class TestValueChangedListener implements ValueChangedListener {
		public Value mLastReportedValue;
		public Value mLastOldValue;
		
		public void valueChanged(Value value) {
			mLastReportedValue = value;
		}
		
		/**
		 * Called if the one of the registered Value objects were replaced in the ValueManager.
		 */
		 public void valueReplaced(Value oldValue, Value newValue) {
			mLastOldValue = oldValue;
			mLastReportedValue = newValue;
		 }
	}

}
