/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.value;

public class NormalizedDoubleWrapper extends Value implements ValueChangedListener {

protected NormalizedDoubleValue mValue;
	
	public NormalizedDoubleWrapper(NormalizedDoubleValue value) {
		super();
		if(value == null) {
			throw new IllegalArgumentException("The parameter value may not be null");
		}
		mValue = value;
		mValue.addValueChangedListener(this);
	}
	
	/**
	 * Returns the double value represented by this DoubelValue object.
	 * 
	 * @return the value of this DoubleValue.
	 */
	public synchronized double get() {
		return mValue.getNormalized();
	}
	
	/**
	 * Changes the encapsulated double value to the value given as parameter.
	 * 
	 * @param value the new value of this DoubleValue.
	 */
	public synchronized void set(double value) {
		mValue.setNormalized(value);
		notifyValueChanged();
	}
	
	/**
	 * Returns the content of the Value as String.
	 * This method should be implemented in a suitable way in each subclass.
	 * @return the string representation of the Values content.
	 */
	public String getValueAsString() {
		return Double.toString(mValue.getNormalized());
	}
	
	
	/**
	 * Sets the content of the Value from a String.
	 * This method should be implemented in a suitable way in each subclass.
	 * @return true if the parameter could successfully be parsed to set the value.
	 */
	public boolean setValueFromString(String value) {
		return mValue.setNormalizedValueFromString(value);
	}

	public void valueChanged(Value value) {
		if(value != null && value == mValue) {
			notifyValueChanged();
		}
	}

	public void valueReplaced(Value oldValue, Value newValue) {
		mValue = (NormalizedDoubleValue) ValueManager.synchronizeValue(mValue, oldValue, newValue, this);
	}
	
	
	
}
