/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.value;

import java.awt.geom.Point2D;

public class PointValue extends Value {
	
	private Point2D.Double mValue;
	
	public PointValue() {
		mValue = new Point2D.Double();
	}
	
	public PointValue(double x, double y) {
		mValue = new Point2D.Double(x, y);
	}
	
	public PointValue(Point2D.Double point) {
		mValue = new Point2D.Double(point.x, point.y);
	}
	
	public Point2D.Double get() {
		//TODO check: for performance reasons the internal variable is returned. Should be a copy.
		return mValue;
	}
	
	public double getX() {
		return mValue.x;
	}
	
	public double getY()  {
		return mValue.y;
	}
	
	public void set(double x, double y) {
		mValue.setLocation(x, y);
		notifyValueChanged();
	}
	
	public void set(Point2D.Double point) {
		mValue.setLocation(point);
		notifyValueChanged();
	}
	
	/**
	 * Returns the content of the Value as String.
	 * This method should be implemented in a suitable way in each subclass.
	 * @return the string representation of the Values content.
	 */
	public String getValueAsString() {
		return "(" + mValue.x + ", " + mValue.y + ")";
	}
	
	/**
	 * Sets the content of the Value from a String.
	 * This method should be implemented in a suitable way in each subclass.
	 * @return true if the parameter could successfully be parsed to set the value.
	 */
	public boolean setValueFromString(String value) {
		if(value.startsWith("(")) {
			value = value.substring(1);
		}
		if(value.endsWith(")")) {
			value = value.substring(0, value.length() - 1);
		}
		String[] coordinates = value.split(",");
		if(coordinates.length == 2) {
			try {
				double x = Double.parseDouble(coordinates[0].trim());
				double y = Double.parseDouble(coordinates[1].trim());
				if(mValue.x != x || mValue.y != y) {
					set(x, y);
				}
				return true;
			} catch (NumberFormatException e) {
				return false;
			}
		}
		return false;
	}
	
	

}
