/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.visualization;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JCheckBox;
import javax.swing.JPanel;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.value.IntValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueChangedListener;
import de.fhg.iais.outdoor.simba.value.ValueManager;

public class EvolutionInformationDisplay implements Visualization, ValueChangedListener,
											OptimizePerformanceListener
{
	
	private JPanel mDisplay;
	private JCheckBox mGenerationLabel;
	private JCheckBox mIndividualLabel;
	private JCheckBox mTryLabel;
	private JCheckBox mStepLabel;
	
	private IntValue mCurrentTry;
	private IntValue mCurrentIndividual;
	private IntValue mCurrentGeneration;
	private IntValue mCurrentStep;
	private IntValue mNumberOfTries;
	private IntValue mNumberOfSteps;
	
	private boolean mShowSteps;
	
	public EvolutionInformationDisplay() {
		mDisplay = new JPanel(new BorderLayout());
		Box box = Box.createVerticalBox();
		mDisplay.add(BorderLayout.CENTER, box);
		mDisplay.setBorder(BorderFactory.createTitledBorder("Evolution Info"));
		mDisplay.setPreferredSize(new Dimension(140, 130));
		
		mGenerationLabel = new JCheckBox("Generation: ");
		mIndividualLabel = new JCheckBox("Individual: ");
		mTryLabel = new JCheckBox("Try:  ");
		mStepLabel = new JCheckBox("Step: ");
		mGenerationLabel.setSelected(true);
		mIndividualLabel.setSelected(true);
		mTryLabel.setSelected(true);
		mStepLabel.setSelected(true);
		
		mGenerationLabel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				handleCheckBoxChanged();
			}
		});
		mIndividualLabel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				handleCheckBoxChanged();
			}
		});
		mTryLabel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				handleCheckBoxChanged();
			}
		});
		mStepLabel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				handleCheckBoxChanged();
			}
		});
		
		box.add(mGenerationLabel);
		box.add(mIndividualLabel);
		box.add(mTryLabel);
		box.add(mStepLabel);
		
		Simulator.getInstance().addOptimizePerformanceListener(this);

		mShowSteps = true;
	}
	
	public void handleCheckBoxChanged() {
		if(mStepLabel.isSelected() && mCurrentStep != null) {
			mCurrentStep.addValueChangedListener(this);
			if(mNumberOfSteps != null) {
				mNumberOfSteps.addValueChangedListener(this);
			}
			mShowSteps = true;
			valueChanged(mCurrentStep);
		}
		if(!mStepLabel.isSelected() && mCurrentStep != null) {
			mCurrentStep.removeValueChangedListener(this);
			if(mNumberOfSteps != null) {
				mNumberOfSteps.removeValueChangedListener(this);
			}
			mShowSteps = false;
			mStepLabel.setText("Step: -");
		}
		if(mIndividualLabel.isSelected() && mCurrentIndividual != null) {
			mCurrentIndividual.addValueChangedListener(this);
			valueChanged(mCurrentIndividual);
		}
		if(!mIndividualLabel.isSelected() && mCurrentIndividual != null) {
			mCurrentIndividual.removeValueChangedListener(this);
			mIndividualLabel.setText("Individuum: -");
		}
		if(mGenerationLabel.isSelected() && mCurrentGeneration != null) {
			mCurrentGeneration.addValueChangedListener(this);
			valueChanged(mCurrentGeneration);
		}
		if(!mGenerationLabel.isSelected() && mCurrentGeneration != null) {
			mCurrentGeneration.removeValueChangedListener(this);
			mGenerationLabel.setText("Generation: -");
		}
		if(mTryLabel.isSelected() && mCurrentTry != null) {
			mCurrentTry.addValueChangedListener(this);
			if(mNumberOfTries != null) {
				mNumberOfTries.addValueChangedListener(this);
			}
			valueChanged(mCurrentTry);
		}
		if(!mTryLabel.isSelected() && mCurrentTry != null) {
			mCurrentTry.removeValueChangedListener(this);
			if(mNumberOfTries != null) {
				mNumberOfTries.removeValueChangedListener(this);
			}
			mTryLabel.setText("Try: -");
		}
		
	}

	public JPanel getPanel() {
		return mDisplay;
	}

	public boolean cleanUp() {
		return true;
	}

	public boolean init() {
		
		ValueManager vm = Simulator.getInstance().getValueManager();
		mCurrentGeneration = vm.getIntValue("/Simulation/CurrentGeneration");
		mCurrentIndividual = vm.getIntValue("/Simulation/CurrentIndividual");
		mCurrentStep = vm.getIntValue("/Simulation/CurrentStep");
		mCurrentTry = vm.getIntValue("/Simulation/CurrentTry");
		mNumberOfTries = vm.getIntValue("/Simulation/NumberOfTries");
		mNumberOfSteps = vm.getIntValue("/Simulation/NumberOfSteps");
		
		handleCheckBoxChanged();
		return true;
	}
	
	

	public void valueChanged(Value value) {
		if(value == null) {
			return;
		}
		else if(value == mCurrentStep || value == mNumberOfSteps) {
			StringBuffer text = new StringBuffer("Step: ");
			if(mCurrentStep != null) {
				text.append((mCurrentStep.get() + 1));
			}
			if(mNumberOfSteps != null) {
				text.append("/" + mNumberOfSteps.get());
			}
			mStepLabel.setText(text.toString());
		}
		else if(value == mCurrentTry || value == mNumberOfTries) {
			StringBuffer text = new StringBuffer("Try:  ");
			if(mCurrentTry != null) {
				text.append((mCurrentTry.get() + 1));
			}
			if(mNumberOfTries != null) {
				text.append("/" + mNumberOfTries.get());
			}
			mTryLabel.setText(text.toString());
		}
		else if(value == mCurrentGeneration) {
			mGenerationLabel.setText("Generation: " + (mCurrentGeneration.get()));
		}
		else if(value == mCurrentIndividual) {
			mIndividualLabel.setText("Individuum: " + (mCurrentIndividual.get()));
		}
		
		
	}

	public void valueReplaced(Value oldValue, Value newValue) {
		mCurrentGeneration = (IntValue) ValueManager.synchronizeValue(mCurrentGeneration, oldValue, newValue, this);
		mCurrentIndividual = (IntValue) ValueManager.synchronizeValue(mCurrentIndividual, oldValue, newValue, this);
		mCurrentTry = (IntValue) ValueManager.synchronizeValue(mCurrentTry, oldValue, newValue, this);
		mCurrentStep = (IntValue) ValueManager.synchronizeValue(mCurrentStep, oldValue, newValue, this);
		mNumberOfTries = (IntValue) ValueManager.synchronizeValue(mNumberOfTries, oldValue, newValue, this);
		mNumberOfSteps = (IntValue) ValueManager.synchronizeValue(mNumberOfSteps, oldValue, newValue, this);
	}
	
	

	public void optimizePerformance(boolean optimize) {
		boolean showSteps = mShowSteps;
		if(optimize) {
			mStepLabel.setSelected(false);
		}
		else {
			if(showSteps) {
				mStepLabel.setSelected(true);
			}
		}
		handleCheckBoxChanged();
		mShowSteps = showSteps;
	}

	
	
	
	

}
