/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.visualization;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import de.fhg.iais.outdoor.simba.core.GenerationListener;
import de.fhg.iais.outdoor.simba.core.IndividualListener;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.core.StepListener;
import de.fhg.iais.outdoor.simba.core.TryListener;
import de.fhg.iais.outdoor.simba.fitness.FitnessFunction;

public class FitnessDisplay implements Visualization, StepListener, 
						OptimizePerformanceListener, GenerationListener,
						IndividualListener, TryListener {

	
	private JPanel mDisplay;
	private Box mContent;
	private JLabel mFitnessLabel;
	private JLabel mLastFitnessLabel;
	private JLabel mBestFitnessLabel;
	private JLabel mBestTryLabel;
	private JLabel mMeanLabel;
	private FitnessFunction mFitnessFunction;
	private JCheckBox mActivationCheckBox;
	private boolean mShowFitness;
	private double mCurrentFitness;
	private double mBestFitness;
	private double mLastFitness;
	private double mBestTryFitness;
	private double mMeanFitness;
	private int mBestIndividuum;
	private int mBestTry;
	
	public FitnessDisplay() {
		mActivationCheckBox = new JCheckBox("Activate");
		mActivationCheckBox.setSelected(true);
		
		mActivationCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				handleCheckBoxChanged();
			}
		});
		
		mDisplay = new JPanel(new BorderLayout());
		mDisplay.setBorder(BorderFactory.createTitledBorder("FitnessFunctions"));
		mContent = Box.createVerticalBox();
		mDisplay.add(BorderLayout.CENTER, mContent);
		mFitnessLabel = new JLabel("Fitness:");
		mLastFitnessLabel = new JLabel("Best: ");
		mBestFitnessLabel = new JLabel("Last: ");
		mBestTryLabel = new JLabel("Best Try: ");
		mMeanLabel = new JLabel("Mean: ");
		
		mDisplay.add(BorderLayout.NORTH, mActivationCheckBox);
		mContent.add(mFitnessLabel);
		mContent.add(mMeanLabel);
		mContent.add(mLastFitnessLabel);
		mContent.add(mBestTryLabel);
		mContent.add(mBestFitnessLabel);
		
		Simulator.getInstance().addOptimizePerformanceListener(this);
		Simulator.getInstance().getScheduler().addGenerationListener(this);
		Simulator.getInstance().getScheduler().addIndividualListener(this);
		Simulator.getInstance().getScheduler().addTryListener(this);
		
		mBestTry = -1;
		mBestIndividuum = -1;
		
		handleCheckBoxChanged();
	}
	
	public void handleCheckBoxChanged() {
		if(mActivationCheckBox.isSelected()) {
			mShowFitness = true;
			Simulator.getInstance().getScheduler().addStepListener(this);
			//Simulator.getInstance().getScheduler().addTryListener(this);
			updateLabels();
		}
		else {
			mShowFitness = false;
			Simulator.getInstance().getScheduler().removeStepListener(this);
			//Simulator.getInstance().getScheduler().removeTryListener(this);
		}
	}
	
	public JPanel getPanel() {
		return mDisplay;
	}

	public boolean cleanUp() {
		return true;
	}


	public void stepCompleted(int stepNumber) {
		if(mFitnessFunction != null) {
			mCurrentFitness = mFitnessFunction.getFitness();
			mMeanFitness = mFitnessFunction.getMeanFitness();
			updateLabels();
		}
	}

	public void stepStarted(int stepNumber) {
	}
	
	public void generationCompleted(int generationId) {
		mBestFitness = 0.0;
		mBestIndividuum = -1;
		updateLabels();
	}

	public void generationStarted(int generationId) {
	}

	public void individualCompleted(int individualId) {
		if(mFitnessFunction != null) {
			if(mMeanFitness > mBestFitness) {
				mBestFitness = mMeanFitness;
				mBestIndividuum = individualId;
			}
			updateLabels();
		}
		mBestTryFitness = 0.0;
		mBestTry = -1;
	}

	public void individualStarted(int individualId) {
		
	}

	public void tryCompleted(int tryId) {
		if(mFitnessFunction != null) {
			mCurrentFitness = mFitnessFunction.getFitness();
			mMeanFitness = mFitnessFunction.getMeanFitness();
			mLastFitness = mCurrentFitness;
			if(mCurrentFitness > mBestTryFitness) {
				mBestTryFitness = mCurrentFitness;
				mBestTry = tryId;
			}
			updateLabels();
		}
	}

	public void tryStarted(int tryId) {
	}

	private void updateLabels() {
		if(mFitnessFunction != null) {
			mFitnessLabel.setText("Current: " + mCurrentFitness);
			mLastFitnessLabel.setText("Last: " + mLastFitness);
			mBestTryLabel.setText("Best Try: " + mBestTryFitness + " (" + (mBestTry + 1) + ")");
			mMeanLabel.setText("Mean: " + mMeanFitness);
			mBestFitnessLabel.setText("Best: " + mBestFitness + " (" + (mBestIndividuum) + ")");
		}
	}

	public boolean init() {
		Vector fitnessFunctions = Simulator.getInstance().getFitnessManager().getFitnessFunctions();
		if(fitnessFunctions.size() > 0) {
			mFitnessFunction = (FitnessFunction) fitnessFunctions.elementAt(0);
			mActivationCheckBox.setText(mFitnessFunction.getName());
		}
		updateLabels();
		
		return true;
	}

	
	public void optimizePerformance(boolean optimize) {
		boolean show = mShowFitness;
		if(optimize) {
			mActivationCheckBox.setSelected(false);
		}
		else {
			mActivationCheckBox.setSelected(show);
		}
		handleCheckBoxChanged();
		mShowFitness = show;
	}

}
