/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.visualization;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBox;
import javax.swing.JPanel;

public class HideableVisualization extends VisualizationArea {

	private JPanel mDisplay;
	private JCheckBox mHideCheckBox;
	private JPanel mContent;
	
	public HideableVisualization() {
		mDisplay = new JPanel(new BorderLayout());
		
		mHideCheckBox = new JCheckBox();
		mHideCheckBox.setSelected(false);
		mHideCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(mContent == null) {
					mHideCheckBox.setSelected(false);
				}
				else {
					mContent.setVisible(mHideCheckBox.isSelected());
					notifyVisibilityChanged(mHideCheckBox.isSelected());
				}
			}
		});
		
		mContent = null;
		
		mDisplay.add(BorderLayout.WEST, mHideCheckBox);
	}
	
	protected boolean installVisualization(Visualization visu, String positionIdentifier) {
		if(visu != null && visu.getPanel() != null) {
			mContent = visu.getPanel();
			mDisplay.add(BorderLayout.CENTER, mContent);
			mHideCheckBox.setSelected(true);
			
			return true;
		}

		return false;
	}

	public JPanel getPanel() {
		return mDisplay;
	}

}
