/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.visualization;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.JPanel;
import javax.swing.JToggleButton;

import de.fhg.iais.outdoor.simba.core.Simulator;

public class OptimizePerformanceButton implements Visualization, OptimizePerformanceListener {

	
	private JPanel mDisplay;
	private JToggleButton mOptimizeButton;
	private Timer mTimer;
	private int mAutoOptimizeDelay;

	public OptimizePerformanceButton() {
		mDisplay = new JPanel();
		mOptimizeButton = new JToggleButton("Performance");
		mDisplay.add(mOptimizeButton);
		
		mOptimizeButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(mOptimizeButton.isSelected()) {
					if(!Simulator.getInstance().isPerformanceOptimized()) {
						Simulator.getInstance().optimizePerformance(true);
					}
				}
				else {
					if(Simulator.getInstance().isPerformanceOptimized()) {
						Simulator.getInstance().optimizePerformance(false);
					}
				}
			}
		});
		
		mAutoOptimizeDelay = 1000 * 60 * 15; //15 Minuten.
		mTimer = new Timer();
		mTimer.schedule(new AutoPerformanceTimerTask(), mAutoOptimizeDelay, mAutoOptimizeDelay); //15 minuten
	
		
		Simulator.getInstance().addOptimizePerformanceListener(this);
	}
	
	public JPanel getPanel() {
		return mDisplay;
	}

	public boolean cleanUp() {
		return true;
	}

	public boolean init() {
		return true;
	}

	public void optimizePerformance(boolean optimize) {
		mOptimizeButton.setSelected(optimize);
		if(mTimer != null) {
			mTimer.cancel();
		}
		if(!optimize) {
			mTimer = new Timer();
			mTimer.schedule(new AutoPerformanceTimerTask(), mAutoOptimizeDelay, mAutoOptimizeDelay); //15 minuten
		}
	}

	public class AutoPerformanceTimerTask extends TimerTask {
		
		public void run() {
			if(!Simulator.getInstance().isPerformanceOptimized()) {
				Simulator.getInstance().optimizePerformance(true);
				Simulator.getInstance().reportStatus("Automatic optimization delay of " + mAutoOptimizeDelay / 60 / 1000 + " minutes expired.");
			}
		}
	}
	
	
	
}
