/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.visualization;

import java.awt.BorderLayout;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.logging.Logger;

import javax.swing.BoxLayout;
import javax.swing.JFrame;

import de.fhg.iais.outdoor.simba.communication.UDPSchedulerConnection;
import de.fhg.iais.outdoor.simba.core.EnvironmentModel;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.evolution.EvolutionConfigLogHandler;
import de.fhg.iais.outdoor.simba.evolution.EvolutionManager;
import de.fhg.iais.outdoor.simba.evolution.SimbaDefaultEvolution;
import de.fhg.iais.outdoor.simba.fitness.FitnessFunction;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetAgentInterface;
import de.fhg.iais.outdoor.simba.neuralnet.NeuronWrapper;
import de.fhg.iais.outdoor.simba.statistic.NetworkStatistics;
import de.fhg.iais.outdoor.simba.visualization.evolution.EvolutionParameterPanel;
import de.fhg.iais.outdoor.simba.visualization.evolution.GenerationSummaryPanel;
import de.fhg.iais.outdoor.simba.visualization.neuron.NetworkTableTexConverter;
import de.fhg.iais.outdoor.simba.visualization.neuron.NeuralNetworkEditor;
import de.fhg.iais.outdoor.simba.visualization.valuepanel.ValuePanelDisplay;


public class SimbaDefaultFrame {
	
	private JFrame mFrame;
	private SimulationVisualization mVisu;
	
	public SimbaDefaultFrame(String name, int width, int height) {
		mFrame = new JFrame(name);
		mFrame.setSize(width, height);
		
		mFrame.addWindowListener(
			new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					Simulator.getInstance().shutDown();
					closeWindow(e);
				}
			}
		);
		
		//Install packages, extensions and modules.
		new NetworkStatistics();
		new EvolutionConfigLogHandler();
		new SimbaDefaultEvolution();
		
		
		//set up the gui
		initGui();
	}
	
	public void initGui() {
		createGUIStructure();
	}
	
	//TODO init and cleanup als systemobject anmelden. Position und Gre speichern.
	
	public void init() {
		fillGUIStructure();
		Simulator.getInstance().init();
		mFrame.setVisible(true);
	}
	
	public void createGUIStructure() {
		
		
		//Create main visualization
		mVisu = new SimulationVisualization();
		
		//Set offset and scaling of the visualization.
		mVisu.setOffset(-1 * (mFrame.getSize().width / 2.0), (mFrame.getSize().height / 2.0));
		mVisu.setScale(1.0);
		mVisu.setDefaultView();
		mVisu.setVisualizationInterval(1);
		
		
		
		VisualizationArea root = Simulator.getInstance().getVisualizationManager().getRootArea();
		root.addVisualization(mVisu, BorderLayout.CENTER);
		
		mFrame.getContentPane().setLayout(new BorderLayout());
		
		//Set the root Visualization as frame content.
		mFrame.getContentPane().add(root.getPanel(), BorderLayout.CENTER);
		
	}
	
	public void fillGUIStructure() {
		
		ShowDraggedObjectPositionVisualization draggedObjVisu = new ShowDraggedObjectPositionVisualization();
		SimulationDelayController delayController = new SimulationDelayController();
		StepsPerSecondDisplay stepsDisplay = new StepsPerSecondDisplay();
		VisualizationIntervalController intervalController = new VisualizationIntervalController("/Visualization/Control/VisualizationInterval");
		StatusMessageLabel statusMessages = new StatusMessageLabel(2500);
		EvolutionInformationDisplay evolutionInfo = new EvolutionInformationDisplay();
		FitnessDisplay fitDisp = new FitnessDisplay();
		TerminateTryButton terminateButton = new TerminateTryButton();
		OptimizePerformanceButton optimizeButton = new OptimizePerformanceButton();
		SkipVisualizationButtonPanel skipButtons = new SkipVisualizationButtonPanel();
		SingleStepButton stepButton = new SingleStepButton();
		
		VisualizationArea root = Simulator.getInstance().getVisualizationManager().getRootArea();
		root.addVisualization(mVisu, BorderLayout.CENTER);
		
		BorderLayoutVisualizationArea lowerArea = new BorderLayoutVisualizationArea();
		root.addVisualization(lowerArea, BorderLayout.SOUTH);
		
		BorderLayoutVisualizationArea lowerRightArea = new BorderLayoutVisualizationArea();
		lowerArea.addVisualization(lowerRightArea, BorderLayout.EAST);
	
		HideableVisualization infoHider = new HideableVisualization();
		BorderLayoutVisualizationArea lowerLeftArea = new BorderLayoutVisualizationArea();
		lowerLeftArea.addVisualization(evolutionInfo, BorderLayout.WEST);
		lowerLeftArea.addVisualization(fitDisp, BorderLayout.CENTER);
		infoHider.addVisualization(lowerLeftArea, "");
		
		
		BorderLayoutVisualizationArea miniButtonPanel = new BorderLayoutVisualizationArea();
		miniButtonPanel.addVisualization(stepButton, BorderLayout.EAST);
		miniButtonPanel.addVisualization(skipButtons, BorderLayout.CENTER);
		
		BorderLayoutVisualizationArea optimizationArea = new BorderLayoutVisualizationArea();
		optimizationArea.addVisualization(optimizeButton, BorderLayout.WEST);
		optimizationArea.addVisualization(miniButtonPanel, BorderLayout.CENTER);
		optimizationArea.addVisualization(terminateButton, BorderLayout.EAST);
		
		BoxLayoutVisualizationArea boxArea = new BoxLayoutVisualizationArea();
		Simulator.getInstance().getVisualizationManager().addVisualizationArea("LowerBox", boxArea);
		lowerArea.addVisualization(boxArea, BorderLayout.CENTER);
		lowerRightArea.addVisualization(intervalController, BorderLayout.EAST);
		
		
		boxArea.addVisualization(stepsDisplay, "");
		boxArea.addVisualization(draggedObjVisu, "");
		boxArea.addVisualization(infoHider, "");
		boxArea.addVisualization(optimizationArea, "");
		boxArea.addVisualization(statusMessages, "");
		
		BoxLayoutVisualizationArea horizontalBox = new BoxLayoutVisualizationArea(BoxLayout.X_AXIS);
		Simulator.getInstance().getVisualizationManager().addVisualizationArea("HorizontalBox", horizontalBox);
		
		//TODO remove
		FrameVisualizationArea valueDialog = new FrameVisualizationArea("Values", 500, 500, 
				mFrame.getLocation().x + mFrame.getWidth(), mFrame.getLocation().y);
		ValuePanelDisplay valuePanel = new ValuePanelDisplay();
		valueDialog.addVisualization(valuePanel, BorderLayout.CENTER);
		
		horizontalBox.addVisualization(valueDialog, "");
		
		FrameVisualizationArea simValueDialog = new FrameVisualizationArea("SimValues", 500, 500, 
				mFrame.getLocation().x + mFrame.getWidth(), mFrame.getLocation().y);
		ValuePanelDisplay simValuePanel = new ValuePanelDisplay("/Simulation.*");
		simValueDialog.addVisualization(simValuePanel, BorderLayout.CENTER);
		
		horizontalBox.addVisualization(simValueDialog, "");
		
		
		//TODO remove
		FrameVisualizationArea neuronDialog = new FrameVisualizationArea("Neurons", 700, 700, 
				mFrame.getLocation().x + mFrame.getWidth(), mFrame.getLocation().y);
		NeuralNetworkEditor neuronEditor = new NeuralNetworkEditor();
		neuronDialog.addVisualization(neuronEditor, BorderLayout.CENTER);
		
		StaseButton staseButton = new StaseButton();
		horizontalBox.addVisualization(staseButton, "");


		EvolutionParameterPanel evoArea = new EvolutionParameterPanel(700, 700, 
				mFrame.getLocation().x + mFrame.getWidth(), mFrame.getLocation().y);
		horizontalBox.addVisualization(evoArea, "");
		
		
		FrameVisualizationArea summaryDialog = new FrameVisualizationArea("Summary", 500, 500,
				mFrame.getLocation().x + mFrame.getWidth(), mFrame.getLocation().y);
		GenerationSummaryPanel summary = new GenerationSummaryPanel();
		summaryDialog.addVisualization(summary, BorderLayout.CENTER);
		
		horizontalBox.addVisualization(summaryDialog, "");
		
		horizontalBox.addVisualization(neuronDialog, "");
		boxArea.addVisualization(horizontalBox, "");
		
		root.addVisualization(delayController, BorderLayout.EAST);
		
		
		NetworkTableTexConverter converter = new NetworkTableTexConverter();
		FrameVisualizationArea converterDialog = new FrameVisualizationArea("Latex Code Converter", 700, 300, 
				mFrame.getLocation().x + mFrame.getWidth(), mFrame.getLocation().y);
		converterDialog.addVisualization(converter, BorderLayout.CENTER);
		horizontalBox.addVisualization(converterDialog, "");
		
	}
	
	public void closeWindow(WindowEvent e) {
		System.exit(0);
	}
	
	public void setMasterEnvironment(EnvironmentModel environment) {
		Simulator.getInstance().getScheduler().setMasterEnvironment(environment);
	}
	
	public void setDefaultOffset(double offsetX, double offsetY) {
		mVisu.setOffset(offsetX, offsetY);
		mVisu.setDefaultView();
	}
	
	public void setDefaultZoom(double zoom) {
		mVisu.setScale(zoom);
		mVisu.setDefaultView();
	}
	
	public void setSimulationDelay(int delayMillies) {
		Simulator.getInstance().setSimulationDelay(delayMillies);
	}
	
	public void setVisualizationInterval(int intervalSteps) {
		mVisu.setVisualizationInterval(intervalSteps);
	}
	
	public JFrame getFrame() {
		return mFrame;
	}
	
	public boolean setFitnessFunction(FitnessFunction fitnessFunction, double maxFitness) {
		
		Simulator.getInstance().getFitnessManager().addFitnessFunction(
				fitnessFunction);
		
		NeuronWrapper fitness = new NeuronWrapper("/Fitness", 0.0, 0.0, maxFitness);
		if(!Simulator.getInstance().getValueManager().replaceValue(
				"/FitnessFunctions/" + fitnessFunction.getName() + "/MeanFitness", fitness))
		{
			Logger.getLogger("Simulator").warning("SimbaDefaultFrame.setFitnessFunction: Could not replace fitness with name " + fitnessFunction.getName());
			return false;
		}
		NeuralNetAgentInterface agentInterface = Simulator.getInstance().getNeuralNetManager().getNeuralNetInterface();
		if(agentInterface != null) {
			agentInterface.addInfoNeuron(fitness);
		}
		else {
			Logger.getLogger("Simulator").warning("SinbaDefaultFrame.setFitnessFunction: Could not locate neural net interface.");
			return false;
		}
		return true;
	}
	
	public void startSystem(NeuralNetAgentInterface controller, String[] args, int gen, int ind, int tries, int steps) {
		int port = 50100;
		Simulator.getInstance().getNeuralNetManager().setNeuralNetInterface(controller);
		init();
		if(args.length > 0) {
			if(args[0].equals("sim")) {
				Simulator.getInstance().emulateEvolutionRun(gen, ind, tries, steps);
				return;
			}
			else if(args[0].equals("evo")) {
				//TODO should start with a button.
				Thread t = new Thread() {
					public void run() {
						EvolutionManager evoManager = Simulator.getInstance().getEvolutionManager();
						evoManager.runEvolution();
					}
				};
				t.setPriority(Thread.NORM_PRIORITY - 1);
				t.start();	
			}
			else {
				try {
					int portNumber = Integer.parseInt(args[0]);
					port = portNumber;
				} catch (NumberFormatException e) {
				}
				UDPSchedulerConnection connection = new UDPSchedulerConnection(controller);
				connection.startServer(port);
			}
		}
		else {
			UDPSchedulerConnection connection = new UDPSchedulerConnection(controller);
			connection.startServer(port);
		}
		
	}
	
	public void startSystem(NeuralNetAgentInterface controller, String[] args) {
		startSystem(controller, args, -1, 15, 12, 2500);
	}
	

}
