/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.visualization;


import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JPanel;
import javax.swing.JToggleButton;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueChangedListener;
import de.fhg.iais.outdoor.simba.value.ValueManager;

public class StaseButton implements Visualization, ValueChangedListener {

	private JPanel mDisplay;
	private JToggleButton mStaseButton;
	private BooleanValue mStaseValue;
	private BooleanValue mEvolutionRunning;
	private Color mButtonBackgroundColor;
	
	public StaseButton() {
		mDisplay = new JPanel();
		mStaseButton = new JToggleButton("Stase");
		mDisplay.add(mStaseButton);
		mButtonBackgroundColor = mDisplay.getBackground();
		
		mStaseButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(mStaseValue != null) {
					mStaseValue.set(mStaseButton.isSelected());
					if(mStaseButton.isSelected()) {
						mDisplay.setBackground(Color.RED);
					}
					else {
						mDisplay.setBackground(mButtonBackgroundColor);
					}
				}
			}
		});
	}
	
	
	public JPanel getPanel() {
		return mDisplay;
	}

	public boolean cleanUp() {
		return true;
	}

	public boolean init() {

		mStaseValue = Simulator.getInstance().getValueManager().getBooleanValue("/Evolution/Control/StaseMode");
		mEvolutionRunning = Simulator.getInstance().getValueManager().getBooleanValue("/Evolution/Status/Running");
		
		if(mStaseValue != null) {
			mStaseValue.addValueChangedListener(this);
		}
		if(mEvolutionRunning != null) {
			mEvolutionRunning.addValueChangedListener(this);
		}

		mStaseButton.setEnabled(mStaseValue != null && mEvolutionRunning != null && mEvolutionRunning.get());
		if(!mStaseButton.isEnabled()) {
			mDisplay.setBackground(mButtonBackgroundColor);
		}
		return true;
	}


	public void valueChanged(Value value) {
		if(value != null && value == mStaseValue) {
			mStaseButton.setSelected(mStaseValue.get());
			if(mStaseButton.isSelected()) {
				mDisplay.setBackground(Color.RED);
			}
			else {
				mDisplay.setBackground(mButtonBackgroundColor);
			}
		}
		mStaseButton.setEnabled(mStaseValue != null && mEvolutionRunning != null && mEvolutionRunning.get());
		if(!mStaseButton.isEnabled()) {
			mDisplay.setBackground(mButtonBackgroundColor);
		}
	}


	public void valueReplaced(Value oldValue, Value newValue) {
		mStaseValue = (BooleanValue) ValueManager.synchronizeValue(mStaseValue, oldValue, newValue, this);
		mEvolutionRunning = (BooleanValue) ValueManager.synchronizeValue(mEvolutionRunning, oldValue, newValue, this);
		
		mStaseButton.setEnabled(mStaseValue != null && mEvolutionRunning != null && mEvolutionRunning.get());
		if(!mStaseButton.isEnabled()) {
			mDisplay.setBackground(mButtonBackgroundColor);
		}
	}

	
}
