/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.visualization;

import java.awt.BorderLayout;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.JLabel;
import javax.swing.JPanel;

import de.fhg.iais.outdoor.simba.core.StepListener;
import de.fhg.iais.outdoor.simba.core.Simulator;

public class StepsPerSecondDisplay implements Visualization, StepListener {
	
	JPanel mDisplay;
	JLabel mLabel;
	int mCounter;
	Timer mTimer;
	
	public StepsPerSecondDisplay() {
		mDisplay = new JPanel(new BorderLayout());
		mLabel = new JLabel("Steps per second: ");
		mDisplay.add(BorderLayout.CENTER, mLabel);
		mCounter = 0;
		mTimer = new Timer();
	}
	
	public boolean init() {
		boolean ok = true;
		Simulator.getInstance().getScheduler().addStepListener(this);
		mTimer.cancel();
		mTimer = new Timer();
		mTimer.scheduleAtFixedRate(new StepTimer(), 1000, 1000);
		mCounter = 0;
		return ok;
	}
	
	public boolean cleanUp() {
		boolean ok = true;
		mTimer.cancel();
		return ok;
	}
	
	public JPanel getPanel() {
		return mDisplay;
	}
	
	public void stepStarted(int stepNumber) {
		
	}
	
	public synchronized void stepCompleted(int step) {
		mCounter++;
	}
	
	public class StepTimer extends TimerTask {
	
		public synchronized void run() {
			mLabel.setText("Steps per second: " + mCounter);
			mCounter = 0;	
		}	
	}


}
