/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.visualization;

import java.awt.BorderLayout;

import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

public class TabbedVisualizationArea extends VisualizationArea {
	
	private JPanel mDisplay;
	private JTabbedPane mTabbedPane;
	private boolean mVisible;
	private int mLastVisibleIndex;
	
	public TabbedVisualizationArea() {
		this(JTabbedPane.TOP);
		
		mTabbedPane.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				if(mLastVisibleIndex >= 0 && mLastVisibleIndex != mTabbedPane.getSelectedIndex()) {
					Visualization visu = (Visualization) mVisualizations.elementAt(mLastVisibleIndex);
					if(visu instanceof VisibilityListener) {
						((VisibilityListener) visu).visibilityChanged(false, this);
					}
				}
				if(mVisible) {
					Visualization visu = (Visualization) mVisualizations.elementAt(mTabbedPane.getSelectedIndex());
					if(visu instanceof VisibilityListener) {
						((VisibilityListener) visu).visibilityChanged(true, this);
					}
				}
				mLastVisibleIndex = mTabbedPane.getSelectedIndex();
			}
		});
	}
	
	public TabbedVisualizationArea(int tabPlacement) {
		mDisplay = new JPanel(new BorderLayout());
		mTabbedPane = new JTabbedPane();
		mDisplay.add(mTabbedPane, BorderLayout.CENTER);
	}

	protected boolean installVisualization(Visualization visu, String positionIdentifier) {
		mTabbedPane.add(visu.getPanel(), positionIdentifier);
		mLastVisibleIndex = mTabbedPane.getSelectedIndex();
		return true;
	}

	public JPanel getPanel() {
		return mDisplay;
	}

	public void visibilityChanged(boolean visible, Object source) {
		mVisible = visible;
		for(int i = 0; i < mVisualizations.size(); ++i) {
			Object obj = mVisualizations.elementAt(i);
			if(obj instanceof VisibilityListener) {
				if(i != mTabbedPane.getSelectedIndex()) {
					((VisibilityListener) obj).visibilityChanged(false, this);
				}
				else {
					((VisibilityListener) obj).visibilityChanged(visible, this);
				}
			}
			
		}
	}
	
	

}
