/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.visualization;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JPanel;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.value.BooleanValue;

public class TerminateTryButton implements Visualization {

	
	private JPanel mDisplay;
	private JButton mTerminateButton;
	private JCheckBox mEnableTerminateButtonCheckBox;
	private BooleanValue mTerminateTryValue;
	
	
	public TerminateTryButton() {
		mDisplay = new JPanel();
		mTerminateButton = new JButton("Terminate Try");
		mEnableTerminateButtonCheckBox = new JCheckBox();
		mEnableTerminateButtonCheckBox.setSelected(false);
		mDisplay.add(mEnableTerminateButtonCheckBox);
		mDisplay.add(mTerminateButton);
		
		mTerminateButton.setEnabled(false);
		mEnableTerminateButtonCheckBox.setEnabled(false);
		
		mTerminateButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(mTerminateTryValue != null && !mTerminateTryValue.get()) {
					mTerminateTryValue.set(true);
				}
			}
		});
		
		mEnableTerminateButtonCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(mEnableTerminateButtonCheckBox.isSelected()) {
					mTerminateButton.setEnabled(true);
				}
				else {
					mTerminateButton.setEnabled(false);
				}
			}
		});
	}
	
	
	public JPanel getPanel() {
		return mDisplay;
	}

	public boolean cleanUp() {
		return true;
	}

	public boolean init() {

		mTerminateTryValue = Simulator.getInstance().getValueManager().getBooleanValue("/Simulation/TerminateCurrentTry");
		
		if(mTerminateTryValue != null) {
			mEnableTerminateButtonCheckBox.setEnabled(true);
			if(mEnableTerminateButtonCheckBox.isSelected()) {
				mTerminateButton.setEnabled(true);
			}
		}
		
		return true;
	}

}
