/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.visualization;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.util.logging.Logger;

import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.value.IntValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueChangedListener;

public class VisualizationIntervalController implements Visualization, ValueChangedListener, ChangeListener { 

	private JPanel mDisplay;
	private JSlider mSlider;
	private IntValue mVisualizationInterval;
	private boolean mChangingInterval;
	private String mIntervalValueName;
	
	public VisualizationIntervalController(String intervalValueName) {
		this(intervalValueName, SwingConstants.VERTICAL);	
	}
	
	public VisualizationIntervalController(String intervalValueName, int orientation) {
		mIntervalValueName = intervalValueName;
		mDisplay = new JPanel(new BorderLayout());
		mSlider = new JSlider(orientation, 1, 100, 1);
		mSlider.addChangeListener(this);
		mDisplay.add(BorderLayout.CENTER, mSlider);
		mSlider.setPreferredSize(new Dimension(20, 40));
		mChangingInterval = false;
	}
	
	public boolean init() {
		boolean ok = true;
		
		acquireValueObjects();
		
		return ok;
	}
	
	private void acquireValueObjects() {
		mVisualizationInterval= Simulator.getInstance().getValueManager().getIntValue(mIntervalValueName);
		if(mVisualizationInterval == null) {
			Logger.getLogger("Simulator")
				.warning("VisualizationIntervalController: Could not find IntValue:\n" +
						"[/Visualization/Control/VisualizationInterval]");
		}
		else {
			mSlider.setValue(mVisualizationInterval.get());
			mVisualizationInterval.addValueChangedListener(this);
		}
	}
	
	public boolean cleanUp() {
		boolean ok = true;
		
		return ok;
	}
	
	public JPanel getPanel() {
		return mDisplay;
	}
	
	public void valueChanged(Value value) {
		if(mVisualizationInterval != null && value == mVisualizationInterval && !mChangingInterval) {
			mSlider.setValue(mVisualizationInterval.get());
		}
	}
	
	/**
	 * Called if the one of the registered Value objects were replaced in the ValueManager.
	 */
	 public void valueReplaced(Value oldValue, Value newValue) {
		if(oldValue != newValue && oldValue != null) {
			oldValue.removeValueChangedListener(this);
			if(mVisualizationInterval  == oldValue) {
				acquireValueObjects();
			}
		}
	 }
	
	public void stateChanged(ChangeEvent e) {
		if(mVisualizationInterval != null) {
			mChangingInterval = true;
			mVisualizationInterval.set(mSlider.getValue());
			mChangingInterval = false;
			Simulator.getInstance().reportStatus("New visualization interval: " + mVisualizationInterval.get() + " steps.");
		}
	}
	
	/**
	 * Sets the maximal interval this slider can set.
	 * @param maxInterval the maximal interval.
	 */
	public void setMaximalInterval(int maxInterval) {
		mSlider.setMaximum(maxInterval);
		if(mVisualizationInterval != null && mVisualizationInterval.get() > maxInterval) {
			mChangingInterval = true;
			mVisualizationInterval.set(maxInterval);
			mChangingInterval = false;
		}
	}
	
	/**
	 * Returns the maximum this slider control can set.
	 * @return the maximum.
	 */
	public int getMaximalInterval() {
		return mSlider.getMaximum();
	}

}
