/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.visualization;

import java.util.Hashtable;
import java.util.Vector;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.value.ValueManager;

public class VisualizationManager {
	
	private Hashtable mVisualizationAreas = new Hashtable();
	private VisualizationArea mRootArea;
	private Vector mVisualizations = new Vector();
	
	public VisualizationManager(ValueManager valueManager) {
		mRootArea = new BorderLayoutVisualizationArea();
		mVisualizationAreas.put("root", mRootArea);
	}
	
	/**
	 * Registers a VisualizationArea with a new name. A visualizationArea can be 
	 * associated with several names.
	 * 
	 * @param name the name the VisualizationArea is associated with.
	 * @param area the area to associate with the name.
	 * @return
	 */
	public boolean addVisualizationArea(String name, VisualizationArea area) {
		if(area == null || name == null || mVisualizationAreas.containsKey(name)) {
			return false;
		}
		mVisualizationAreas.put(name, area);
		Simulator.getInstance().addSystemObject(area);
		return true;
	}
	
	/**
	 * Adds a Visualisation. Visualizations that are added to a VisualizationArea
	 * are automatically added here.
	 * 
	 * @param visu the Visualization to add.
	 * @return true if successful, otherwise false.
	 */
	public boolean addVisualization(Visualization visu) {
		if(visu == null || mVisualizations.contains(visu)) {
			return false;
		}
		mVisualizations.add(visu);
		return true;
	}
	
	/**
	 * Returns a Vector with all known Visualization objects.
	 * 
	 * @return a Vector with all Visualizations.
	 */
	public Vector getVisualizations() {
		return new Vector(mVisualizations);
	}
	
	/**
	 * Returns the VisualiationArea object that is associated with the given name.
	 * If no such object exists, then null is returned. 
	 * @param name the associated name of the desired object.
	 * @return the desired VisualizationArea object if found, otherwise null.
	 */
	public VisualizationArea getVisualizationArea(String name) {
		Object area = mVisualizationAreas.get(name);
		if(area != null) {
			return (VisualizationArea) area;
		}
		return null;
	}
	
	/**
	 * Returns the default root area. 
	 * @return the root area.
	 */
	public VisualizationArea getRootArea() {
		return mRootArea;
	}
	


}
