/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.visualization.evolution;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.Properties;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.visualization.Visualization;
import de.fhg.iais.outdoor.simba.visualization.valuepanel.ValuePanel;

public class ChooseLogFileNameVisualization implements Visualization {

	private JPanel mDisplay;
	private ValuePanel mLogFilePanel;
	private JButton mChooseFileNameButton;
	private JFileChooser mFileChooser;
	
	public ChooseLogFileNameVisualization() {
		mDisplay = new JPanel();
		
		mLogFilePanel = new ValuePanel(null, "/Simulation/LogFileName", "LogFile", 35, false, true);
		mLogFilePanel.setBorder(BorderFactory.createEtchedBorder());
		mDisplay.add(mLogFilePanel);
	
		mChooseFileNameButton = new JButton("File");
		mChooseFileNameButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				chooseFileName();
			}
		});
		mDisplay.add(mChooseFileNameButton);
		
		mFileChooser = new JFileChooser();
		mFileChooser.setApproveButtonText("Choose");
		
		Simulator.getInstance().addSystemObject(this);
	}
	
	protected void chooseFileName() {
		//Choose directory.
		Properties props = Simulator.getInstance().getSystemProperties();
		String directory = props.getProperty("/FileChooser/CurrentDirectory");
		if(directory != null) {
			mFileChooser.setCurrentDirectory(new File(directory));
		}
		//if there is already a file name, then set this file name
		String fileName = mLogFilePanel.getValueContent();
		int index = fileName.lastIndexOf(File.pathSeparator);
		if(index != -1) {
			fileName = fileName.substring(index);
		}
		if(fileName.length() > 0) {
			mFileChooser.setSelectedFile(new File(fileName));
		}
		
		//Save parameters
		if(mFileChooser.showSaveDialog(null) == JFileChooser.APPROVE_OPTION) {
			File file = mFileChooser.getSelectedFile();
			if(!file.getName().endsWith(".elg")) {
				file = new File(file.getAbsolutePath() + ".elg");
			}
			
			//Memorize directory
			directory = mFileChooser.getCurrentDirectory().getPath();
			props.put("/FileChooser/CurrentDirectory", directory);
			
			if(file.exists()) {
				int res = JOptionPane.showConfirmDialog(null, "A file with name " + file.getName() + " already exists.\nDo you want to overwrite it?", 
								"File exists warning", JOptionPane.OK_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE);
				if(res == JOptionPane.CANCEL_OPTION) {
					return;
				}
			}
			//set file name
			mLogFilePanel.setValueContent(file.getPath());
			mLogFilePanel.setValueFromContentField();
		}
		else {
			//Memorize directory
			directory = mFileChooser.getCurrentDirectory().getPath();
			props.put("/FileChooser/CurrentDirectory", directory);
		}
	}
	
	public JPanel getPanel() {
		return mDisplay;
	}

	public boolean cleanUp() {
		return true;
	}

	public boolean init() {
		mLogFilePanel.setValue(Simulator.getInstance().getScheduler().getLogFileName());
		return true;
	}
	
	public ValuePanel getLogFilePanel() {
		return mLogFilePanel;
	}

}
