/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.visualization.evolution;

import de.fhg.iais.outdoor.simba.core.Scheduler;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;

public class NetworkRunner extends Thread {
	
	private NeuralNet mNet;
	private int mNumberOfSteps;
	private int mNumberOfTries;
	private int mId;
	private boolean mTerminate;
	
	public NetworkRunner(NeuralNet net, int id, int numberOfTries, int numberOfSteps) {
		mNet = net;
		mNumberOfSteps = numberOfSteps;
		mNumberOfTries = numberOfTries;
		mId = id;
		mTerminate = false;
	}

	public void run() {
		if(mNet == null) {
			return;
		}
		
		Simulator.getInstance().getNeuralNetManager().setCurrentNet(mNet);
		Scheduler scheduler = Simulator.getInstance().getScheduler();
		
		if(mTerminate) {
			return;
		}
		
		scheduler.prepareNextIndividual(mId);
		for(int tries = 0; tries < mNumberOfTries; ++tries) {
			scheduler.prepareNextTry(tries);
			for(int steps = 0; steps < mNumberOfSteps; ++steps) {
				scheduler.executeSimulationStep(steps);
				if(mTerminate) {
					return;
				}
				if(scheduler.shouldCurrentTryBeTerminated()) {
					break;
				}
			}
		}
	}

	public void terminateRunner() {
		mTerminate = true;
	}
}
