/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.visualization.evolution;

import java.awt.BorderLayout;
import java.util.Iterator;
import java.util.Vector;


import javax.swing.Box;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import de.fhg.iais.outdoor.simba.core.ParameterElement;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.evolution.EvolutionManager;
import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueChangedListener;
import de.fhg.iais.outdoor.simba.value.ValueManager;
import de.fhg.iais.outdoor.simba.visualization.Visualization;

public class ParameterPanel implements Visualization, ValueChangedListener {
	
	private JPanel mDisplay;
	private Box mContent;
	private BooleanValue mEvolutionEnabled;
	private Box mOperatorArea;
	private JScrollPane mScrollPanel;
	private Vector mParamPanels = new Vector();
	
	public ParameterPanel(Vector parameterElements) {
		this(parameterElements, null);
	}
	
	public ParameterPanel(Vector parameterElements, Vector ignoreList) {
		mDisplay = new JPanel(new BorderLayout());
		mOperatorArea = Box.createVerticalBox();
		
		mContent = Box.createVerticalBox();
		
		JPanel mainPanel = new JPanel(new BorderLayout());
		mScrollPanel = new JScrollPane(mainPanel);
		mainPanel.add(mContent, BorderLayout.NORTH);
		mDisplay.add(mScrollPanel, BorderLayout.CENTER);
		
		mContent.add(mOperatorArea);
		
		EvolutionManager em = Simulator.getInstance().getEvolutionManager();
		
		mEvolutionEnabled = em.getEvolutionEnabledValue();
		mEvolutionEnabled.addValueChangedListener(this);
		
		for(Iterator i = parameterElements.iterator(); i.hasNext();) {
			ParameterElement pe = (ParameterElement) i.next();
			ParameterElementPanel panel = new ParameterElementPanel(pe, ignoreList);
			mOperatorArea.add(panel);
			mParamPanels.add(panel);
		}
		
		mOperatorArea.revalidate();
		mScrollPanel.revalidate();
		
	}

	public JPanel getPanel() {
		return mDisplay;
	}

	public boolean cleanUp() {
		return true;
	}

	public boolean init() {

		return true;
	}

	public void valueChanged(Value value) {
		if(value != null) {
			if(value == mEvolutionEnabled) {
				if(mEvolutionEnabled.get()) {
					
				}
			}
		}
	}
	
	public void valueReplaced(Value oldValue, Value newValue) {
		mEvolutionEnabled = (BooleanValue) ValueManager.synchronizeValue(mEvolutionEnabled, oldValue, newValue, this);
	}
	
	public Vector getValuePanels() {
		Vector panels = new Vector();
		for(Iterator i = mParamPanels.iterator(); i.hasNext();) {
			panels.addAll(((ParameterElementPanel) i.next()).getValuePanels());
		}
		return panels;
	}
	

}
