/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.visualization.evolution;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JPanel;
import javax.swing.JToggleButton;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueChangedListener;
import de.fhg.iais.outdoor.simba.value.ValueManager;
import de.fhg.iais.outdoor.simba.visualization.Visualization;

public class RunEvolutionButton implements Visualization, ValueChangedListener {

	JPanel mDisplay;
	private JToggleButton mStartEvolutionButton;
	private BooleanValue mEvolutionEnabled;
	private BooleanValue mEvolutionRunning;
	private BooleanValue mRunEvolution;
	
	public RunEvolutionButton() {
		mDisplay = new JPanel();
		
		mStartEvolutionButton = new JToggleButton("Run Evolution");
		mStartEvolutionButton.setEnabled(false);
		mStartEvolutionButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				mRunEvolution.set(mStartEvolutionButton.isSelected());
			}
		});
		
		//Acquire control Values.
		mEvolutionEnabled = Simulator.getInstance().getEvolutionManager().getEvolutionEnabledValue();
		mEvolutionRunning = Simulator.getInstance().getEvolutionManager().getEvolutionRunningValue();
		mRunEvolution = Simulator.getInstance().getEvolutionManager().getRunEvolutionValue();
		
		mEvolutionEnabled.addValueChangedListener(this);
		mEvolutionRunning.addValueChangedListener(this);
		mRunEvolution.addValueChangedListener(this);
		
		mDisplay.add(mStartEvolutionButton);
		
	}
	
	public JPanel getPanel() {
		return mDisplay;
	}

	public boolean cleanUp() {
		return true;
	}

	public boolean init() {
		return true;
	}
	
	public void valueReplaced(Value oldValue, Value newValue) {
		mEvolutionEnabled = (BooleanValue) ValueManager.synchronizeValue(mEvolutionEnabled, oldValue, newValue, this);
		mEvolutionRunning = (BooleanValue) ValueManager.synchronizeValue(mEvolutionRunning, oldValue, newValue, this);
		mRunEvolution = (BooleanValue) ValueManager.synchronizeValue(mRunEvolution, oldValue, newValue, this);
	}
	
	public void valueChanged(Value value) {
		if(value == null) {
			return;
		}
		if(value == mEvolutionEnabled) {
			mStartEvolutionButton.setEnabled(mEvolutionEnabled.get());
		}
		else if(value == mEvolutionRunning || value == mRunEvolution) {
			mStartEvolutionButton.setSelected(mRunEvolution.get());
			
			if(!mEvolutionRunning.get()) {
				mStartEvolutionButton.setText("Start Evolution");
			}
			else if(!mRunEvolution.get()) {
				mStartEvolutionButton.setText("Stopping Evolution");
			}
			else {
				mStartEvolutionButton.setText("Stop Evolution");
			}
		}
	}

}
