/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.visualization.neuron;


import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetListener;
import de.fhg.iais.outdoor.simba.visualization.SimulationVisualization;
import de.fhg.iais.outdoor.simba.visualization.VisibilityListener;

public class NetworkViewer extends SimulationVisualization implements NeuralNetListener,
								VisibilityListener {
	
	private NeuronVisualizationEnvironment mNeuronEnvironment;
	private boolean mOptimized;
	private boolean mVisible;
	private boolean mNetworkInvalid;
	
	public NetworkViewer(NeuronVisualizationEnvironment environmentToVisualize) {
		super("/Visualization/Neurons/Control", environmentToVisualize);
		mNeuronEnvironment = environmentToVisualize;
		Simulator.getInstance().getScheduler().removeStepListener(this);
		mNetworkInvalid = true;
	}
	
	
	public boolean init() {
		boolean ok = super.init();
		
		Simulator.getInstance().getNeuralNetManager().addNeuralNetListener(this);
		
		return ok;
	}


	/**
	 * Repaints the visualization after each step.
	 */
	public void repaintNetwork() {
		mVisualizationIntervalCounter++;
		if(mShowSimulation.get()) {
			if(mVisualizationIntervalCounter >= mVisualizationInterval.get()) {
				mVisualizationIntervalCounter = 0;
				mVisuArea.repaint();
			}
		}
	}


	public void optimizePerformance(boolean optimize) {
		super.optimizePerformance(optimize);
		Simulator.getInstance().getScheduler().removeStepListener(this); //always remove.
		if(!optimize && mVisible) {
			//Simulator.getInstance().getNeuralNetManager().addNeuralNetListener(this);
			if(mNetworkInvalid && mNeuronEnvironment != null) {
				mNeuronEnvironment.resetEnvironmentObjects();
				mNetworkInvalid = false;
			}
		}
		else {
			//Simulator.getInstance().getNeuralNetManager().removeNeuralNetListener(this);
		}
		mOptimized = optimize;
	}


	public void currentNeuralNetChanged(NeuralNet currentNet) {
		if(!mVisible || mOptimized) {
			mNetworkInvalid = true;
			return;
		}
		if(mNeuronEnvironment != null) {
			mNeuronEnvironment.resetEnvironmentObjects();
		}
		repaintNetwork();
	}


	public void networkActivationsUpdated(NeuralNet currentNet) {
		if(!mVisible || mOptimized) {
			return;
		}
		mModelToVisualize.initEnvironment();
		mModelToVisualize.executeSimulationStep();
		repaintNetwork();
	}


	public void networkStructureChanged(NeuralNet currentNet) {
		if(!mVisible || mOptimized) {
			mNetworkInvalid = true;
			return;
		}
		if(mNeuronEnvironment != null) {
			mNeuronEnvironment.resetEnvironmentObjects();
		}
		repaintNetwork();
	}


	public void visibilityChanged(boolean visible, Object source) {
		if(!mOptimized && visible) {
			//Simulator.getInstance().getNeuralNetManager().addNeuralNetListener(this);
			//mNeuronEnvironment.resetEnvironmentObjects();
			if(mNetworkInvalid && mNeuronEnvironment != null) {
				mNeuronEnvironment.resetEnvironmentObjects();
				mNetworkInvalid = false;
			}
		}
		else {
			//Simulator.getInstance().getNeuralNetManager().removeNeuralNetListener(this);
		}
		mVisible = visible;
	}
	
	
	
	

}
