/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.visualization.neuron;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Graphics2D;

import de.fhg.iais.outdoor.simba.devices.Box;
import de.fhg.iais.outdoor.simba.neuralnet.Neuron;

public class NeuronActivationChangeBar extends Box {
	

	private Neuron mNeuron;
	private Color mPositiveColor;
	private Color mNegativeColor;
	private double mLastActivity;
	private double mCurrentActivity;
	
	public NeuronActivationChangeBar(String name, double x, double y, double angle, 
			double width, double height, boolean registerAtValueManager) 
	{
		super(name, x, y, angle, width, height, registerAtValueManager);
		
		mPositiveColor = Color.RED;
		mNegativeColor = Color.BLUE;
		setShapeColor(Color.WHITE);
	}

	public boolean init() {
		boolean ok = super.init();
		return ok;
	}
	
	public synchronized void setNeuron(Neuron neuron) {
		mNeuron = neuron;
	}

	public synchronized void paintSelf(Graphics2D g) {
		super.paintSelf(g);
		Color c = g.getColor();
		Composite composite = g.getComposite();
		g.setColor(mShapeColor);
		g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, (float) mTransparency.get()));
		
		
		g.setColor(Color.DARK_GRAY);
		g.drawRect((int) (-getWidth() / 2.0), (int) (-getHeight() / 2.0), (int) getWidth(), (int) getHeight());
		if(mNeuron != null && mNeuron.getActivationValue() != null) {
			
			double bias = Math.min(1.0, Math.max(-1.0, mNeuron.getBias()));
			if(bias > 0.0) {
				g.setColor(mPositiveColor);
				double heightOfBar = ((getHeight() / 2.0) - 1.0) * bias;
				g.fillRect((int) (-getWidth() / 2.0) + 1, 0, Math.max(1, (int) ((getWidth() - 2) / 3.0)), (int) Math.round(heightOfBar));
			}
			else {
				g.setColor(mNegativeColor);
				double heightOfBar = ((getHeight() / 2.0) - 1.0) * bias;
				g.fillRect((int) (-getWidth() / 2.0) + 1, (int) Math.round(heightOfBar), Math.max(1, (int) ((getWidth() - 2) / 3.0)), (int) Math.round(heightOfBar) * -1);
			}
			
			double activity = mNeuron.getActivationValue().getNormalized();
			if(activity > 0.0) {
				g.setColor(mPositiveColor);
				double heightOfBar = ((getHeight() / 2.0) - 1.0) * activity;
				g.fillRect((int) (-getWidth() / 2.0) + (int)(getWidth() / 3.0) + 1, 0, Math.max(1, (int) ((getWidth() - 2) / 3.0)), (int) Math.round(heightOfBar));
			}
			else {
				g.setColor(mNegativeColor);
				double heightOfBar = ((getHeight() / 2.0) - 1.0) * activity;
				g.fillRect((int) (-getWidth() / 2.0) + (int)(getWidth() / 3.0) + 1, (int) Math.round(heightOfBar), Math.max(1, (int) ((getWidth() - 2) / 3.0)), (int) Math.round(heightOfBar) * -1);
			}
			
			double activityChange = Math.max(-1.0, Math.min(1.0, ((mCurrentActivity - mLastActivity) * 100.0)));
			if(activityChange > 0.0) {
				g.setColor(mPositiveColor);
				double heightOfBar = ((getHeight() / 2.0) - 1.0) * activityChange / 2.0;
				g.fillRect((int) (-getWidth() / 2.0) + (int)(getWidth() / 3.0 * 2.0) + 1, 0, Math.max(1, (int) ((getWidth() - 2) / 3.0)), (int) Math.round(heightOfBar));
			}
			else {
				g.setColor(mNegativeColor);
				double heightOfBar = ((getHeight() / 2.0) - 1.0) * activityChange / 2.0;
				g.fillRect((int) (-getWidth() / 2.0) + (int)(getWidth() / 3.0 * 2.0) + 1, (int) Math.round(heightOfBar), Math.max(1, (int) ((getWidth() - 2) / 3.0)), (int) Math.round(heightOfBar) * -1);
			}
		}
		g.setColor(c);
		g.setComposite(composite);
		
	}

	public void reset() {
		super.reset();
		mLastActivity = 0.0;
		mCurrentActivity = 0.0;
		
	}

	public synchronized boolean update() {
		boolean ok = super.update();
		
		if(mNeuron != null && mNeuron.getActivationValue() != null) {
			mLastActivity = mCurrentActivity;
			mCurrentActivity = mNeuron.getActivationValue().getNormalized();
		}
		
		return ok;
	}
	
	public void setPositiveColor(Color color) {
		mPositiveColor = color;
	}
	
	public void setNegativeColor(Color color) {
		mNegativeColor = color;
	}
	

}
