/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.visualization.neuron;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Vector;

public class NeuronActivtySumPlotter extends NeuronGraphPlotter {
	
	public NeuronActivtySumPlotter() {
		super("ActivitySum");
		setScale(0.2);
	}

	public void plotHistroy(Graphics2D g2d, Vector neurons, Vector colors) {
		
		Vector plotLists = new Vector();
		
		int dataMinSize = Integer.MAX_VALUE;
		
		for(Iterator n = neurons.iterator(); n.hasNext();) {
			NeuronObject obj = (NeuronObject) n.next();
			
			if(obj.getNeuron() != null) {
				LinkedList list = obj.getNeuron().getHistory();
				synchronized(list) {
					Object[] data = list.toArray();
					plotLists.add(data);
					if(dataMinSize > data.length) {
						dataMinSize = data.length;
					}
				}
			}
		}
		if(!plotLists.isEmpty()) {

			int pointId = 0;
			Point point = new Point(0, 0);
			g2d.setColor(Color.RED);
			double pointDist = ((double) getCurrentWidth()) / ((double) dataMinSize);
			
			for(int i = 0; i < dataMinSize; ++i) {
				
				double sum = 0.0;
				for(Iterator j = plotLists.iterator(); j.hasNext();) {
					sum += ((Double) ((Object[]) j.next())[i]).doubleValue();
				}
				Point nextPoint = new Point((int) (pointId * pointDist), (int) translateValue(sum));
				if(pointId != 0) {
					g2d.drawLine(point.x, point.y, nextPoint.x, nextPoint.y);
				}
				pointId++;
				point = nextPoint;				
			}	

		}
	}

}
