/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.visualization.neuron;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JPanel;

import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetListener;
import de.fhg.iais.outdoor.simba.visualization.Visualization;

public class ResetNeuralNetworkButton implements Visualization, NeuralNetListener {

	private JPanel mDisplay;
	private JButton mResetButton;
	private JCheckBox mResetButtonActivator;
	
	public ResetNeuralNetworkButton() {
		mDisplay = new JPanel(new BorderLayout());
		mDisplay.setBorder(BorderFactory.createTitledBorder("Reset"));
		
		mResetButton = new JButton("Reset");
		mResetButton.setEnabled(false);
		mResetButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				NeuralNet net = Simulator.getInstance().getNeuralNetManager().getCurrentNet();
				if(net != null) {
					synchronized (net) {
						net.reset();
					}
				}
			}
		});
		
		mResetButtonActivator = new JCheckBox();
		mResetButtonActivator.setEnabled(false);
		mResetButtonActivator.setSelected(false);
		
		mResetButtonActivator.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(mResetButtonActivator.isSelected()) {
					mResetButton.setEnabled(true);
				}
				else {
					mResetButton.setEnabled(false);
				}
			}
		});
		
		mDisplay.add(mResetButtonActivator, BorderLayout.NORTH);
		mDisplay.add(mResetButton, BorderLayout.CENTER);
		
		Simulator.getInstance().getNeuralNetManager().addNeuralNetListener(this);
		
	}
	
	public JPanel getPanel() {
		return mDisplay;
	}

	public boolean cleanUp() {
		return true;
	}

	public boolean init() {
		return true;
	}

	public void currentNeuralNetChanged(NeuralNet currentNet) {
		if(currentNet == null) {
			mResetButton.setEnabled(false);
			mResetButtonActivator.setEnabled(false);
			mResetButtonActivator.setSelected(false);
		}
		else {
			mResetButtonActivator.setEnabled(true);
			if(mResetButtonActivator.isSelected()) {
				mResetButton.setEnabled(true);
			}
		}
	}

	public void networkActivationsUpdated(NeuralNet currentNet) {
		
	}

	public void networkStructureChanged(NeuralNet currentNet) {
		
	}
	
	

}
