/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.visualization.neuron;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Iterator;
import java.util.Properties;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.filechooser.FileFilter;

import de.fhg.iais.outdoor.simba.communication.NetFileFilter;
import de.fhg.iais.outdoor.simba.core.SimObject;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetAgentInterface;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetListener;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetManager;
import de.fhg.iais.outdoor.simba.neuralnet.Neuron;
import de.fhg.iais.outdoor.simba.visualization.Visualization;
import de.fhg.iais.outdoor.simba.xml.NeuralNetParser;

public class SaveAndLoadNetworkPanel implements Visualization, NeuralNetListener {
	
	private JPanel mDisplay;
	private JButton mSaveButton;
	private JButton mLoadButton;
	private JButton mAddButton;
	private JFileChooser mFileChooser;
	private NetFileFilter mFormatSingleSimba;
	private NetFileFilter mFormatSingleSimbaSubnet;
	private NetFileFilter mFormatSingleEvosun;
	private NetFileFilter mFormatFullEvosun;
	private NetFileFilter mFormatLayout;
	private NeuronVisualizationEnvironment mNeuronVisualizationEnvironment;
	private NetworkViewer mNetworkViewer;
	private JCheckBox mAutoLayout;
	
	
	
	public SaveAndLoadNetworkPanel(NeuronVisualizationEnvironment visuEnvironment,
						NetworkViewer viewer) {
		mNeuronVisualizationEnvironment = visuEnvironment;
		mNetworkViewer = viewer;
		
		mDisplay = new JPanel(new BorderLayout());
		mDisplay.setBorder(BorderFactory.createTitledBorder("File"));
		
		mSaveButton = new JButton("Save");
		mLoadButton = new JButton("Load");
		mAddButton = new JButton("Add");
		mSaveButton.setEnabled(false);
		
		mAutoLayout = new JCheckBox("ConsiderLayout");
		mAutoLayout.setSelected(true);
		
		JPanel panel = new JPanel();
		panel.add(mLoadButton);
		panel.add(mSaveButton);
		panel.add(mAddButton);
		
		mDisplay.add(panel, BorderLayout.NORTH);
		mDisplay.add(mAutoLayout, BorderLayout.SOUTH);
			
		mSaveButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saveNetwork();
			}
		});
		
		mLoadButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				loadNetwork();
			}
		});
		
		mAddButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				addSubnet();
			}
		});
		
		mFileChooser = new JFileChooser(".");
		mFileChooser.removeChoosableFileFilter(mFileChooser.getFileFilter());
		mFileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		Simulator.getInstance().getNeuralNetManager().addNeuralNetListener(this);
		
		mFormatSingleSimba = new NetFileFilter("Simba Single Net", "smb");
		mFormatSingleSimbaSubnet = new NetFileFilter("Simba Single SUBnet", "sub");
		mFormatSingleEvosun = new NetFileFilter("Evosun Singe Net", "sxml");
		mFormatFullEvosun = new NetFileFilter("Evosun Net (p0g0i0)", "xml");
		mFormatLayout = new NetFileFilter("Layout Positions", "lyt");
		
		mFileChooser.setFileFilter(mFormatSingleEvosun);
		mFileChooser.setFileFilter(mFormatFullEvosun);
		mFileChooser.setFileFilter(mFormatSingleSimbaSubnet);
		mFileChooser.setFileFilter(mFormatLayout);
		mFileChooser.setFileFilter(mFormatSingleSimba);
		
		
	}

	public JPanel getPanel() {
		return mDisplay;
	}

	public boolean cleanUp() {
		return true;
	}

	public boolean init() {
		if(Simulator.getInstance().getNeuralNetManager().getCurrentNet() == null) {
			mSaveButton.setEnabled(false);
		}
		else {
			mSaveButton.setEnabled(true);
		}
		return true;
	}
	
	
	/**
	 * Saves a network to XML.
	 * 
	 * If ConsiderLayout checkbox is selected, then the format is saved 
	 * in a file with the same name as the network xml file automatically.
	 *
	 */
	public void saveNetwork() {
		
		//choose directory.
		Properties sysProps = Simulator.getInstance().getSystemProperties();
		String directory = sysProps.getProperty("/FileChooser/CurrentDirectory");
		if(directory != null) {
			mFileChooser.setCurrentDirectory(new File(directory));
		}
		mFileChooser.setApproveButtonText("Save");
		
		
		int returnVal = mFileChooser.showOpenDialog(null);
	    if(returnVal == JFileChooser.APPROVE_OPTION) {
	       
	    	//Memorize directory
			directory = mFileChooser.getCurrentDirectory().getPath();
			sysProps.put("/FileChooser/CurrentDirectory", directory);
			
	    	File file = mFileChooser.getSelectedFile();
	    	NetFileFilter chosenFilter = (NetFileFilter) mFileChooser.getFileFilter();
	        
	    	if(!file.getName().endsWith("." + chosenFilter.getType())) {
	    		file = new File(file.getAbsolutePath() + "." + chosenFilter.getType());
	    	}
	        
	        if(!file.isDirectory()) {
	        	//Warn if the directory already exists.
	        	if(file.exists()) {
	        		int result = JOptionPane.showConfirmDialog(null, 
	        				"The file with name " + file.getName() + " already exists.\n" +
	        						"Do you really want to overwrite this file?");
	        		if(result != JOptionPane.YES_OPTION) {
	        			return;
	        		}
	        	}
	        	//Save layout if requested.
	        	if(chosenFilter == mFormatLayout || mAutoLayout.isSelected()) {
	        		if(mNeuronVisualizationEnvironment != null) {
	        			String fileName = file.getAbsolutePath();
	        			if(!fileName.endsWith(".lyt")) {
	        				fileName = fileName.substring(0, fileName.lastIndexOf(".")) + ".lyt";
	        			}
	        			mNeuronVisualizationEnvironment.saveLayout(fileName, -1, true);
	        		}
	        	}
	        	//Save the network in the desired format if requested.
	        	if(chosenFilter != mFormatLayout) {
	        		NeuralNet net = Simulator.getInstance().getNeuralNetManager().getCurrentNet();
	        		if(net != null) {
	        			synchronized (net) {
	        				String xml = "";
	        				if(chosenFilter == mFormatFullEvosun) {
	        					xml =  NeuralNetParser.createEvosunXMLFromNet(net);
	        					xml = NeuralNetParser.wrapEvosunEvolutionXML(xml);
	        				}
	        				else if(chosenFilter == mFormatSingleEvosun) {
	        					xml =  NeuralNetParser.createEvosunXMLFromNet(net);
	        				}
	        				else if(chosenFilter == mFormatSingleSimba) {
	        					xml = NeuralNetParser.createSimbaXMLFromNet(net);
	        				}
	        				else if(chosenFilter == mFormatSingleSimbaSubnet) {
	        					try {
	        						int result = JOptionPane.showConfirmDialog(null, 
	        		        				"Only the subnetwork including all selected hidden neurons is saved!\n" +
	        		        				"Input and Output neurons and external synapses are ignored!\n" +
	        		        				"Is this really your intention? ");
	        		        		if(result != JOptionPane.YES_OPTION) {
	        		        			return;
	        		        		}
									NeuralNet clonedNet = (NeuralNet) net.clone();
									Vector objects = mNetworkViewer.getSelectedObjects();
									Vector neurons = new Vector();
									for(Iterator i = objects.iterator(); i.hasNext();) {
										SimObject obj = (SimObject) i.next();
										if(!obj.isActive()) {
											continue;
										}
										if(obj instanceof NeuronObject) {
											Neuron n = ((NeuronObject) obj).getNeuron();
											if(n.getType().equals(Neuron.HIDDEN)) {
												neurons.add(n);
											}
										}
									}
									Vector clonedNeurons = NeuralNet.cloneNeuronGroup(neurons);
									Vector clonedNetNeurons = clonedNet.getNeurons();
									for(Iterator i = clonedNetNeurons.iterator(); i.hasNext();) {
										clonedNet.removeNeuron((Neuron) i.next());
									}
									for(Iterator i = clonedNeurons.iterator(); i.hasNext();) {
										clonedNet.addNeuron((Neuron) i.next());
									}
									clonedNet.updateNetwork();
									xml = NeuralNetParser.createSimbaXMLFromNet(clonedNet);
								} catch (CloneNotSupportedException e) {
								}
	        				}
	        				if(xml != null) {
		        				try {
		        					FileWriter writer = new FileWriter(file);
									writer.write(xml);
									writer.close();
									Simulator.getInstance().reportStatus("Current network saved to file " + file.getName());
								} catch (IOException e) {
									e.printStackTrace();
								}
	        				}
	        				else {
	        					Simulator.getInstance().reportStatus("Could not generate xml code!");
	        				}
						}
	        		}
        		}
	        		
	        }
	    }
        else {
        	//Memorize directory
			directory = mFileChooser.getCurrentDirectory().getPath();
			sysProps.put("/FileChooser/CurrentDirectory", directory);
        }
	}

	
	/**
	 * Loads a neural net and/or a layout from a file.
	 * Supported formats are: Simba Single Net, Evosun Single Net. 
	 *
	 * If the considerFormat checkbox is selected, then this method tries
	 * to find a layout file with the same name than the network xml file
	 * and load this automatically.
	 */
	public void loadNetwork() {

		//choose directory.
		Properties sysProps = Simulator.getInstance().getSystemProperties();
		String directory = sysProps.getProperty("/FileChooser/CurrentDirectory");
		if(directory != null) {
			mFileChooser.setCurrentDirectory(new File(directory));
		}
		mFileChooser.setApproveButtonText("Load");

		int returnVal = mFileChooser.showOpenDialog(null);
		if(returnVal == JFileChooser.APPROVE_OPTION) {
			
			//Memorize directory
			directory = mFileChooser.getCurrentDirectory().getPath();
			sysProps.put("/FileChooser/CurrentDirectory", directory);
			
	    	FileFilter chosenFilter = mFileChooser.getFileFilter();

			File file = mFileChooser.getSelectedFile();

			if(!file.isDirectory()) {
				if(file.exists()) {
					if(chosenFilter != mFormatLayout) {
						NeuralNetAgentInterface netInterface = Simulator.getInstance().getNeuralNetManager().getNeuralNetInterface();
						if(netInterface != null) {
							NeuralNet net = null;
							//Create a network from Simba Single Net XML
							if(chosenFilter == mFormatSingleSimba) {
								net = NeuralNetParser.createNetFromSimbaXMLFile(file.getAbsolutePath(), netInterface);
							}
							//Create a network from Evosun Single Net XML
							else if(chosenFilter == mFormatSingleEvosun) {
								net = NeuralNetParser.createNetFromEvosunXMLFile(file.getAbsolutePath(), netInterface);
							}
							//Create a network from Evosun Full Evolution XML
							else if(chosenFilter == mFormatFullEvosun) {
								//not supported yet.
							}
							if(net != null) {
								NeuralNetManager netManager = Simulator.getInstance().getNeuralNetManager();
								if(netManager.getCurrentNet() != null) {
									int result = JOptionPane.showConfirmDialog(null, 
					        				"There is currently a network in the manager.\n" +
					        						"Do you want to modify it with the new loaded net?");
					        		if(result != JOptionPane.YES_OPTION) {
					        			NeuralNet oldNet = netManager.getCurrentNet();
					        			if(oldNet != null) {
					        				oldNet.clearNetwork();
					        				Vector neurons = net.getNeurons();
					        				for(Iterator n = neurons.iterator(); n.hasNext();) {
					        					Neuron neuron = (Neuron) n.next();
					        					oldNet.addNeuron(neuron);
					        				}
					        				oldNet.setDefaultTransferFunction(net.getDefaultTransferFunction());
					        				netManager.notifyCurrentNeuralNetworkChanged();
					        			}
					        		}
					        		else {
					        			netManager.setCurrentNet(net);
					        		}
								}
								else {
									netManager.setCurrentNet(net);
								}
								Simulator.getInstance().reportStatus("Current network loaded from file " + file.getName());
							}
							else {
								Simulator.getInstance().reportStatus("Could not create network from loaded file!");
							}
						}
		        	}
					//load layout if requested.
					if(chosenFilter == mFormatLayout || mAutoLayout.isSelected()) {
						String fileName = file.getAbsolutePath();
	        			if(!fileName.endsWith(".lyt")) {
	        				fileName = fileName.substring(0, fileName.lastIndexOf(".")) + ".lyt";
	        			}
	        			if(mNeuronVisualizationEnvironment != null) {
		        			mNeuronVisualizationEnvironment.loadInitialPositions(fileName, false);
		        		}
					}
					rearrangeSynapses();
				}
			}
		}
		else {
        	//Memorize directory
			directory = mFileChooser.getCurrentDirectory().getPath();
			sysProps.put("/FileChooser/CurrentDirectory", directory);
        }
	}
	
	public void addSubnet() {
		//choose directory.
		Properties sysProps = Simulator.getInstance().getSystemProperties();
		String directory = sysProps.getProperty("/FileChooser/CurrentDirectory");
		if(directory != null) {
			mFileChooser.setCurrentDirectory(new File(directory));
		}
		mFileChooser.setApproveButtonText("Add");

		int returnVal = mFileChooser.showOpenDialog(null);
		if(returnVal == JFileChooser.APPROVE_OPTION) {
			
			//Memorize directory
			directory = mFileChooser.getCurrentDirectory().getPath();
			sysProps.put("/FileChooser/CurrentDirectory", directory);
			
	    	FileFilter chosenFilter = mFileChooser.getFileFilter();

			File file = mFileChooser.getSelectedFile();

			if(!file.isDirectory()) {
				if(file.exists()) {
					if(chosenFilter != mFormatLayout) {
						NeuralNetAgentInterface netInterface = Simulator.getInstance().getNeuralNetManager().getNeuralNetInterface();
						if(netInterface != null) {
							NeuralNet net = null;
							//Create a network from Simba Single Net XML
							if(chosenFilter == mFormatSingleSimba) {
								net = NeuralNetParser.createNetFromSimbaXMLFile(file.getAbsolutePath(), netInterface);
							}
							else if(chosenFilter == mFormatSingleSimbaSubnet) {
								net = NeuralNetParser.createNetFromSimbaXMLFile(file.getAbsolutePath(), netInterface);
							}
							//Create a network from Evosun Single Net XML
							else if(chosenFilter == mFormatSingleEvosun) {
								net = NeuralNetParser.createNetFromEvosunXMLFile(file.getAbsolutePath(), netInterface);
							}
							if(net != null) {
								System.out.println("Found Net");
								NeuralNet currentNet = Simulator.getInstance().getNeuralNetManager().getCurrentNet();
								if(currentNet != null) {
									synchronized(currentNet) {
										Vector newNeurons = net.getNeurons();
										Vector clonedNeurons = new Vector();
										for(Iterator i = newNeurons.iterator(); i.hasNext();) {
											Neuron n = (Neuron) i.next();
											if(n.getType().equals(Neuron.HIDDEN)) {
												clonedNeurons.add(n);
											}
										}
										clonedNeurons = NeuralNet.cloneNeuronGroup(clonedNeurons);
										for(Iterator i = clonedNeurons.iterator(); i.hasNext();) {
											currentNet.addNeuron((Neuron) i.next());
										}
									}
								}
								Simulator.getInstance().reportStatus("Added subnet from file " + file.getName());
							}
							else {
								Simulator.getInstance().reportStatus("Could not add a subnet from loaded file!");
							}
						}
		        	}
//					//load layout if requested.
//					if(chosenFilter == mFormatLayout || mAutoLayout.isSelected()) {
//						String fileName = file.getAbsolutePath();
//	        			if(!fileName.endsWith(".lyt")) {
//	        				fileName = fileName.substring(0, fileName.lastIndexOf(".")) + ".lyt";
//	        			}
//	        			if(mNeuronVisualizationEnvironment != null) {
//		        			mNeuronVisualizationEnvironment.loadInitialPositions(fileName, false);
//		        		}
//					}
					rearrangeSynapses();
				}
			}
		}
		else {
        	//Memorize directory
			directory = mFileChooser.getCurrentDirectory().getPath();
			sysProps.put("/FileChooser/CurrentDirectory", directory);
        }
	}

	/**
	 * Enables or disables the save button. If there is no neural net available,
	 * then the save button is disabled.
	 */
	public void currentNeuralNetChanged(NeuralNet currentNet) {
		if(currentNet == null) {
			mSaveButton.setEnabled(false);
		}
		else {
			mSaveButton.setEnabled(true);
		}
	}

	public void networkActivationsUpdated(NeuralNet currentNet) {
	}

	public void networkStructureChanged(NeuralNet currentNet) {
	}

	/**
	 * Normalizes the synapse visualizations accoring to the neuron positions. 
	 */
	public void rearrangeSynapses() {
		if(mNeuronVisualizationEnvironment !=  null) {
			synchronized (mNeuronVisualizationEnvironment) {
				Vector synapses = mNeuronVisualizationEnvironment.getSynapseObjects();
				for(Iterator i = synapses.iterator(); i.hasNext();) {
					((SynapseObject) i.next()).centerPosition();
				}
			}
			Simulator.getInstance().getNeuralNetManager().notifyNetworkStructureChanged(
					Simulator.getInstance().getNeuralNetManager().getCurrentNet());
		}
	}

}
