/***************************************************************************
 *   SIMBA Neuro-Workbench                                                 *
 *                                                                         *
 *   Fraunhofer Institute IAIS                                             *
 *   (Intelligent Analysis and Information Systems)                        *
 *   Sankt Augustin, Germany                                               *
 *   INDY (Intelligent Dynamics) Group                                     *
 *                                                                         *
 *   Project homepage: simba.x-bot.org                                     *
 *                                                                         *
 *   Copyright (C) 2007 by Christian Rempis                                *
 *   Contact: Christian Rempis                                             *
 *   christian.rempis@uni-osnabrueck.de                                    *
 *                                                                         *
 *                                                                         *
 *   Acknowledgments:                                                      *
 *   The SIMBA Workbench is part of the DFG Project OUTDOOR.               *
 *   This work was partially funded (2007) by DFG SPP 1125.                *
 *                                                                         *
 *                                                                         *
 *   License Agreement:                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/


package de.fhg.iais.outdoor.simba.visualization.neuron;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import de.fhg.iais.outdoor.simba.core.SimObject;
import de.fhg.iais.outdoor.simba.core.Simulator;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNet;
import de.fhg.iais.outdoor.simba.neuralnet.NeuralNetListener;
import de.fhg.iais.outdoor.simba.neuralnet.Synapse;
import de.fhg.iais.outdoor.simba.value.BooleanValue;
import de.fhg.iais.outdoor.simba.value.DoubleValue;
import de.fhg.iais.outdoor.simba.value.SimObjectValue;
import de.fhg.iais.outdoor.simba.value.Value;
import de.fhg.iais.outdoor.simba.value.ValueChangedListener;
import de.fhg.iais.outdoor.simba.value.ValueManager;
import de.fhg.iais.outdoor.simba.visualization.OptimizePerformanceListener;
import de.fhg.iais.outdoor.simba.visualization.VisibilityListener;
import de.fhg.iais.outdoor.simba.visualization.Visualization;
import de.fhg.iais.outdoor.simba.visualization.valuepanel.ValuePanel;

public class SynapseEditorPanel implements Visualization, ValueChangedListener, 
							NeuralNetListener, VisibilityListener, OptimizePerformanceListener {
	
	
	private JPanel mDisplay;
	private JTextField mSourcePanel;
	private JTextField mTargetPanel;
	private ValuePanel mStrengthPanel;
	private SimObjectValue mSelectedObject;
	private SynapseObject mLastSelectedObject;
	private boolean mOptimizePerformance;
	private boolean mVisible;
	private Vector mProtectionPrototypes = new Vector();
	private JComboBox mProtectionMode;
	private NetworkViewer mNetworkViewer;
	private ActionListener mProtectionModeListener;
	private JCheckBox mDisableSynapseCheckBox;
	private ActionListener mDisableSynapseListener;
	private BooleanValue mEnableSynapseValue;

	public SynapseEditorPanel(NetworkViewer viewer) {
		mNetworkViewer = viewer;
		
		mDisplay = new JPanel(new BorderLayout());
		mDisplay.setBorder(BorderFactory.createTitledBorder("Selected Synapse"));
		Box box = Box.createVerticalBox();
		mDisplay.add(box, BorderLayout.CENTER);
		
		JPanel upperPanel = new JPanel();
		mStrengthPanel = new ValuePanel(null, "Strength:", 8, false, true);
		upperPanel.add(mStrengthPanel);
		
		mSourcePanel = new JTextField();
		mTargetPanel = new JTextField();
		mSourcePanel.setEditable(false);
		mSourcePanel.setColumns(12);
		mTargetPanel.setEditable(false);
		mTargetPanel.setColumns(12);
		
		mProtectionMode = new JComboBox();
		mProtectionMode.setEnabled(false);
		mProtectionModeListener = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				updateProtectionMode();
			}
		};
		mProtectionMode.addActionListener(mProtectionModeListener);
		
		mDisableSynapseCheckBox = new JCheckBox("Disable");
		mDisableSynapseCheckBox.setSelected(false);
		mDisableSynapseCheckBox.setEnabled(false);
		mDisableSynapseListener = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				updateEnableSelection();
			}
		};
		upperPanel.add(mDisableSynapseCheckBox);
		
		JPanel protectionPanel = new JPanel(new BorderLayout());
		protectionPanel.add(mProtectionMode, BorderLayout.EAST);
		protectionPanel.add(new JLabel("Protection:"), BorderLayout.CENTER);
		
		box.add(upperPanel);
		box.add(mSourcePanel);
		box.add(mTargetPanel);
		box.add(protectionPanel);
				
		Simulator.getInstance().addSystemObject(this);
		Simulator.getInstance().addOptimizePerformanceListener(this);
	}

	public JPanel getPanel() {
		return mDisplay;
	}

	public boolean cleanUp() {
		return true;
	}

	public boolean init() {
		
		ValueManager vm = Simulator.getInstance().getValueManager();
		
		mSelectedObject = vm.getSimObjectValue("/Visualization/Neurons/Control/LastSelectedObject");
		if(mSelectedObject != null) {
			mSelectedObject.addValueReplacedListener(this);
			mSelectedObject.addValueChangedListener(this);
		}
		
		mProtectionPrototypes.clear();
		new Synapse(null); //make sure that the prototypes are created.
		mProtectionPrototypes.addAll(Synapse.mProtectionModes);
		for(Iterator i = mProtectionPrototypes.iterator(); i.hasNext();) {
			mProtectionMode.addItem(i.next());
		}
		
		
		return true;
	}
	
	public void synchronizeWithSelectedObject() {
		Vector selectedObjects = mNetworkViewer.getSelectedObjects();
		if(mSelectedObject != null && mSelectedObject.get() != null) {
			if(mSelectedObject.get() instanceof SynapseObject) {
				SynapseObject so = (SynapseObject) mSelectedObject.get();
				mLastSelectedObject = so;
			}
		}
		else {
			mLastSelectedObject = null;
		}
		if(mLastSelectedObject == null || !selectedObjects.contains(mLastSelectedObject)) {
			mLastSelectedObject = null;
			for(Iterator i = selectedObjects.iterator(); i.hasNext();) {
				SimObject obj = (SimObject) i.next();
				if(obj instanceof SynapseObject) {
					mLastSelectedObject = (SynapseObject) obj;
					break;
				}
			}
		}
	
		if(Simulator.getInstance().getNeuralNetManager().getCurrentNet() == null
				|| mSelectedObject == null || mSelectedObject.get() == null
				|| mLastSelectedObject == null || mNetworkViewer == null) 
		{
			mLastSelectedObject = null;
			if(mStrengthPanel.getValue() != null) {
				mStrengthPanel.getValue().removeValueChangedListener(this);
			}
			mStrengthPanel.setValue(null);
			if(mEnableSynapseValue != null) {
				mEnableSynapseValue.removeValueChangedListener(this);
			}
			mEnableSynapseValue = null;
			mSourcePanel.setText("Source: ");
			mTargetPanel.setText("Target: ");
		}
		else if(mLastSelectedObject != null) {
			if(mLastSelectedObject.isActive()) {
				Synapse synapse = mLastSelectedObject.getSynapse();
				if(synapse == null) {
					mLastSelectedObject = null;
					return;
				}
				mStrengthPanel.setValue(synapse.getStrengthValue());
				synapse.getStrengthValue().addValueChangedListener(this);
				mSourcePanel.setText("Source: " + mLastSelectedObject.getSourceNeuron().getName());
				mTargetPanel.setText("Target: " + mLastSelectedObject.getTargetNeuron().getName());
				
				mProtectionMode.setEnabled(true);
				//Protection mode
				int index = 0;
				mProtectionMode.removeActionListener(mProtectionModeListener);
				mProtectionMode.setEnabled(true);
				for(; index < mProtectionPrototypes.size(); ++index) {
					String protection = (String) mProtectionPrototypes.elementAt(index);
					if(synapse.getProtectionMode().equals(protection)) {
						break;
					}
				}
				mProtectionMode.setSelectedIndex(index);
				mProtectionMode.addActionListener(mProtectionModeListener);
				
				mDisableSynapseCheckBox.removeActionListener(mDisableSynapseListener);
				mDisableSynapseCheckBox.setEnabled(true);
				mEnableSynapseValue = synapse.getEnabledValue();
				mEnableSynapseValue.addValueChangedListener(this);
				mDisableSynapseCheckBox.setSelected(!mEnableSynapseValue.get());
				mDisableSynapseCheckBox.addActionListener(mDisableSynapseListener);
			}
			else {
				mLastSelectedObject = null;
				mStrengthPanel.setValue(null);
				mSourcePanel.setText("Source: ");
				mTargetPanel.setText("Target: ");
				mProtectionMode.setEnabled(false);
				mDisableSynapseCheckBox.setEnabled(false);
			}
		}
		
	}
	
	
	public void updateEnableSelection() {
		if(mLastSelectedObject != null && mLastSelectedObject.isActive()) {
			Vector selectedObjects = mNetworkViewer.getSelectedObjects();
			if(selectedObjects.contains(mLastSelectedObject)) {
				boolean disabled = mDisableSynapseCheckBox.isSelected();
				for(Iterator i = selectedObjects.iterator(); i.hasNext();) {
					SimObject obj = (SimObject) i.next();
					if(!(obj instanceof SynapseObject)) {
						continue;
					}
					Synapse synapse = ((SynapseObject) obj).getSynapse();
					if(synapse == null) {
						return;
					}
					if(synapse.getEnabledValue().get() != !disabled) {
						synapse.getEnabledValue().set(!disabled);
					}
				}
			}
			else {
				boolean disabled = mDisableSynapseCheckBox.isSelected();
				Synapse synapse = mLastSelectedObject.getSynapse();
				if(synapse == null) {
					return;
				}
				if(synapse.getEnabledValue().get() != !disabled) {
					synapse.getEnabledValue().set(!disabled);
				}
			}
		}
	}
	
	public void updateProtectionMode() {
		
		
		if(mLastSelectedObject != null && mLastSelectedObject.isActive()) {
			Vector selectedObjects = mNetworkViewer.getSelectedObjects();
			if(selectedObjects.contains(mLastSelectedObject)) {
				String protection = (String) mProtectionMode.getSelectedItem();
				for(Iterator i = selectedObjects.iterator(); i.hasNext();) {
					SimObject obj = (SimObject) i.next();
					if(!(obj instanceof SynapseObject)) {
						continue;
					}
					Synapse synapse = ((SynapseObject) obj).getSynapse();
					if(synapse == null) {
						return;
					}
					synapse.setProtectionMode(protection);
				}
			}
			else {
				Synapse synapse = mLastSelectedObject.getSynapse();
				if(synapse == null) {
					return;
				}
				String protection = (String) mProtectionMode.getSelectedItem();
				if(protection != null) {
					synapse.setProtectionMode(protection);
				}
			}
		}
	}

	public void valueChanged(Value value) {
		if(value == mSelectedObject) {
			synchronizeWithSelectedObject();
		}
		if(mLastSelectedObject != null && mLastSelectedObject.isActive()) {
			if(value == mStrengthPanel.getValue()) {
				Vector selectedObjects = mNetworkViewer.getSelectedObjects();
				if(selectedObjects.contains(mLastSelectedObject)) {
					double strength = ((DoubleValue) value).get();
					for(Iterator i = selectedObjects.iterator(); i.hasNext();) {
						SimObject obj = (SimObject) i.next();
						if(obj == mLastSelectedObject || !(obj instanceof SynapseObject)) {
							continue;
						}
						Synapse synapse = ((SynapseObject) obj).getSynapse();
						if(synapse == null) {
							return;
						}
						synapse.setStrength(strength);
					}
				}
			}
			else if(value == mEnableSynapseValue){
				mDisableSynapseCheckBox.setSelected(!mEnableSynapseValue.get());
			}
			mNetworkViewer.getDisplayPanel().repaint();
		}
	}

	public void valueReplaced(Value oldValue, Value newValue) {
		mSelectedObject = (SimObjectValue) ValueManager.synchronizeValue(mSelectedObject, oldValue, newValue, this);
	}

	public void currentNeuralNetChanged(NeuralNet currentNet) {
		synchronizeWithSelectedObject();
	}

	public void networkActivationsUpdated(NeuralNet currentNet) {
	}

	public void networkStructureChanged(NeuralNet currentNet) {
		synchronizeWithSelectedObject();
	}
	
	/**
	 * Called whenever the source object gets visible or invisible. Usually this is a
	 * Frame like the FrameVisualizationArea that indicates whether or not the Frame
	 * with all its content is visible. 
	 * In the case that the Frame is not visible, all observations of all ValuePanels
	 * are temporarily disabled to increase the performance (if the frame is not seen
	 * then there is no need to request values).
	 */
	public void visibilityChanged(boolean visible, Object source) {
		mVisible = visible;
		mStrengthPanel.enableObservation(mVisible && !mOptimizePerformance);
		if(mVisible && !mOptimizePerformance) {
			Simulator.getInstance().getNeuralNetManager().addNeuralNetListener(this);
		}
		else {
			Simulator.getInstance().getNeuralNetManager().removeNeuralNetListener(this);
		}
	}
	
	
	/**
	 * Called if the system was set to optimized performance mode.
	 * In this mode this Visualization disables the observation of all ValuePanels in the
	 * PanelWrappers to avoid computations for displaying values.
	 */
	public void optimizePerformance(boolean optimize) {
		mOptimizePerformance = optimize;
		
		mStrengthPanel.enableObservation(mVisible && !mOptimizePerformance);
		
		if(mVisible && !mOptimizePerformance) {
			Simulator.getInstance().getNeuralNetManager().addNeuralNetListener(this);
		}
		else {
			Simulator.getInstance().getNeuralNetManager().removeNeuralNetListener(this);
		}
	}

}
